@@ -25,6 +25,8 @@
 
 @TOP@
 
+#undef PACKAGE
+
 /* Always defined */
 #undef D_INO_IN_DIRENT
 #undef IS_AIX
@@ -43,9 +45,12 @@
 #undef STAT_STATFS4
 #undef STAT_STATVFS
 
-/* Does your system provide the umode_t typedef? */
+/* Define umode_t if your system does not provide it */
 #undef umode_t
 
+/* Define nlink_t if your system does not provide it */
+#undef nlink_t
+
 /* Does the file command accepts the -L option */
 #undef FILE_L
 
@@ -199,6 +204,13 @@
 /* Version of ncurses */
 #undef NCURSES_970530
 
+#undef HAVE_STPCPY
+
+#undef ENABLE_NLS
+#undef HAVE_CATGETS
+#undef HAVE_GETTEXT
+#undef HAVE_LC_MESSAGES
+
 @BOTTOM@
 
 #ifdef HAVE_LIBPT
