@@ -42,9 +42,10 @@
    first take priority when a key is assigned to more than one command */
     static long *key_map;
     static long cooledit_key_map[] =
-    {OUR_BACKSPACE_KEY, CK_BackSpace, OUR_DELETE_KEY, CK_Delete, '\n', CK_Return,
+    {OUR_BACKSPACE_KEY, CK_BackSpace, OUR_DELETE_KEY, CK_Delete,
+     XCTRL ('d'), CK_Delete, '\n', CK_Enter,
      KEY_PPAGE, CK_Page_Up, KEY_NPAGE, CK_Page_Down, KEY_LEFT, CK_Left,
-     KEY_RIGHT, CK_Right, KEY_UP, CK_Up, KEY_DOWN, CK_Down, ALT ('\t'), CK_Enter, ALT ('\n'), CK_Enter,
+     KEY_RIGHT, CK_Right, KEY_UP, CK_Up, KEY_DOWN, CK_Down, ALT ('\t'), CK_Return, ALT ('\n'), CK_Return,
      KEY_HOME, CK_Home, KEY_END, CK_End, '\t', CK_Tab, XCTRL ('u'), CK_Undo, KEY_IC, CK_Toggle_Insert,
      XCTRL ('o'), CK_Load, KEY_F (3), CK_Mark, KEY_F (5), CK_Copy,
      KEY_F (6), CK_Move, KEY_F (8), CK_Remove, KEY_F (12), CK_Save_As,
@@ -54,20 +55,20 @@
      XCTRL ('p'), /*spell check */ CK_Pipe_Block (1),
      KEY_F (15), CK_Insert_File,
      XCTRL ('f'), CK_Save_Block, KEY_F (1), CK_Help,
-     ALT ('t'), CK_Sort,
+     ALT ('t'), CK_Sort, ALT ('m'), CK_Mail,
      XCTRL ('z'), CK_Word_Left, XCTRL ('x'), CK_Word_Right,
      KEY_F (4), CK_Replace, KEY_F (7), CK_Find, KEY_F (14), CK_Replace_Again,
      XCTRL ('h'), CK_BackSpace, ALT ('l'), CK_Goto, ALT ('L'), CK_Goto, XCTRL ('y'), CK_Delete_Line,
-     KEY_F (17), CK_Find_Again, XCTRL ('d'), CK_Date, 0, 0};
+     KEY_F (17), CK_Find_Again, ALT ('p'), CK_Paragraph_Format, 0};
 
     static long emacs_key_map[] =
-    {OUR_BACKSPACE_KEY, CK_BackSpace, OUR_DELETE_KEY, CK_Delete, '\n', CK_Return,
+    {OUR_BACKSPACE_KEY, CK_BackSpace, OUR_DELETE_KEY, CK_Delete, '\n', CK_Enter,
      KEY_PPAGE, CK_Page_Up, KEY_NPAGE, CK_Page_Down, KEY_LEFT, CK_Left,
-     KEY_RIGHT, CK_Right, KEY_UP, CK_Up, KEY_DOWN, CK_Down, ALT ('\t'), CK_Enter, ALT ('\n'), CK_Enter,
+     KEY_RIGHT, CK_Right, KEY_UP, CK_Up, KEY_DOWN, CK_Down, ALT ('\t'), CK_Return, ALT ('\n'), CK_Return,
      KEY_HOME, CK_Home, KEY_END, CK_End, '\t', CK_Tab, XCTRL ('u'), CK_Undo, KEY_IC, CK_Toggle_Insert,
      XCTRL ('o'), CK_Load, KEY_F (3), CK_Mark, KEY_F (5), CK_Copy,
      KEY_F (6), CK_Move, KEY_F (8), CK_Remove, KEY_F (12), CK_Save_As,
-     KEY_F (2), CK_Save,
+     KEY_F (2), CK_Save, ALT ('p'), CK_Paragraph_Format,
 
      ALT ('t'), CK_Sort,
 
@@ -93,7 +94,11 @@
 
      KEY_F (4), CK_Replace, KEY_F (7), CK_Find, KEY_F (14), CK_Replace_Again,
      XCTRL ('h'), CK_BackSpace, ALT ('l'), CK_Goto, ALT ('L'), CK_Goto,
-     KEY_F (17), CK_Find_Again, 0, 0};
+     KEY_F (17), CK_Find_Again,
+     ALT ('<'), CK_Beginning_Of_Text,
+     ALT ('>'), CK_End_Of_Text,
+     
+     0, 0};
 
     static long key_pad_map[10] =
     {XCTRL ('o'), KEY_END, KEY_DOWN, KEY_NPAGE, KEY_LEFT,
@@ -113,7 +118,7 @@
 	key_map = emacs_key_map;
 	if (x_key == XCTRL ('x')) {
 	    int ext_key;
-	    ext_key = edit_raw_key_query (" Ctrl-X ", " Emacs key: ", 0);
+	    ext_key = edit_raw_key_query (" Ctrl-X ", _(" Emacs key: "), 0);
 	    switch (ext_key) {
 	    case 's':
 		command = CK_Save;
@@ -125,7 +130,7 @@
 		command = CK_New;
 		goto fin;
 	    case 'e':
-		command = CK_Macro (edit_raw_key_query (" Execute Macro ", " Press macro hotkey: ", 1));
+		command = CK_Macro (edit_raw_key_query (_(" Execute Macro "), _(" Press macro hotkey: "), 1));
 		if (command == CK_Macro (0))
 		    command = -1;
 		goto fin;
@@ -136,7 +141,7 @@
     }
 
     if (x_key == XCTRL ('q')) {
-	char_for_insertion = edit_raw_key_query (" Insert Literal ", " Press any key: ", 0);
+	char_for_insertion = edit_raw_key_query (_(" Insert Literal "), _(" Press any key: "), 0);
 	goto fin;
     }
     if (x_key == XCTRL ('a') && edit_key_emulation != EDIT_KEY_EMULATION_EMACS) {
