@@ -24,7 +24,6 @@
 #ifdef OS2_NT
 #include <io.h>
 #include <fcntl.h>
-#include <sys/stat.h>
 #endif
 #include <ctype.h>
 #include "edit.h"
@@ -72,7 +71,7 @@ char *strcasechr (const unsigned char *s, int c)
 }
 
 
-#include "src/mad.h"
+#include "../src/mad.h"
 
 #ifndef HAVE_MEMMOVE
 /* for Christophe */
@@ -167,6 +166,10 @@ void edit_refresh_cmd (WEdit * edit)
     clr_scr();
     do_refresh();
 #else
+    {
+	int fg, bg;
+	edit_get_syntax_color (edit, -1, &fg, &bg);
+    }
     touchwin(stdscr);
 #endif
     mc_refresh();
@@ -393,6 +396,7 @@ int edit_save_as_cmd (WEdit * edit)
 {
 /* This heads the 'Save As' dialog box */
     char *exp = edit_get_save_file (edit->dir, edit->filename, _(" Save As "));
+    int different_filename = 0;
     edit_push_action (edit, KEY_PRESS + edit->start_display);
     edit->force |= REDRAW_COMPLETELY;
 
@@ -403,6 +407,7 @@ int edit_save_as_cmd (WEdit * edit)
 	} else {
 	    if (strcmp(catstrs (edit->dir, edit->filename, 0), exp)) {
 		int file;
+		different_filename = 1;
 		if ((file = open ((char *) exp, O_RDONLY)) != -1) {	/* the file exists */
 		    close (file);
 		    if (edit_query_dialog2 (_(" Warning "), 
@@ -419,6 +424,8 @@ int edit_save_as_cmd (WEdit * edit)
 #ifdef MIDNIGHT
 	        edit->delete_file = 0;
 #endif		
+		if (different_filename && !edit->explicit_syntax)
+		    edit_load_syntax (edit, 0, 0);
 		return 1;
 	    } else {
 		free (exp);
@@ -964,7 +971,7 @@ int edit_replace_prompt (WEdit * edit, char *replace_text, int xpos, int ypos)
 
 	{
 	    QuickDialog Quick_input =
-	    {66, 6, 0, 0, _(" Replace "),
+	    {66, 6, 0, 0, N_(" Replace "),
 	     "[Input Line Keys]", "quick_input", 0 /*quick_widgets */ };
 
 	    Quick_input.widgets = quick_widgets;
@@ -1041,7 +1048,7 @@ void edit_replace_dialog (WEdit * edit, char **search_text, char **replace_text,
     quick_widgets[13].text = *search_text;
     {
 	QuickDialog Quick_input =
-	{50, REPLACE_DLG_HEIGHT, -1, 0, _(" Replace "),
+	{50, REPLACE_DLG_HEIGHT, -1, 0, N_(" Replace "),
 	 "[Input Line Keys]", "quick_input", 0 /*quick_widgets */ };
 
 	Quick_input.widgets = quick_widgets;
@@ -1109,7 +1116,7 @@ void edit_search_dialog (WEdit * edit, char **search_text)
 
     {
 	QuickDialog Quick_input =
-	{50, SEARCH_DLG_HEIGHT, -1, 0, _(" Search "),
+	{50, SEARCH_DLG_HEIGHT, -1, 0, N_(" Search "),
 	 "[Input Line Keys]", "quick_input", 0 /*quick_widgets */ };
 
 	Quick_input.widgets = quick_widgets;
@@ -1184,8 +1191,8 @@ void edit_search_replace_dialog (Window parent, int x, int y, char **search_text
     yc = yh;
     CGetHintPos (0, &yh);
     CDrawSwitch ("replace.reg", win, xh, yh, replace_regexp, _(" Regular expression "), 1);
-    CSetToolHint ("replace.reg", _("See the regex man page for how to compose a regular exporession"));
-    CSetToolHint ("replace.reg.label", _("See the regex man page for how to compose a regular exporession"));
+    CSetToolHint ("replace.reg", _("See the regex man page for how to compose a regular expression"));
+    CSetToolHint ("replace.reg.label", _("See the regex man page for how to compose a regular expression"));
     yb = yh;
     CGetHintPos (0, &yh);
     CGetHintPos (&xb, 0);
@@ -2480,7 +2487,7 @@ void edit_mail_dialog (WEdit * edit)
     static char *mail_to_last = 0;
 
     QuickDialog Quick_input =
-    {50, MAIL_DLG_HEIGHT, -1, 0, _(" Mail "),
+    {50, MAIL_DLG_HEIGHT, -1, 0, N_(" Mail "),
 /* NLS ? */
      "[Input Line Keys]", "quick_input", 0};
 
