@@ -26,7 +26,7 @@
 
 #ifdef MIDNIGHT
 
-#include "src/mad.h"
+#include "../src/mad.h"
 
 extern int edit_key_emulation;
 extern WEdit *wedit;
@@ -114,166 +114,165 @@ void menu_options (void)		{ edit_options_dialog (); }
 
 static menu_entry FileMenu[] =
 {
-    {' ', "Open/load...     C-o", 'O', menu_load_cmd},
-    {' ', "New              C-n", 'N', menu_new_cmd},
+    {' ', N_("&Open/load...     C-o"), 'O', menu_load_cmd},
+    {' ', N_("&New              C-n"), 'N', menu_new_cmd},
     {' ', "", ' ', 0},
-    {' ', "Save              F2", 'S', menu_save_cmd},
-    {' ', "save As...       F12", 'A', menu_save_as_cmd},
+    {' ', N_("&Save              F2"), 'S', menu_save_cmd},
+    {' ', N_("save &As...       F12"), 'A', menu_save_as_cmd},
     {' ', "", ' ', 0},
-    {' ', "Insert file...   F15", 'I', menu_insert_file_cmd},
-    {' ', "copy to File...  C-f", 'F', menu_cut_cmd},
+    {' ', N_("&Insert file...   F15"), 'I', menu_insert_file_cmd},
+    {' ', N_("copy to &File...  C-f"), 'F', menu_cut_cmd},
     {' ', "", ' ', 0},
-    {' ', "aBout...            ", 'B', edit_about_cmd},
+    {' ', N_("a&Bout...            "), 'B', edit_about_cmd},
     {' ', "", ' ', 0},
-    {' ', "Quit             F10", 'Q', menu_quit_cmd}
-};
+    {' ', N_("&Quit             F10"), 'Q', menu_quit_cmd}
+ };
 
 static menu_entry FileMenuEmacs[] =
 {
-    {' ', "Open/load...     C-o", 'O', menu_load_cmd},
-    {' ', "New            C-x k", 'N', menu_new_cmd},
+    {' ', N_("&Open/load...     C-o"), 'O', menu_load_cmd},
+    {' ', N_("&New            C-x k"), 'N', menu_new_cmd},
     {' ', "", ' ', 0},
-    {' ', "Save              F2", 'S', menu_save_cmd},
-    {' ', "save As...       F12", 'A', menu_save_as_cmd},
+    {' ', N_("&Save              F2"), 'S', menu_save_cmd},
+    {' ', N_("save &As...       F12"), 'A', menu_save_as_cmd},
     {' ', "", ' ', 0},
-    {' ', "Insert file...   F15", 'I', menu_insert_file_cmd},
-    {' ', "copy to File...     ", 'F', menu_cut_cmd},
+    {' ', N_("&Insert file...   F15"), 'I', menu_insert_file_cmd},
+    {' ', N_("copy to &File...     "), 'F', menu_cut_cmd},
     {' ', "", ' ', 0},
-    {' ', "aBout...            ", 'B', edit_about_cmd},
+    {' ', N_("a&Bout...            "), 'B', edit_about_cmd},
     {' ', "", ' ', 0},
-    {' ', "Quit             F10", 'Q', menu_quit_cmd}
+    {' ', N_("&Quit             F10"), 'Q', menu_quit_cmd}
 };
 
 static menu_entry EditMenu[] =
 {
-    {' ', "Toggle Mark       F3", 'T', menu_mark_cmd},
+    {' ', N_("&Toggle Mark       F3"), 'T', menu_mark_cmd},
     {' ', "", ' ', 0},
-    {' ', "toggle Ins/overw Ins", 'I', menu_ins_cmd},
+    {' ', N_("toggle &Ins/overw Ins"), 'I', menu_ins_cmd},
     {' ', "", ' ', 0},
-    {' ', "Copy              F5", 'C', menu_copy_cmd},
-    {' ', "Move              F6", 'M', menu_move_cmd},
-    {' ', "Delete            F8", 'D', menu_delete_cmd},
+    {' ', N_("&Copy              F5"), 'C', menu_copy_cmd},
+    {' ', N_("&Move              F6"), 'M', menu_move_cmd},
+    {' ', N_("&Delete            F8"), 'D', menu_delete_cmd},
     {' ', "", ' ', 0},
-    {' ', "Undo             C-u", 'U', menu_undo_cmd},
+    {' ', N_("&Undo             C-u"), 'U', menu_undo_cmd},
     {' ', "", ' ', 0},
-    {' ', "Beginning     C-PgUp", 'B', menu_beginning_cmd},
-    {' ', "End           C-PgDn", 'E', menu_end_cmd}
+    {' ', N_("&Beginning     C-PgUp"), 'B', menu_beginning_cmd},
+    {' ', N_("&End           C-PgDn"), 'E', menu_end_cmd}
 };
 
 static menu_entry EditMenuEmacs[] =
 {
-    {' ', "Toggle Mark       F3", 'T', menu_mark_cmd},
+    {' ', N_("&Toggle Mark       F3"), 'T', menu_mark_cmd},
     {' ', "", ' ', 0},
-    {' ', "toggle Ins/overw Ins", 'I', menu_ins_cmd},
+    {' ', N_("toggle &Ins/overw Ins"), 'I', menu_ins_cmd},
     {' ', "", ' ', 0},
-    {' ', "Copy              F5", 'C', menu_copy_cmd},
-    {' ', "Move              F6", 'M', menu_move_cmd},
-    {' ', "Delete            F8", 'D', menu_delete_cmd},
+    {' ', N_("&Copy              F5"), 'C', menu_copy_cmd},
+    {' ', N_("&Move              F6"), 'M', menu_move_cmd},
+    {' ', N_("&Delete            F8"), 'D', menu_delete_cmd},
     {' ', "", ' ', 0},
-    {' ', "Undo             C-u", 'U', menu_undo_cmd},
+    {' ', N_("&Undo             C-u"), 'U', menu_undo_cmd},
     {' ', "", ' ', 0},
-    {' ', "Beginning     C-PgUp", 'B', menu_beginning_cmd},
-    {' ', "End           C-PgDn", 'E', menu_end_cmd}
+    {' ', N_("&Beginning     C-PgUp"), 'B', menu_beginning_cmd},
+    {' ', N_("&End           C-PgDn"), 'E', menu_end_cmd}
 };
 
 static menu_entry SearReplMenu[] =
 {
-    {' ', "Search...         F7", 'S', menu_search_cmd},
-    {' ', "search Again     F17", 'A', menu_search_again_cmd},
-    {' ', "Replace...        F4", 'R', menu_replace_cmd}
+    {' ', N_("&Search...         F7"), 'S', menu_search_cmd},
+    {' ', N_("search &Again     F17"), 'A', menu_search_again_cmd},
+    {' ', N_("&Replace...        F4"), 'R', menu_replace_cmd}
 };
 
 static menu_entry SearReplMenuEmacs[] =
 {
-    {' ', "Search...         F7", 'S', menu_search_cmd},
-    {' ', "search Again     F17", 'A', menu_search_again_cmd},
-    {' ', "Replace...        F4", 'R', menu_replace_cmd}
+    {' ', N_("&Search...         F7"), 'S', menu_search_cmd},
+    {' ', N_("search &Again     F17"), 'A', menu_search_again_cmd},
+    {' ', N_("&Replace...        F4"), 'R', menu_replace_cmd}
 };
 
 static menu_entry CmdMenu[] =
 {
-    {' ', "Goto line...            M-l", 'G', menu_goto_line},
+    {' ', N_("&Goto line...            M-l"), 'G', menu_goto_line},
     {' ', "", ' ', 0},
-    {' ', "insert Literal...       C-q", 'L', menu_lit_cmd},
+    {' ', N_("insert &Literal...       C-q"), 'L', menu_lit_cmd},
     {' ', "", ' ', 0},
-    {' ', "Refresh screen          C-l", 'R', menu_refresh_cmd},
+    {' ', N_("&Refresh screen          C-l"), 'R', menu_refresh_cmd},
     {' ', "", ' ', 0},
-    {' ', "Start record macro      C-r", 'S', menu_begin_record_cmd},
-    {' ', "Finish record macro...  C-r", 'F', menu_end_record_cmd},
-    {' ', "Execute macro...   C-a, KEY", 'E', menu_exec_macro_cmd},
-    {' ', "Delete macro...            ", 'o', menu_exec_macro_delete_cmd},
+    {' ', N_("&Start record macro      C-r"), 'S', menu_begin_record_cmd},
+    {' ', N_("&Finish record macro...  C-r"), 'F', menu_end_record_cmd},
+    {' ', N_("&Execute macro...   C-a, KEY"), 'E', menu_exec_macro_cmd},
+    {' ', N_("delete macr&O...            "), 'O', menu_exec_macro_delete_cmd},
     {' ', "", ' ', 0},
-    {' ', "insert Date/time           ", 'D', menu_date_cmd},
+    {' ', N_("insert &Date/time           "), 'D', menu_date_cmd},
     {' ', "", ' ', 0},
-    {' ', "Format paragraph        M-p", 'a', menu_format_paragraph},
-    {' ', "'ispell' Spell Check    C-p", 'P', menu_ispell_cmd},
-    {' ', "Sort...                 M-t", 'O', menu_sort_cmd},
-    {' ', "'indent' C Formatter    F19", 'C', menu_c_form_cmd},
-    {' ', "Mail...                    ", 'M', menu_mail_cmd}
+    {' ', N_("format p&Aragraph        M-p"), 'A', menu_format_paragraph},
+    {' ', N_("'ispell' s&Pell check    C-p"), 'P', menu_ispell_cmd},
+    {' ', N_("sor&T...                 M-t"), 'T', menu_sort_cmd},
+    {' ', N_("'indent' &C Formatter    F19"), 'C', menu_c_form_cmd},
+    {' ', N_("&Mail...                    "), 'M', menu_mail_cmd}
 };
 
 static menu_entry CmdMenuEmacs[] =
 {
-    {' ', "Goto line...            M-l", 'G', menu_goto_line},
+    {' ', N_("&Goto line...            M-l"), 'G', menu_goto_line},
     {' ', "", ' ', 0},
-    {' ', "insert Literal...       C-q", 'L', menu_lit_cmd},
+    {' ', N_("insert &Literal...       C-q"), 'L', menu_lit_cmd},
     {' ', "", ' ', 0},
-    {' ', "Refresh screen          C-l", 'R', menu_refresh_cmd},
+    {' ', N_("&Refresh screen          C-l"), 'R', menu_refresh_cmd},
     {' ', "", ' ', 0},
-    {' ', "Start record macro      C-r", 'S', menu_begin_record_cmd},
-    {' ', "Finish record macro...  C-r", 'F', menu_end_record_cmd},
-    {' ', "Execute macro... C-x e, KEY", 'E', menu_exec_macro_cmd},
-    {' ', "Delete macro...            ", 'o', menu_exec_macro_delete_cmd},
+    {' ', N_("&Start record macro      C-r"), 'S', menu_begin_record_cmd},
+    {' ', N_("&Finish record macro...  C-r"), 'F', menu_end_record_cmd},
+    {' ', N_("&Execute macro... C-x e, KEY"), 'E', menu_exec_macro_cmd},
+    {' ', N_("delete macr&O...            "), 'o', menu_exec_macro_delete_cmd},
     {' ', "", ' ', 0},
-    {' ', "insert Date/time           ", 'D', menu_date_cmd},
+    {' ', N_("insert &Date/time           "), 'D', menu_date_cmd},
     {' ', "", ' ', 0},
-    {' ', "Format paragraph        M-p", 'a', menu_format_paragraph},
-    {' ', "'ispell' Spell Check    M-$", 'P', menu_ispell_cmd},
-    {' ', "Sort...                 M-t", 'T', menu_sort_cmd},
-    {' ', "'indent' C Formatter    F19", 'C', menu_c_form_cmd}
+    {' ', N_("format p&Aragraph        M-p"), 'a', menu_format_paragraph},
+    {' ', N_("'ispell' s&Pell check    M-$"), 'P', menu_ispell_cmd},
+    {' ', N_("sor&T...                 M-t"), 'T', menu_sort_cmd},
+    {' ', N_("'indent' &C Formatter    F19"), 'C', menu_c_form_cmd}
 };
 
 extern void menu_save_mode_cmd (void);
 
 static menu_entry OptMenu[] =
 {
-    {' ', "General...  ", 'G', menu_options},
-    {' ', "Save mode...", 'S', menu_save_mode_cmd}
+    {' ', N_("&General...  "), 'G', menu_options},
+    {' ', N_("&Save mode..."), 'S', menu_save_mode_cmd}
 #if 0
-    {' ', "Layout...", 'L', menu_layout_cmd}
+    {' ', N_("&Layout..."),    'L', menu_layout_cmd}
 #endif
 };
 
 static menu_entry OptMenuEmacs[] =
 {
-    {' ', "General...  ", 'G', menu_options},
-    {' ', "Save mode...", 'S', menu_save_mode_cmd}
+    {' ', N_("&General...  "), 'G', menu_options},
+    {' ', N_("&Save mode..."), 'S', menu_save_mode_cmd}
 #if 0
-    {' ', "Layout...", 'L', menu_layout_cmd}
+    {' ', N_("&Layout..."),    'L', menu_layout_cmd}
 #endif
 };
 
-
 #define menu_entries(x) sizeof(x)/sizeof(menu_entry)
 
 Menu EditMenuBar[N_menus];
 
 void edit_init_menu_normal (void)
 {
-    EditMenuBar[0] = create_menu (" File ", FileMenu, menu_entries (FileMenu));
-    EditMenuBar[1] = create_menu (" Edit ", EditMenu, menu_entries (EditMenu));
-    EditMenuBar[2] = create_menu (" Sear/Repl ", SearReplMenu, menu_entries (SearReplMenu));
-    EditMenuBar[3] = create_menu (" Command ", CmdMenu, menu_entries (CmdMenu));
-    EditMenuBar[4] = create_menu (" Options ", OptMenu, menu_entries (OptMenu));
+    EditMenuBar[0] = create_menu (_(" File "), FileMenu, menu_entries (FileMenu));
+    EditMenuBar[1] = create_menu (_(" Edit "), EditMenu, menu_entries (EditMenu));
+    EditMenuBar[2] = create_menu (_(" Sear/Repl "), SearReplMenu, menu_entries (SearReplMenu));
+    EditMenuBar[3] = create_menu (_(" Command "), CmdMenu, menu_entries (CmdMenu));
+    EditMenuBar[4] = create_menu (_(" Options "), OptMenu, menu_entries (OptMenu));
 }
 
 void edit_init_menu_emacs (void)
 {
-    EditMenuBar[0] = create_menu (" File ", FileMenuEmacs, menu_entries (FileMenuEmacs));
-    EditMenuBar[1] = create_menu (" Edit ", EditMenuEmacs, menu_entries (EditMenuEmacs));
-    EditMenuBar[2] = create_menu (" Sear/Repl ", SearReplMenuEmacs, menu_entries (SearReplMenuEmacs));
-    EditMenuBar[3] = create_menu (" Command ", CmdMenuEmacs, menu_entries (CmdMenuEmacs));
-    EditMenuBar[4] = create_menu (" Options ", OptMenuEmacs, menu_entries (OptMenuEmacs));
+    EditMenuBar[0] = create_menu (_(" File "), FileMenuEmacs, menu_entries (FileMenuEmacs));
+    EditMenuBar[1] = create_menu (_(" Edit "), EditMenuEmacs, menu_entries (EditMenuEmacs));
+    EditMenuBar[2] = create_menu (_(" Sear/Repl "), SearReplMenuEmacs, menu_entries (SearReplMenuEmacs));
+    EditMenuBar[3] = create_menu (_(" Command "), CmdMenuEmacs, menu_entries (CmdMenuEmacs));
+    EditMenuBar[4] = create_menu (_(" Options "), OptMenuEmacs, menu_entries (OptMenuEmacs));
 }
 
 void edit_done_menu (void)
