@@ -1,6 +1,14 @@
+/*
+ * (C) 1998 the Free Software Foundation
+ *
+ * Written by Miguel de Icaza
+ */
+#include <config.h>
 #include "util.h"
 #include <gnome.h>
 #include <gdk/gdkkeysyms.h>
+#include <string.h>
+#include "gconf.h"
 #include "myslang.h"
 #include "dlg.h"
 #undef HAVE_LIBGPM
@@ -9,7 +17,9 @@
 #include "widget.h"
 #include "wtools.h"
 #include "dialog.h"
-
+#include "color.h"
+#include "gmain.h"
+Dlg_head *last_query_dlg;
 static int sel_pos;
 
 void query_set_sel (int new_sel)
@@ -17,47 +27,90 @@ void query_set_sel (int new_sel)
 	sel_pos = new_sel;
 }
 
+static void
+pack_button (WButton *button, GtkBox *box)
+{
+	if (!button)
+		return;
+	gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (button->widget.wdata), 0, 0, 0);
+}
+
 int query_dialog (char *header, char *text, int flags, int count, ...)
 {
 	va_list ap;
+	Dlg_head  *h;
+	WLabel    *label;
 	GtkDialog *dialog;
-	GtkWidget *label, *focus_widget;
-	int i;
+	GList     *list;
+	int i, result = -1;
 	
-	va_start (ap, count);
+	if (header == MSG_ERROR)
+		header = _("Error");
+	
+	h = create_dlg (0, 0, 0, 0, dialog_colors, default_dlg_callback, "[QueryBox]", "query",
+			DLG_NO_TED | DLG_NO_TOPLEVEL);
 	dialog = GTK_DIALOG (gtk_dialog_new ());
-	label = gtk_label_new (text);
-	gtk_widget_show (label);
+	x_dlg_set_window (h, GTK_WIDGET (dialog));
+	gtk_window_set_policy (GTK_WINDOW (dialog), 0, 0, 0);
 
+	x_set_dialog_title (h, header);
+	
+	label = label_new (0, 0, text, NULL);
+	add_widget (h, label);
+	
 	if (flags & D_ERROR)
 		fprintf (stderr, "Messagebox: this should be displayed like an error\n");
 	
-	gtk_container_add (GTK_CONTAINER (dialog->vbox), label);
-	gtk_container_border_width (GTK_CONTAINER (dialog->vbox), 5);
-	
+	va_start (ap, count);
+	list = g_list_alloc ();
 	for (i = 0; i < count; i++){
-		GtkWidget *button;
+		WButton *button;
 		char *cur_name = va_arg (ap, char *);
 		
-		button = gtk_button_new_with_label (cur_name);
-		gtk_widget_show (button);
+		button = button_new (0, 0, B_USER+i, NORMAL_BUTTON, cur_name, 0, 0, NULL);
 
-		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->action_area), button);
+		add_widget (h, button);
+		list = g_list_append (list, button);
 
-		if (i == sel_pos){
-			focus_widget = button;
-		}
+		if (i == sel_pos)
+			h->initfocus = h->current;
 	}
-	if (focus_widget)
-		gtk_widget_grab_focus (focus_widget);
+	va_end (ap);
+	
+	/* Init the widgets */
+	init_dlg (h);
+	if (!(flags & D_INSERT))
+		gtk_grab_add (GTK_WIDGET (dialog));
+
+	/* Now do the GTK stuff */
+	gtk_container_add (GTK_CONTAINER (dialog->vbox), GTK_WIDGET (label->widget.wdata));
+	gtk_container_border_width (GTK_CONTAINER (dialog->vbox), 5);
+
+	g_list_foreach (list, (GFunc) pack_button, dialog->action_area);
+	g_list_free (list);
 	
 	gtk_widget_show (GTK_WIDGET (dialog));
 
-	gtk_grab_add (GTK_WIDGET (dialog));
-	gtk_main ();
+	if (flags & D_INSERT){
+		last_query_dlg = h;
+		return 0;
+	}
+	
+	frontend_run_dlg (h);
+	dlg_run_done (h);
+	last_query_dlg = h;
 	gtk_grab_remove (GTK_WIDGET (dialog));
+
+	switch (h->ret_value){
+	case B_CANCEL:
+		break;
+	default:
+		result = h->ret_value - B_USER;
+	}
+	destroy_dlg (h);
 	
 	sel_pos = 0;
+	return result;
 }
 
 /* To show nice messages to the users */
@@ -66,7 +119,7 @@ Dlg_head *message (int error, char *header, char *text, ...)
 	va_list  args;
 	char     buffer [4096];
 	Dlg_head *d;
-	
+
 	/* Setup the display information */
 	strcpy (buffer, "\n");
 	va_start (args, text);
@@ -74,7 +127,14 @@ Dlg_head *message (int error, char *header, char *text, ...)
 	strcat (buffer, "\n");
 	va_end (args);
 	
-	query_dialog (header, buffer, error, 1, "&Ok");
+	query_dialog (header, buffer, error, 1, _("&Ok"));
+	d = last_query_dlg;
+
+	if (error & D_INSERT){
+		x_flush_events ();
+		return d;
+	}
+	return d;
 }
 
 int
@@ -86,6 +146,8 @@ translate_gdk_keysym_to_curses (GdkEventKey *event)
 	case GDK_BackSpace:
 		return KEY_BACKSPACE;
 	case GDK_Tab:
+		if (event->state & GDK_SHIFT_MASK)
+			return '\t'; /* return KEY_BTAB */
 		return '\t';
 	case GDK_KP_Enter:
 	case GDK_Return:
@@ -191,7 +253,8 @@ translate_gdk_keysym_to_curses (GdkEventKey *event)
 			return key;
 		}
 	}
-	
+
+	return -1;
 }
 
 
