@@ -39,6 +39,10 @@
 #include <fcntl.h>		/* open, O_RDWR */
 #include <errno.h>
 
+#ifdef OS2_NT
+#   include <io.h>
+#endif
+
 #ifdef USE_NETCODE
 #include <netdb.h>
 #endif
@@ -75,6 +79,8 @@
 #include "color.h"
 #include "user.h"
 #include "setup.h"
+#include "x.h"
+#include "profile.h"
 
 #define MIDNIGHT
 #ifdef  USE_INTERNAL_EDIT
@@ -98,14 +104,14 @@ int output_starts_shell = 0;
 int source_route = 0;
 
 /* If set, use the builtin editor */
-int use_internal_edit = 0;
+int use_internal_edit = 1;
 
 /* Ugly hack in order to distinguish between left and right panel in menubar */
 int is_right;
 #define MENU_PANEL_IDX  (is_right ? 1 : 0)
 
 
-#ifndef HAVE_TK
+#ifndef PORT_HAS_FILTER_CHANGED
 #    define x_filter_changed(p)
 #endif
 
@@ -140,20 +146,27 @@ int view_file_at_line (char *filename, int plain_view, int internal, int start_l
     if (plain_view) {
         int changed_hex_mode = 0;
         int changed_nroff_flag = 0;
+        int changed_magic_flag = 0;
     
         altered_hex_mode = 0;
         altered_nroff_flag = 0;
+        altered_magic_flag = 0;
         if (default_hex_mode)
             changed_hex_mode = 1;
         if (default_nroff_flag)
             changed_nroff_flag = 1;
+        if (default_magic_flag)
+            changed_magic_flag = 1;
         default_hex_mode = 0;
         default_nroff_flag = 0;
+        default_magic_flag = 0;
         view (0, filename, &move_dir, start_line);
         if (changed_hex_mode && !altered_hex_mode)
             default_hex_mode = 1;
         if (changed_nroff_flag && !altered_nroff_flag)
             default_nroff_flag = 1;
+        if (changed_magic_flag && !altered_magic_flag)
+            default_magic_flag = 1;
         repaint_screen ();
         return move_dir;
     }
@@ -181,7 +194,7 @@ int view_file_at_line (char *filename, int plain_view, int internal, int start_l
 	if (!vfs_file_is_local (filename)){
 	    localcopy = mc_getlocalcopy (filename);
 	    if (localcopy == NULL){
-		message (1, " Error ", " Can not fetch a local copy of %s ", filename);
+		message (1, MSG_ERROR, _(" Can not fetch a local copy of %s "), filename);
 		return 0;
 	    }
 	    execute_internal (viewer, localcopy);
@@ -231,8 +244,8 @@ static void do_view_cmd (WPanel *panel, int normal)
     if (S_ISDIR (selection (panel)->buf.st_mode) ||
 	link_isdir (selection (panel))){
 	if (confirm_view_dir && (panel->marked || panel->dirs_marked)){
-	    if (query_dialog (" CD ", "Files tagged, want to cd?",
-			      0, 2, "&Yes", "&No") == 1){
+	    if (query_dialog (_(" CD "), _("Files tagged, want to cd?"),
+			      0, 2, _("&Yes"), _("&No")) == 1){
 		return;
 	    }
 	}
@@ -259,9 +272,9 @@ void view_cmd (WPanel *panel)
     do_view_cmd (panel, 0);
 }
 
-void view_simple_cmd (void)
+void view_simple_cmd (WPanel *panel)
 {
-    do_view_cmd (cpanel, 1);
+    do_view_cmd (panel, 1);
 }
 
 void filtered_view_cmd (WPanel *panel)
@@ -269,7 +282,7 @@ void filtered_view_cmd (WPanel *panel)
     char *command;
 
     panel = get_a_panel (panel);
-    command = input_dialog (" Filtered view ", " Filter command and arguments:",
+    command = input_dialog (_(" Filtered view "), _(" Filter command and arguments:"),
 			    selection (panel)->fname);
     if (!command)
 	return;
@@ -319,15 +332,14 @@ void edit_cmd (WPanel *panel)
 
 void edit_cmd_new (WPanel *panel)
 {
-    panel = get_a_panel(panel);
     do_edit ("");
 }
 
 void copy_cmd (void)
 {
     save_cwds_stat ();
-    if (panel_operate (OP_COPY, NULL)){
-	update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+    if (panel_operate (cpanel, OP_COPY, NULL)){
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
 }
@@ -335,8 +347,8 @@ void copy_cmd (void)
 void ren_cmd (void)
 {
     save_cwds_stat ();
-    if (panel_operate (OP_MOVE, NULL)){
-	update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+    if (panel_operate (cpanel, OP_MOVE, NULL)){
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
 }
@@ -344,8 +356,8 @@ void ren_cmd (void)
 void copymove_cmd_with_default (int copy, char *thedefault)
 {
     save_cwds_stat ();
-    if (panel_operate (copy ? OP_COPY : OP_MOVE, thedefault)){
-	update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+    if (panel_operate (cpanel, copy ? OP_COPY : OP_MOVE, thedefault)){
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
 }
@@ -355,29 +367,29 @@ void mkdir_cmd (WPanel *panel)
     char *dir;
     
     panel = get_a_panel (panel);
-    dir = input_expand_dialog (" Mkdir ", " Enter directory name:" , "");
+    dir = input_expand_dialog (_(" Mkdir "), _(" Enter directory name:") , "");
     
     if (!dir)
 	return;
 
     save_cwds_stat ();
     if (my_mkdir (dir, 0777) == 0){
-	update_panels (UP_OPTIMIZE, dir, UP_KEEPSEL);
+	update_panels (UP_OPTIMIZE, dir);
 	repaint_screen ();
 	select_item (cpanel);
 	free (dir);
 	return;
     }
     free (dir);
-    message (1, " Error ", "  %s  ", unix_error_string (errno));
+    message (1, MSG_ERROR, "  %s  ", unix_error_string (errno));
 }
 
 void delete_cmd (void)
 {
     save_cwds_stat ();
 
-    if (panel_operate (OP_DELETE, NULL)){
-	update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+    if (panel_operate (cpanel, OP_DELETE, NULL)){
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
 }
@@ -387,6 +399,21 @@ void find_cmd (void)
     do_find ();
 }
 
+void
+set_panel_filter_to (WPanel *p, char *allocated_filter_string)
+{
+    if (p->filter){
+	free (p->filter);
+	p->filter = 0;
+    }
+    if (!(allocated_filter_string [0] == '*' && allocated_filter_string [1] == 0))
+	p->filter = allocated_filter_string;
+    else
+	free (allocated_filter_string);
+    reread_cmd ();
+    x_filter_changed (p);
+}
+
 /* Set a given panel filter expression */
 void set_panel_filter (WPanel *p)
 {
@@ -395,17 +422,10 @@ void set_panel_filter (WPanel *p)
     
     x = p->filter ? p->filter : easy_patterns ? "*" : ".";
 	
-    reg_exp = input_dialog (" Filter ", " Set expression for filtering filenames", x);
+    reg_exp = input_dialog (_(" Filter "), _(" Set expression for filtering filenames"), x);
     if (!reg_exp)
 	return;
-    if (p->filter){
-	free (p->filter);
-	p->filter = 0;
-    }
-    if (!(reg_exp [0] == '*' && reg_exp [1] == 0))
-	p->filter = reg_exp;
-    reread_cmd ();
-    x_filter_changed (p);
+    set_panel_filter_to (p, reg_exp);
 }
 
 /* Invoked from the left/right menus */
@@ -431,7 +451,7 @@ void reread_cmd (void)
     else
 	flag = UP_ONLY_CURRENT;
 	
-    update_panels (UP_RELOAD|flag, UP_KEEPSEL, UP_KEEPSEL);
+    update_panels (UP_RELOAD|flag, UP_KEEPSEL);
     repaint_screen ();
 }
 
@@ -460,28 +480,33 @@ void do_re_sort (WPanel *panel)
     panel_update_contents (panel);
 }
 
-void reverse_selection_cmd (void)
+void reverse_selection_cmd_panel (WPanel *panel)
 {
     file_entry *file;
     int i;
 
-    for (i = 0; i < cpanel->count; i++){
-	file = &cpanel->dir.list [i];
+    for (i = 0; i < panel->count; i++){
+	file = &panel->dir.list [i];
 	if (S_ISDIR (file->buf.st_mode))
 	    continue;
-	do_file_mark (cpanel, i, !file->f.marked);
+	do_file_mark (panel, i, !file->f.marked);
     }
-    paint_panel (cpanel);
+    paint_panel (panel);
 }
 
-void select_cmd (void)
+void reverse_selection_cmd (void)
+{
+    reverse_selection_cmd_panel (cpanel);
+}
+
+void select_cmd_panel (WPanel *panel)
 {
     char *reg_exp, *reg_exp_t;
     int i;
     int c;
     int dirflag = 0;
 
-    reg_exp = input_dialog (" Select ", "", easy_patterns ? "*" : ".");
+    reg_exp = input_dialog (_(" Select "), "", easy_patterns ? "*" : ".");
     if (!reg_exp)
 	return;
     
@@ -497,38 +522,43 @@ void select_cmd (void)
         reg_exp_t [strlen(reg_exp_t) - 1] = 0;
     }
 
-    for (i = 0; i < cpanel->count; i++){
-        if (!strcmp( cpanel->dir.list [i].fname, ".."))
+    for (i = 0; i < panel->count; i++){
+        if (!strcmp (panel->dir.list [i].fname, ".."))
             continue;
-	if (S_ISDIR (cpanel->dir.list [i].buf.st_mode)){
+	if (S_ISDIR (panel->dir.list [i].buf.st_mode)){
 	    if (!dirflag)
                 continue;
         } else {
             if (dirflag)
                 continue;
 	}
-	c = regexp_match (reg_exp_t, cpanel->dir.list [i].fname, match_file);
+	c = regexp_match (reg_exp_t, panel->dir.list [i].fname, match_file);
 	if (c == -1){
-	    message (1, " Error ", "  Malformed regular expression  ");
+	    message (1, MSG_ERROR, _("  Malformed regular expression  "));
 	    free (reg_exp);
 	    return;
 	}
 	if (c){
-	    do_file_mark (cpanel, i, 1);
+	    do_file_mark (panel, i, 1);
 	}
     }
-    paint_panel (cpanel);
+    paint_panel (panel);
     free (reg_exp);
 }
 
-void unselect_cmd (void)
+void select_cmd (void)
+{
+	select_cmd_panel (cpanel);
+}
+
+void unselect_cmd_panel (WPanel *panel)
 {
     char *reg_exp, *reg_exp_t;
     int i;
     int c;
     int dirflag = 0;
 
-    reg_exp = input_dialog (" Unselect ","", easy_patterns ? "*" : ".");
+    reg_exp = input_dialog (_(" Unselect "),"", easy_patterns ? "*" : ".");
     if (!reg_exp)
 	return;
     
@@ -543,30 +573,35 @@ void unselect_cmd (void)
         dirflag = 1;
         reg_exp_t [strlen(reg_exp_t) - 1] = 0;
     }
-    for (i = 0; i < cpanel->count; i++){
-        if (!strcmp( cpanel->dir.list [i].fname, "..")) 
+    for (i = 0; i < panel->count; i++){
+        if (!strcmp (panel->dir.list [i].fname, "..")) 
             continue;
-	if (S_ISDIR (cpanel->dir.list [i].buf.st_mode)){
+	if (S_ISDIR (panel->dir.list [i].buf.st_mode)){
 	    if (!dirflag)
 	        continue;
         } else {
             if (dirflag)
                 continue;
         }
-	c = regexp_match (reg_exp_t, cpanel->dir.list [i].fname, match_file);
+	c = regexp_match (reg_exp_t, panel->dir.list [i].fname, match_file);
 	if (c == -1){
-	    message (1, " Error ", "  Malformed regular expression  ");
+	    message (1, MSG_ERROR, _("  Malformed regular expression  "));
 	    free (reg_exp);
 	    return;
 	}
 	if (c){
-	    do_file_mark (cpanel, i, 0);
+	    do_file_mark (panel, i, 0);
 	}
     }
-    paint_panel (cpanel);
+    paint_panel (panel);
     free (reg_exp);
 }
 
+void unselect_cmd (void)
+{
+	unselect_cmd_panel (cpanel);
+}
+
 /* Check if the file exists */
 /* If not copy the default */
 static int check_for_default(char *default_file, char *file)
@@ -592,9 +627,9 @@ void ext_cmd (void)
 
     dir = 0;
     if (geteuid () == 0){
-	dir = query_dialog ("Extension file edit",
-			    " Which extension file you want to edit? ", 0, 2,
-			    "&User", "&System Wide");
+	dir = query_dialog (_("Extension file edit"),
+			    _(" Which extension file you want to edit? "), 0, 2,
+			    _("&User"), _("&System Wide"));
     }
     extdir = concat_dir_and_file (mc_home, MC_LIB_EXT);
 
@@ -617,10 +652,10 @@ void menu_edit_cmd (void)
     int dir = 0;
     
     dir = query_dialog (
-	"Menu file edit",
-	" Which menu file will you edit? ", 
+	_("Menu file edit"),
+	_(" Which menu file will you edit? "), 
 	0, geteuid() ? 2 : 3,
-	"&Local", "&Home", "&System Wide"
+	_("&Local"), _("&Home"), _("&System Wide")
     );
 
     menufile = concat_dir_and_file(mc_home, MC_GLOBAL_MENU);
@@ -801,8 +836,8 @@ void compare_dirs_cmd (void)
 {
     enum CompareMode thorough_flag = compare_quick;
 
-    thorough_flag = query_dialog (" Compare directories ", " Select compare method: ",
-				  0, 3, "&Quick", "&Size only", "&Thorough", "&Cancel");
+    thorough_flag = query_dialog (_(" Compare directories "), _(" Select compare method: "),
+				  0, 3, _("&Quick"), _("&Size only"), _("&Thorough"), _("&Cancel"));
     if (thorough_flag < 0 || thorough_flag > 2)
 	return;
     if (get_current_type () == view_listing &&
@@ -812,7 +847,7 @@ void compare_dirs_cmd (void)
 	paint_panel (cpanel);
 	paint_panel (opanel);
     } else {
-	message (1, " Error ", " Both panels should be on the listing view mode to use this command ");
+	message (1, MSG_ERROR, _(" Both panels should be on the listing view mode to use this command "));
     }
 }
 
@@ -826,13 +861,13 @@ void history_cmd (void)
 	    input_w (cmdline)->need_push = 0;
     }
     if (!input_w (cmdline)->history){
-	message (1, " Error ", " The command history is empty ");
+	message (1, MSG_ERROR, _(" The command history is empty "));
 	return;
     }
     current = input_w (cmdline)->history;
     while (current->prev)
 	current = current->prev;
-    listbox = create_listbox_window (60, 10, " Command history ",
+    listbox = create_listbox_window (60, 10, _(" Command history "),
 				     "[Command Menu]");
     while (current){
 	LISTBOX_APPEND_TEXT (listbox, 0, current->text,
@@ -851,7 +886,7 @@ void history_cmd (void)
 	return;
     input_w (cmdline)->history = current;
     assign_text (input_w (cmdline), input_w (cmdline)->history->text);
-    update_input (input_w (cmdline));
+    update_input (input_w (cmdline), 1);
 }
 
 #if !defined(HAVE_XVIEW) && !defined(HAVE_GNOME)
@@ -874,23 +909,18 @@ view_other_cmd (void)
 
     if (!xterm_flag && !console_flag && !use_subshell){
 	if (message_flag)
-	    message (1, " Error ", " Not an xterm or Linux console; \n"
-				   " the panels cannot be toggled. ");
+	    message (1, MSG_ERROR, _(" Not an xterm or Linux console; \n"
+				     " the panels cannot be toggled. "));
 	message_flag = FALSE;
     } else {
 #ifndef HAVE_X
 	if (use_mouse_p)
 	    shut_mouse ();
-#endif	    
-
-#ifndef HAVE_X
 	if (clear_before_exec)
 	    clr_scr ();
+        if (alternate_plus_minus)
+            numeric_keypad_mode ();
 #endif
-	
-	if (alternate_plus_minus && (console_flag || xterm_flag)) {
-	    fprintf (stdout, "\033>"); fflush (stdout);
-	}
 #ifndef HAVE_SLANG
 	/* With slang we don't want any of this, since there
 	 * is no mc_raw_mode supported
@@ -916,8 +946,8 @@ view_other_cmd (void)
 	{
 	    if (output_starts_shell){
 		fprintf (stderr,
-		 "Type `exit' to return to the Midnight Commander\n\r\n\r");
-		my_system (1, shell, NULL);
+		 _("Type `exit' to return to the Midnight Commander\n\r\n\r"));
+		my_system (EXECUTE_AS_SHELL, shell, NULL);
 	    } else
 		get_key_code (0);
 	}
@@ -926,15 +956,14 @@ view_other_cmd (void)
 
 	if (!status_using_ncurses)
 	    do_enter_ca_mode ();
-	    
-	if (alternate_plus_minus && (console_flag || xterm_flag)) {
-	    fprintf (stdout, "\033="); fflush (stdout);
-	}
+
 	reset_prog_mode ();
 	keypad(stdscr, TRUE);
 #ifndef HAVE_X	
 	if (use_mouse_p)
 	    init_mouse ();
+        if (alternate_plus_minus)
+            application_keypad_mode ();
 #endif	    
 
 #ifdef HAVE_SUBSHELL_SUPPORT
@@ -958,17 +987,18 @@ view_other_cmd (void)
 }
 
 #ifndef OS2_NT
-static void do_link (int symbolic_link)
+static void
+do_link (int symbolic_link, char *fname)
 {
     struct stat s;
     char *dest, *src;
     int  stat_r;
 
     if (!symbolic_link){
-	stat_r = mc_stat (selection (cpanel)->fname, &s);
+	stat_r = mc_stat (fname, &s);
 	if (stat_r != 0){
-	    message (1, " Error ", " Couldn't stat %s \n %s ",
-		     selection (cpanel)->fname, unix_error_string (errno));
+	    message (1, MSG_ERROR, _(" Couldn't stat %s \n %s "),
+		     fname, unix_error_string (errno));
 	    return;
 	}
 	if (!S_ISREG (s.st_mode))
@@ -976,9 +1006,9 @@ static void do_link (int symbolic_link)
     }
     
     if (!symbolic_link){
-        src = copy_strings (" Link ", name_trunc (selection (cpanel)->fname, 46), 
-            " to:", NULL);
-	dest = input_expand_dialog (" Link ", src, "");
+        src = copy_strings (_(" Link "), name_trunc (fname, 46), 
+            _(" to:"), NULL);
+	dest = input_expand_dialog (_(" Link "), src, "");
 	free (src);
 	if (!dest)
 	    return;
@@ -987,11 +1017,11 @@ static void do_link (int symbolic_link)
 	    return;
 	}
 	save_cwds_stat ();
-	if (-1 == mc_link (selection (cpanel)->fname, dest))
-	    message (1, " Error ", " link: %s ", unix_error_string (errno));
+	if (-1 == mc_link (fname, dest))
+	    message (1, MSG_ERROR, _(" link: %s "), unix_error_string (errno));
     } else {
 #ifdef OLD_SYMLINK_VERSION
-        symlink_dialog (selection (cpanel)->fname, "", &dest, &src);
+        symlink_dialog (fname, "", &dest, &src);
 #else
 	/* suggest the full path for symlink */
         char s[MC_MAXPATHLEN];
@@ -1000,7 +1030,7 @@ static void do_link (int symbolic_link)
         strcpy(s, cpanel->cwd);
         if ( ! ((s[0] == '/') && (s[1] == 0)))
             strcat(s, "/");
-        strcat(s, selection (cpanel)->fname);
+        strcat(s, fname);
 	if (get_other_type () == view_listing)
 	    strcpy(d, opanel->cwd);
 	else
@@ -1021,25 +1051,25 @@ static void do_link (int symbolic_link)
 	    if (*src) {
 	        save_cwds_stat ();
 	        if (-1 == mc_symlink (dest, src))
-		    message (1, " Error ", " symlink: %s ",
+		    message (1, MSG_ERROR, _(" symlink: %s "),
 			     unix_error_string (errno));
 	    }
 	    free (src);
 	}
     }
     free (dest);
-    update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+    update_panels (UP_OPTIMIZE, UP_KEEPSEL);
     repaint_screen ();
 }
 
 void link_cmd (void)
 {
-    do_link (0);
+    do_link (0, selection (cpanel)->fname);
 }
 
 void symlink_cmd (void)
 {
-    do_link (1);
+    do_link (1, selection (cpanel)->fname);
 }
 
 void edit_symlink_cmd (void)
@@ -1047,20 +1077,20 @@ void edit_symlink_cmd (void)
     if (S_ISLNK (selection (cpanel)->buf.st_mode)) {
 	char buffer [MC_MAXPATHLEN], *p = selection (cpanel)->fname;
 	int i;
-	char *dest, *q = copy_strings (" Symlink ", name_trunc (p, 32), " points to:", NULL);
+	char *dest, *q = copy_strings (_(" Symlink "), name_trunc (p, 32), _(" points to:"), NULL);
 	
 	i = readlink (p, buffer, MC_MAXPATHLEN);
 	if (i > 0) {
 	    buffer [i] = 0;
-	    dest = input_expand_dialog (" Edit symlink ", q, buffer);
+	    dest = input_expand_dialog (_(" Edit symlink "), q, buffer);
 	    if (dest) {
 		if (*dest && strcmp (buffer, dest)) {
 		    save_cwds_stat ();
 		    mc_unlink (p);
 		    if (-1 == mc_symlink (dest, p))
-		        message (1, " Error ", " edit symlink: %s ",
+		        message (1, MSG_ERROR, _(" edit symlink: %s "),
 				 unix_error_string (errno));
-		    update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+		    update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 		    repaint_screen ();
 		}
 		free (dest);
@@ -1082,8 +1112,8 @@ void other_symlink_cmd (void)
     p = concat_dir_and_file (cpanel->cwd, selection (cpanel)->fname);
     r = concat_dir_and_file (opanel->cwd, selection (cpanel)->fname);
     
-    q = copy_strings (" Link symbolically ", name_trunc (p, 32), " to:", NULL);
-    dest = input_expand_dialog (" Relative symlink ", q, r);
+    q = copy_strings (_(" Link symbolically "), name_trunc (p, 32), _(" to:"), NULL);
+    dest = input_expand_dialog (_(" Relative symlink "), q, r);
     if (dest) {
 	if (*dest) {
 	    t = strrchr (dest, PATH_SEP);
@@ -1094,9 +1124,9 @@ void other_symlink_cmd (void)
 		if (s) {
 		    save_cwds_stat ();
 		    if (-1 == mc_symlink (dest, s))
-		        message (1, " Error ", " relative symlink: %s ",
+		        message (1, MSG_ERROR, _(" relative symlink: %s "),
 				 unix_error_string (errno));
-		    update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+		    update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 		    repaint_screen ();
 		    free (s);
 		}
@@ -1151,7 +1181,7 @@ char *get_random_hint (void)
 	return strdup ("");
     last = now;
 #else
-    static last_sec;
+    static int last_sec;
     static struct timeval tv;
     
     gettimeofday (&tv, NULL);
@@ -1197,7 +1227,7 @@ char *get_random_hint (void)
 
 #ifdef USE_NETCODE
 
-static char *machine_str = " Enter machine name (F1 for details): ";
+static char *machine_str = N_(" Enter machine name (F1 for details): ");
 
 static void nice_cd (char *text, char *xtext, char *help, char *prefix, int to_home)
 {
@@ -1218,20 +1248,23 @@ static void nice_cd (char *text, char *xtext, char *help, char *prefix, int to_h
     else 
 	cd_path = copy_strings (prefix, machine, to_home ? "/~/" : NULL, NULL);
     
-    do_panel_cd (MENU_PANEL, cd_path, 0);
+    if (do_panel_cd (MENU_PANEL, cd_path, 0))
+	directory_history_add (MENU_PANEL, (MENU_PANEL)->cwd);
+    else
+	message (1, MSG_ERROR, N_(" Could not chdir to %s "), cd_path);
     free (cd_path);
     free (machine);
 }
 
 void netlink_cmd (void)
 {
-    nice_cd (" Link to a remote machine ", machine_str,
+    nice_cd (_(" Link to a remote machine "), _(machine_str),
 	     "[Network File System]", "mc:", 1);
 }
 
 void ftplink_cmd (void)
 {
-    nice_cd (" FTP to machine ", machine_str,
+    nice_cd (_(" FTP to machine "), _(machine_str),
 	     "[FTP File System]", "ftp://", 1);
 }
 
@@ -1241,15 +1274,15 @@ void source_routing (void)
     char *source;
     struct hostent *hp;
     
-    source = input_dialog (" Socket source routing setup ",
-			   " Enter host name to use as a source routing hop: ",
+    source = input_dialog (_(" Socket source routing setup "),
+			   _(" Enter host name to use as a source routing hop: ")n,
 			   "");
     if (!source)
 	return;
 
     hp = gethostbyname (source);
     if (!hp){
-	message (1, " Host name ", " Error while looking up IP address ");
+	message (1, _(" Host name "), _(" Error while looking up IP address "));
 	return;
     }
     source_route = *((int *)hp->h_addr);
@@ -1260,9 +1293,9 @@ void source_routing (void)
 #ifdef USE_EXT2FSLIB
 void undelete_cmd (void)
 {
-    nice_cd (" Undelete files on an ext2 file system ",
-	     " Enter the file system name where you want to run the\n "
-	     " undelete file system on: (F1 for details)",
+    nice_cd (_(" Undelete files on an ext2 file system "),
+	     _(" Enter the file system name where you want to run the\n "
+	       " undelete file system on: (F1 for details)"),
 	     "[Undelete File System]", "undel:", 0);
 }
 #endif
@@ -1353,7 +1386,7 @@ void dirsizes_cmd (void)
 	free (p);
         if (pclose (f) < 0)
 #ifndef SCO_FLAVOR
-	    message (0, "Show directory sizes", "Pipe close failed");
+	    message (0, _("Show directory sizes"), _("Pipe close failed"));
 	else
 #else /* SCO_FLAVOR */
  	/* 
@@ -1366,66 +1399,36 @@ void dirsizes_cmd (void)
 	close_error_pipe (0, 0);
 	paint_panel (panel);
     } else
-        close_error_pipe (1, "Cannot invoke du command.");
+        close_error_pipe (1, _("Cannot invoke du command."));
 }
 #endif
 
 void
 save_setup_cmd (void)
 {
+    char *str;
+    
     save_setup ();
     sync_profiles ();
-    message (0, " Setup ", " Setup saved to ~/" PROFILE_NAME);
-}
-
-#ifndef HAVE_GNOME
-void
-info_cmd_no_menu (void)
-{
-    set_display_type (cpanel == left_panel ? 1 : 0, view_info);
+    str = copy_strings ( _(" Setup saved to ~/"), PROFILE_NAME, NULL);
+    
+#ifdef HAVE_GNOME
+    set_hintbar (str);
+#else
+    message (0, _(" Setup "), str);
+#endif
+    free (str);
 }
 
 void
-quick_cmd_no_menu (void)
+configure_panel_listing (WPanel *p, int view_type, int use_msformat, char *user, char *status)
 {
-    set_display_type (cpanel == left_panel ? 1 : 0, view_quick);
-}
-
-void
-switch_to_listing (int panel_index)
-{
-    if (get_display_type (panel_index) != view_listing)
-	set_display_type (panel_index, view_listing);
-}
-
-void
-listing_cmd (void)
-{
-    int   view_type, use_msformat;
-    char  *user, *status;
-    WPanel *p;
-    int   display_type, err;
-
-    display_type = get_display_type (MENU_PANEL_IDX);
-    if (display_type == view_listing)
-	p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
-    else
-	p = 0;
-
-    view_type = display_box (p, &user, &status, &use_msformat, MENU_PANEL_IDX);
-
-    if (view_type == -1)
-	return;
-
-    switch_to_listing (MENU_PANEL_IDX);
-
-    p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
-
+    int err;
+    
     p->user_mini_status = use_msformat; 
     p->list_type = view_type;
     
     if (view_type == list_user || use_msformat){
-    
 	free (p->user_format);
 	p->user_format = user;
     
@@ -1457,6 +1460,52 @@ listing_cmd (void)
     do_refresh ();
 }
 
+#ifndef HAVE_GNOME
+void
+info_cmd_no_menu (void)
+{
+    set_display_type (cpanel == left_panel ? 1 : 0, view_info);
+}
+
+void
+quick_cmd_no_menu (void)
+{
+    set_display_type (cpanel == left_panel ? 1 : 0, view_quick);
+}
+
+void
+switch_to_listing (int panel_index)
+{
+    if (get_display_type (panel_index) != view_listing)
+	set_display_type (panel_index, view_listing);
+}
+
+void
+listing_cmd (void)
+{
+    int   view_type, use_msformat;
+    char  *user, *status;
+    WPanel *p;
+    int   display_type;
+
+    display_type = get_display_type (MENU_PANEL_IDX);
+    if (display_type == view_listing)
+	p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
+    else
+	p = 0;
+
+    view_type = display_box (p, &user, &status, &use_msformat, MENU_PANEL_IDX);
+
+    if (view_type == -1)
+	return;
+
+    switch_to_listing (MENU_PANEL_IDX);
+
+    p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
+
+    configure_panel_listing (p, view_type, use_msformat, user, status);
+}
+
 void
 tree_cmd (void)
 {
