@@ -166,12 +166,6 @@ char *filename_completion_function (char *text, int state)
 	    strcat (tmp, PATH_SEP_STR);
 	    strcat (tmp, entry->d_name);
 	    canonicalize_pathname (tmp);
-#ifdef OS2_NT
-	    if (!stat (tmp, &tempstat)){
-		if (tempstat.st_mode & S_IEXEC)
-		    isexec = 1;
-	    }
-#else
 	    /* Unix version */
 	    if (!stat (tmp, &tempstat)){
 	    	uid_t my_uid = getuid ();
@@ -186,7 +180,6 @@ char *filename_completion_function (char *text, int state)
 	                isexec = 1;
 	        }
 	    }
-#endif
 	    free (tmp);
 	}
 	switch (look_for_executables)
@@ -284,7 +277,7 @@ char *username_completion_function (char *text, int state)
 }
 
 extern char **environ;
-#endif /* _OS_NT */
+#endif /* OS2_NT */
 
 /* We assume text [0] == '$' and want to have a look at text [1], if it is
    equal to '{', so that we should append '}' at the end */
@@ -854,7 +847,7 @@ static int insert_text (WInput *in, char *text, int len)
 	}
 	strncpy (in->buffer + start, text, len - start + end);
 	in->point += len;
-	update_input (in);
+	update_input (in, 1);
 	end += len;
     }
     return len != 0;
@@ -873,16 +866,16 @@ static int query_callback (Dlg_head * h, int Par, int Msg)
 	    switch (Par) {
 		case KEY_LEFT:
 		case KEY_RIGHT:
-	    	    h->running = 0;
 	    	    h->ret_value = 0;
+		    dlg_stop (h);
 	    	    return 1;
 	    	    
 	    	case 0177:
 	    	case KEY_BACKSPACE:
 	    	case XCTRL('h'):
 	    	    if (end == min_end){
-	    	    	h->running = 0;
 	    	    	h->ret_value = 0;
+			dlg_stop (h);
 	    	    	return 1;
 	    	    } else {
 	    	    	WLEntry *e, *e1;
@@ -907,9 +900,9 @@ static int query_callback (Dlg_head * h, int Par, int Msg)
 	    	    	if (is_in_input_map (input, Par) == 2){
 	    	    	    if (end == min_end)
 	    	    	        return 1;
-	    	    	    h->running = 0;
 	    	    	    h->ret_value = B_USER; /* This means we want to refill the
 	    	    	         	              list box and start again */
+			    dlg_stop (h);
 	    	    	    return 1;
 	    	    	} else
 	    	    	    return 0;
@@ -948,8 +941,8 @@ static int query_callback (Dlg_head * h, int Par, int Msg)
 	    	    	    insert_text (input, last_text, low);
 	    	    	    send_message (h, h->current->widget,WIDGET_DRAW,0);
 	    	    	} else if (need_redraw == 1){
-	    	    	    h->running = 0;
 	    	    	    h->ret_value = B_ENTER;
+			    dlg_stop (h);
 	    	    	}
 	    	    }
 	    	    return 1;
