@@ -26,10 +26,14 @@ signed char console_flag = 0;
 #if defined(linux) || defined(__linux__)
 
 #include "tty.h"
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#    include <unistd.h>
+#endif
 #include <fcntl.h>
 #include <sys/types.h>
-#include <sys/wait.h>
+#ifdef HAVE_SYS_WAIT_H
+#    include <sys/wait.h>
+#endif
 #include <signal.h>
 #include "util.h"
 #include "win.h"
