@@ -26,7 +26,9 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#    include <unistd.h>
+#endif
 #include <termios.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -149,10 +151,6 @@ char *detect_console (void)
     char *msg;
     int  xlen;
     
-    /* Root privileges required */
-    if (geteuid () != 0)
-	return "No privileges";
-
     /* Must be console */
     /* Handle the case for /dev/tty?? */
     if (tty_name[len-5] == 't')
