@@ -17,13 +17,15 @@
  */
 
 #include <config.h>
-/* "$Id: dlg.c,v 1.3 1995/02/21 19:05:40 miguel Exp $" */
+/* "$Id: dlg.c,v 1.18 1998/05/06 20:04:16 unammx Exp $" */
 #include <string.h>
 #include <stdio.h>
 #include <malloc.h>
+#include <ctype.h>
 #include "tty.h"
 #include <stdarg.h>
 #include "mad.h"
+#include "x.h"
 #include "util.h"
 #include "menu.h"
 #include "global.h"
@@ -36,7 +38,6 @@
 #include "dialog.h"	/* For push_refresh() and pop_refresh() */
 #include "layout.h"
 #include "main.h"	
-#include "x.h"
 
 /* This is the current frame, used to group Tk packings */
 char *the_frame = "";
@@ -50,6 +51,14 @@ Dlg_head *current_dlg = 0;
 /* A hook list for idle events */
 Hook *idle_hook = 0;
 
+#ifndef PORT_HAS_SET_IDLE
+#    define x_set_idle(d,x)
+#endif
+
+#ifndef PORT_HAS_DIALOG_STOP
+#   define x_dialog_stop(d)
+#endif
+
 static void slow_box (Dlg_head *h, int y, int x, int ys, int xs)
 {
     move (h->y+y, h->x+x);
@@ -167,6 +176,8 @@ int default_proc (Dlg_head *h, int Msg, int Par)
 	return 1;
 	
     case WIDGET_UNFOCUS:	/* We accept loose FOCUSes */
+	if (h->current)
+	    x_unfocus_widget (h->current);
 	return 1;
 	
     case WIDGET_DRAW:
@@ -231,6 +242,7 @@ Dlg_head *create_dlg (int y1, int x1, int lines, int cols,
     new_d->raw = 0;
     new_d->grided = 0;
     new_d->initfocus = NULL;
+    new_d->running = 0;
 #ifdef HAVE_X
     if (callback != midnight_callback)
         new_d->wdata = xtoolkit_create_dialog (new_d, flags);
@@ -243,6 +255,7 @@ Dlg_head *create_dlg (int y1, int x1, int lines, int cols,
 void set_idle_proc (Dlg_head *d, int state)
 {
     d->send_idle_msg = state;
+    x_set_idle (d, state);
 }
 
 /* add component to dialog buffer */
@@ -262,29 +275,93 @@ int add_widgetl (Dlg_head *where, void *what, WLay layout)
 
     widget->x += where->x;
     widget->y += where->y;
-    
-    back = where->current;
-    where->current = (Widget_Item *) malloc (sizeof (Widget_Item));
-    if (back){
-	back->prev = where->current;
-	where->current->next = back;
+
+    if (where->running){
+	    Widget_Item *point = where->current;
+
+	    where->current = (Widget_Item *) malloc (sizeof (Widget_Item));
+
+	    if (point){
+		    where->current->next = point->next;
+		    where->current->prev = point;
+		    point->next->prev = where->current;
+		    point->next = where->current;
+	    } else {
+		    where->current->next = where->current;
+		    where->first = where->current;
+		    where->current->prev = where->first;
+		    where->last = where->current;
+		    where->first->next = where->last;
+	    }
     } else {
-	where->current->next = where->current;
-	where->first = where->current;
+	    back = where->current;
+	    where->current = (Widget_Item *) malloc (sizeof (Widget_Item));
+	    if (back){
+		    back->prev = where->current;
+		    where->current->next = back;
+	    } else {
+		    where->current->next = where->current;
+		    where->first = where->current;
+	    }
+	    
+	    where->current->prev = where->first;
+	    where->last = where->current;
+	    where->first->next = where->last;
+	    
     }
-
     where->current->dlg_id = where->count;
-    where->current->prev = where->first;
-    where->last = where->current;
-    where->first->next = where->last;
-
     where->current->widget = what;
     where->current->widget->parent = where;
-
+    
     where->count++;
+
+    /* If the widget is inserted in a running dialog */
+    if (where->running){
+	send_message (where, widget, WIDGET_INIT, 0);
+	send_message (where, widget, WIDGET_DRAW, 0);
+#ifdef HAVE_GNOME
+	x_add_widget (where, where->current);
+#endif
+    }
     return (where->count - 1);
 }
 
+int remove_widget (Dlg_head *h, void *what)
+{
+    Widget_Item *first, *p;
+    
+    first = p = h->current;
+    
+    do {
+	if (p->widget == what){
+	    /* Remove links to this Widget_Item */
+	    p->prev->next = p->next;
+	    p->next->prev = p->prev;
+	    
+	    /* Make sure h->current is always valid */
+	    if (p == h->current){
+		h->current = h->current->next;
+		if (h->current == p)
+		    h->current = 0;
+	    }
+	    h->count--;
+	    free (p);
+	    return 1;
+	}
+	p = p->next;
+    } while (p != first);
+    return 0;
+}
+
+int destroy_widget (Widget *w)
+{
+    send_message (w->parent, w, WIDGET_DESTROY, 0);
+    if (w->destroy)
+	w->destroy (w);
+    free (w);
+    return 1;
+}
+
 int add_widget (Dlg_head *where, void *what)
 {
     return add_widgetl (where, what, XV_WLAY_DONTCARE);
@@ -302,6 +379,9 @@ void dlg_broadcast_msg_to (Dlg_head *h, int message, int reverse, int flags)
 {
     Widget_Item *p, *first;
 
+    if (!h->current)
+	    return;
+		    
     if (reverse)
 	first = p = h->current->prev;
     else
@@ -461,6 +541,8 @@ int send_message_to (Dlg_head *h, Widget *w, int msg, int par)
 
 void update_cursor (Dlg_head *h)
 {
+    if (!h->current)
+         return;
     if (h->current->widget->options & W_WANT_CURSOR)
 	send_message (h, h->current->widget, WIDGET_CURSOR, 0);
     else {
@@ -494,6 +576,12 @@ void dlg_refresh (void *parameter)
     dlg_redraw ((Dlg_head *) parameter);
 }
 
+void dlg_stop (Dlg_head *h)
+{
+    h->running = 0;
+    x_dialog_stop (h);	
+}
+
 static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
 {
    char *hlpfile;
@@ -536,14 +624,15 @@ static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
     case KEY_ENTER:
 	h->ret_value = B_ENTER;
 	h->running = 0;
+	x_dialog_stop (h);
 	break;
 
     case ESC_CHAR:
     case KEY_F (10):
     case XCTRL ('c'):
     case XCTRL ('g'):
-	h->running = 0;
 	h->ret_value = B_CANCEL;
+	dlg_stop (h);
 	break;
     }
 }
@@ -561,14 +650,14 @@ static int dlg_try_hotkey (Dlg_head *h, int d_key)
      */
 
     if (h->current->widget->options & W_IS_INPUT){
-	if (d_key >= 'A' && d_key <= 'z')
-	    return 0;
+		if(d_key < 255 && isalpha(d_key))
+			return 0;
     }
     
     /* If it's an alt key, send the message */
     c = d_key & ~ALT(0);
-    if (d_key & ALT(0) && c >= 'A' && c <= 'z')
-	d_key = c;
+    if (d_key & ALT(0) && c < 255 && isalpha(c))
+	d_key = tolower(c);
 
 #ifdef _OS_NT
 	/* .ado: fix problem with file_permission under Win95 */
@@ -616,8 +705,11 @@ void dlg_key_event (Dlg_head *h, int d_key)
     int handled;
     
     /* TAB used to cycle */
-    if (!h->raw && d_key == '\t')
-	dlg_one_down (h);
+    if (!h->raw && (d_key == '\t' || d_key == KEY_BTAB))
+	if (d_key == '\t')
+	    dlg_one_down (h);
+        else
+	    dlg_one_up (h);
     else {
 	
 	/* first can dlg_callback handle the key */
@@ -716,6 +808,7 @@ void init_dlg (Dlg_head *h)
     while (!dlg_focus (h))
 	h->current = h->current->next;
 
+    h->ret_value = 0;
     h->running = 1;
     x_init_dlg (h);
 }
@@ -800,6 +893,7 @@ destroy_dlg (Dlg_head *h)
     if (h->refresh_pushed)
 	pop_refresh (); 
 
+    x_destroy_dlg_start (h);
     dlg_broadcast_msg (h, WIDGET_DESTROY, 0);
     c = h->current;
     for (i = 0; i < h->count; i++){
