@@ -107,6 +107,9 @@ typedef struct Dlg_head {
     
     widget_data wdata;
     int  grided;	/* Does it use the automatic layout? */
+#ifdef HAVE_GNOME
+    int  idle_fn_tag;	/* Tag for the idle routine, -1 if none */
+#endif
 } Dlg_head;
 
 /* XView widget layout */
@@ -176,6 +179,7 @@ typedef struct Widget {
 #define  W_WANT_CURSOR       4
 #define  W_WANT_IDLE         8
 #define  W_IS_INPUT         16
+#define  W_PANEL_HIDDEN     32
 
 typedef struct Widget_Item {
     int dlg_id;
@@ -186,6 +190,7 @@ typedef struct Widget_Item {
 
 /* draw box in window */
 void draw_box (Dlg_head *h, int y, int x, int ys, int xs);
+
 /* doubled line if possible */
 void draw_double_box (Dlg_head *h, int y, int x, int ys, int xs);
 
@@ -196,32 +201,37 @@ Dlg_head *create_dlg (int y1, int x1, int lines, int cols,
 		      char *help_ctx, char *name, int flags);
 
 /* The flags: */
-#define DLG_NO_TED      8       /* Gnome only: use a regular window, without ted */
-#define DLG_GRID        4       /* Widgets should be created under .widgets */
-#define DLG_TRYUP       2	/* Try to move two lines up the dialog */
-#define DLG_CENTER      1	/* Center the dialog */
-#define DLG_NONE        0	/* No options */
-int  add_widget (Dlg_head *dest, void *Widget);
-int  add_widgetl (Dlg_head *dest, void *Widget, WLay layout);
+#define DLG_NO_TOPLEVEL 32	/* GNOME only: Do not create a toplevel window, user provides it */
+#define DLG_GNOME_APP   16      /* GNOME only: use a gnome-app for the toplevel window */
+#define DLG_NO_TED       8      /* GNOME only: do not manage layout with a GNOME GtkTed widget */
+#define DLG_GRID         4      /* Widgets should be created under .widgets */
+#define DLG_TRYUP        2	/* Try to move two lines up the dialog */
+#define DLG_CENTER       1	/* Center the dialog */
+#define DLG_NONE         0	/* No options */
+int  add_widget           (Dlg_head *dest, void *Widget);
+int  add_widgetl          (Dlg_head *dest, void *Widget, WLay layout);
+int  remove_widget        (Dlg_head *dest, void *Widget);
+int  destroy_widget       (Widget *w);
 
-/* Runs dialog d */
-void run_dlg (Dlg_head *d);
-
-void dlg_run_done (Dlg_head *h);
-void dlg_process_event (Dlg_head *h, int key, Gpm_Event *event);
-void init_dlg (Dlg_head *h);
+/* Runs dialog d */       
+void run_dlg              (Dlg_head *d);
+		          
+void dlg_run_done         (Dlg_head *h);
+void dlg_process_event    (Dlg_head *h, int key, Gpm_Event *event);
+void init_dlg             (Dlg_head *h);
 
 /* To activate/deactivate the idle message generation */
-void set_idle_proc (Dlg_head *d, int state);
+void set_idle_proc        (Dlg_head *d, int state);
+		          
+void dlg_redraw           (Dlg_head *h);
+void dlg_refresh          (void *parameter);
+void destroy_dlg          (Dlg_head *h);
+		          
+void widget_set_size      (Widget *widget, int x1, int y1, int x2, int y2);
 
-void dlg_redraw (Dlg_head *h);
-void dlg_refresh (void *parameter);
-void destroy_dlg (Dlg_head *h);
-void widget_set_size (Widget *widget, int x1, int y1, int x2, int y2);
 void dlg_broadcast_msg_to (Dlg_head *h, int message, int reverse, int flags);
-void dlg_broadcast_msg (Dlg_head *h, int message, int reverse);
-void dlg_mouse (Dlg_head *h, Gpm_Event *event);
-int default_dlg_callback (Dlg_head *h, int id, int msg);
+void dlg_broadcast_msg    (Dlg_head *h, int message, int reverse);
+void dlg_mouse            (Dlg_head *h, Gpm_Event *event);
 
 typedef void  (*destroy_fn)(void *);
 typedef int   (*callback_fn)(Dlg_head *, void *, int, int);
@@ -229,16 +239,11 @@ typedef int   (*callback_fn)(Dlg_head *, void *, int, int);
 void init_widget (Widget *w, int y, int x, int lines, int cols,
 		  callback_fn callback, destroy_fn destroy,
 		  mouse_h mouse_handler, char *tkname);
-int std_callback (Dlg_head *h, int Msg, int Par);
-int default_proc (Dlg_head *h, int Msg, int Par);
 
-/* Nay, it has to be elsewhere, this header if widely used
-#ifdef HAVE_X
-#  ifndef move
-#      define move(a,b)
-#  endif
-#endif
- */
+/* Various default service provision */
+int default_dlg_callback  (Dlg_head *h, int id, int msg);
+int std_callback          (Dlg_head *h, int Msg, int Par);
+int default_proc          (Dlg_head *h, int Msg, int Par);
 
 #define real_widget_move(w, _y, _x) move((w)->y + _y, (w)->x + _x)
 #define dlg_move(h, _y, _x) move(((Dlg_head *) h)->y + _y, \
@@ -250,23 +255,24 @@ int default_proc (Dlg_head *h, int Msg, int Par);
 extern Dlg_head *current_dlg;
 extern Hook *idle_hook;
 
-int send_message (Dlg_head *h, Widget *w, int msg, int par);
-int send_message_to (Dlg_head *h, Widget *w, int msg, int par);
-void dlg_replace_widget (Dlg_head *h, Widget *old, Widget *new);
-void widget_redraw (Dlg_head *h, Widget_Item *w);
-int dlg_overlap (Widget *a, Widget *b);
-void widget_erase (Widget *);
-void dlg_erase (Dlg_head *h);
+int  send_message         (Dlg_head *h, Widget *w, int msg, int par);
+int  send_message_to      (Dlg_head *h, Widget *w, int msg, int par);
+void dlg_replace_widget   (Dlg_head *h, Widget *old, Widget *new);
+void widget_redraw        (Dlg_head *h, Widget_Item *w);
+int  dlg_overlap          (Widget *a, Widget *b);
+void widget_erase         (Widget *);
+void dlg_erase            (Dlg_head *h);
+void dlg_stop             (Dlg_head *h);
 
 /* Widget selection */
-int  dlg_select_widget (struct Dlg_head *h, void *widget);
-void dlg_one_up (Dlg_head *h);
-void dlg_one_down (Dlg_head *h);
-int  dlg_focus (Dlg_head *h);
-int  dlg_unfocus (Dlg_head *h);
-int dlg_select_nth_widget (Dlg_head *h, int n);
-int dlg_item_number (Dlg_head *h);
-Widget *find_widget_type (Dlg_head *h, callback_fn signature);
+int  dlg_select_widget     (Dlg_head *h, void *widget);
+void dlg_one_up            (Dlg_head *h);
+void dlg_one_down          (Dlg_head *h);
+int  dlg_focus             (Dlg_head *h);
+int  dlg_unfocus           (Dlg_head *h);
+int  dlg_select_nth_widget (Dlg_head *h, int n);
+int  dlg_item_number       (Dlg_head *h);
+Widget *find_widget_type   (Dlg_head *h, callback_fn signature);
 
 /* Sets/clear the specified flag in the options field */
 #define widget_option(w,f,i) \
@@ -293,13 +299,19 @@ void update_cursor (Dlg_head *h);
 
 #ifdef HAVE_X
 extern Dlg_head *midnight_dlg;
-void x_focus_widget (Widget_Item *p);
-void x_init_dlg (Dlg_head *h);
-void x_destroy_dlg (Dlg_head *h);
+void x_focus_widget      (Widget_Item *p);
+void x_unfocus_widget    (Widget_Item *p);
+void x_init_dlg          (Dlg_head *h);
+void x_destroy_dlg       (Dlg_head *h);
+void x_destroy_dlg_start (Dlg_head *h);
+void x_set_idle          (Dlg_head *h, int enable_idle);
+void x_dialog_stop       (Dlg_head *h);
 #else
 #    define x_focus_widget(x) {}
+#    define x_unfocus_widget(x) {}
 #    define x_init_dlg(x)     {}
 #    define x_destroy_dlg(x)  {}
+#    define x_destroy_dlg_start(x) {}
 #endif
 
 #endif /* MC_DLG_H */
