@@ -45,6 +45,7 @@
 #include "view.h"
 #include "main.h"
 #include "../vfs/vfs.h"
+#include "x.h"
 
 #include "cons.saver.h"
 #include "layout.h"
@@ -52,7 +53,7 @@
 #include <sys/wait.h>
 #endif /* SCO_FLAVOR */
 
-/* "$Id: ext.c,v 1.10 1995/02/21 19:05:45 miguel Exp $" */
+/* "$Id: ext.c,v 1.7 1998/05/19 18:37:07 unammx Exp $" */
 
 /* If set, we execute the file command to check the file type */
 int use_file_to_check_type = 1;
@@ -73,26 +74,55 @@ flush_extension_file (void)
    
 }
 
+typedef char *(*quote_func_t)(const char *name, int i);
+
+static char *
+quote_block (quote_func_t quote_func, char **quoting_block)
+{
+	char **p = quoting_block;
+	char *result = 0;
+	char *tail   = 0;
+	int current_len = 0;
+
+	for (p = quoting_block; *p; p++){
+		int  temp_len;
+		char *temp = quote_func (*p, 0);
+
+		temp_len = strlen (temp);
+		current_len += temp_len + 2;
+		result = realloc (result, current_len);
+		if (!tail)
+			tail = result;
+		strcpy (tail, temp);
+		strcat (tail, " ");
+		tail += temp_len + 1;
+		free (temp);
+	}
+	
+	return result;
+}
+	     
 static void
-exec_extension (char *filename, char *data, char *drops, int *move_dir, int start_line)
+exec_extension (char *filename, char *data, char **drops, int *move_dir, int start_line)
 {
     char *file_name;
+    int  cmd_file_fd;
     FILE *cmd_file;
     int  expand_prefix_found = 0;
-    int parameter_found = 0;
+    int  parameter_found = 0;
     char prompt [80];
-    int run_view = 0;
-    int def_hex_mode = default_hex_mode, changed_hex_mode = 0;
-    int def_nroff_flag = default_nroff_flag, changed_nroff_flag = 0;
-    int written_nonspace = 0;
-    int is_cd = 0;
+    int  run_view = 0;
+    int  def_hex_mode = default_hex_mode, changed_hex_mode = 0;
+    int  def_nroff_flag = default_nroff_flag, changed_nroff_flag = 0;
+    int  written_nonspace = 0;
+    int  is_cd = 0;
     char buffer [1024];
     char *p = 0;
     char *localcopy = NULL;
+    int  do_local_copy;
     time_t localmtime = 0;
     struct stat mystat;
-    int    do_local_copy;
-    char * (*quote_func)(const char *name, int i) = name_quote;
+    quote_func_t quote_func = name_quote;
 
     /* Avoid making a local copy if we are doing a cd */
     if (!vfs_file_is_local(filename))
@@ -105,18 +135,21 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
      */
     file_name = strdup (tmpnam (NULL));
 
-    if ((cmd_file = fopen (file_name, "w+")) == NULL){
-	message (1, " Error ", " Can't create temporary command file \n %s ",
+    if ((cmd_file_fd = open (file_name, O_RDWR | O_CREAT | O_TRUNC | O_EXCL, 0600)) == -1){
+	message (1, MSG_ERROR, _(" Can't create temporary command file \n %s "),
 		 unix_error_string (errno));
 	return;
     }
+    cmd_file = fdopen (cmd_file_fd, "w");
+    fprintf (cmd_file, "#!%s\n", shell);
+	     
     prompt [0] = 0;
     for (;*data && *data != '\n'; data++){
 	if (parameter_found){
 	    if (*data == '}'){
 		char *parameter;
 		parameter_found = 0;
-		parameter = input_dialog (" Parameter ", prompt, "");
+		parameter = input_dialog (_(" Parameter "), prompt, "");
 		if (!parameter){
 		    /* User canceled */
 		    fclose (cmd_file);
@@ -178,7 +211,7 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
 			    text = (*quote_func) (filename, 0);
 			}
 		    } else if (*data == 'q') {
-		        text = (*quote_func) (drops, 0);
+			text = quote_block (quote_func, drops);
 		    } else
 		        text = expand_format (*data, !is_cd);
 		    if (!is_cd)
@@ -242,28 +275,29 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
     	q[1] = 0;
     	do_cd (p, cd_parse_command);
     } else {
-	shell_execute (file_name, 1);
-
-    if (console_flag)
+	shell_execute (file_name, EXECUTE_INTERNAL | EXECUTE_TEMPFILE);
+	if (console_flag)
 	{
-		handle_console (CONSOLE_SAVE);
+	    handle_console (CONSOLE_SAVE);
 	    if (output_lines && keybar_visible)
-		{
-			show_console_contents (output_start_y,
+	    {
+		show_console_contents (output_start_y,
 				       LINES-keybar_visible-output_lines-1,
 				       LINES-keybar_visible-1);
-			
-		}
+		
+	    }
 	}
-
+	
 #ifdef OLD_CODE
 	if (vfs_current_is_local ())
-	    shell_execute (file_name, 1);
+	    shell_execute (file_name, EXECUTE_INTERNAL);
 	else
-	    message (1, " Warning ", " Can't execute commands on a Virtual File System directory ");
+	    message (1, _(" Warning "), _(" Can't execute commands on a Virtual File System directory "));
 #endif
     }
+#ifndef PORT_DOES_BACKGROUND_EXEC
     unlink (file_name);
+#endif
     if (localcopy) {
         mc_stat (localcopy, &mystat);
         mc_ungetlocalcopy (filename, localcopy, localmtime != mystat.st_mtime);
@@ -298,7 +332,7 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
  * if the value for %d exists, then the viewer is started up at that line number.
  */
 char *regex_command_title = NULL;
-char *regex_command (char *filename, char *action, char *drops, int *move_dir)
+char *regex_command (char *filename, char *action, char **drops, int *move_dir)
 {
     char *extension_file;
     char *p, *q, *r, c;
@@ -355,14 +389,14 @@ check_stock_mc_ext:
 	        } else {
                     char *msg;
                     char *msg2;
-                    msg = copy_strings(" ", mc_home, MC_LIB_EXT " file error");
-                    msg2 = copy_strings("Format of the ", 
+                    msg = copy_strings(" ", mc_home, MC_LIB_EXT, _(" file error"), NULL);
+                    msg2 = copy_strings(_("Format of the "), 
                                          mc_home, 
-"mc.ext file has changed\n\
+("mc.ext file has changed\n\
 with version 3.0. It seems that installation\n\
 failed. Please fetch a fresh new copy from the\n\
 Midnight Commander package or in case you don't\n\
-have any, get it from ftp://ftp.nuclecu.unam.mx.", 0);
+have any, get it from ftp://ftp.nuclecu.unam.mx."), 0);
 	            message (1, msg, msg2);
                     free (msg);
                     free (msg2);
@@ -374,12 +408,12 @@ have any, get it from ftp://ftp.nuclecu.unam.mx.", 0);
 	if (home_error) {
             char *msg;
             char *msg2;
-            msg = copy_strings(" ~/" MC_USER_EXT " file error ");
-            msg2 = copy_strings("Format of the ~/" MC_USER_EXT " file has changed\n\
+            msg = copy_strings(" ~/", MC_USER_EXT, _(" file error "), NULL);
+            msg2 = copy_strings(_("Format of the ~/"), MC_USER_EXT, _(" file has changed\n\
 with version 3.0. You may want either to\n\
-copy it from ", mc_home, "mc.ext or use that\n\
+copy it from "), mc_home, _("mc.ext or use that\n\
 file as an example of how to write it.\n\
-", mc_home,  "mc.ext will be used for this moment.", 0);
+"), mc_home,  _("mc.ext will be used for this moment."), 0);
 	    message (1, msg, msg2);
             free (msg);
             free (msg2);
@@ -388,7 +422,10 @@ file as an example of how to write it.\n\
     }
     mc_stat (filename, &mystat);
     
-    regex_command_title = NULL;
+    if (regex_command_title){
+	free (regex_command_title);
+	regex_command_title = NULL;
+    }
     old_patterns = easy_patterns;
     easy_patterns = 0; /* Real regular expressions are needed :) */
     include_target = NULL;
@@ -617,7 +654,7 @@ match_file_output:
     	                if (p < q) { 
 			    char *filename_copy = strdup (filename);
 			    
-    	                    exec_extension (filename_copy, r + 1, drops, move_dir, view_at_line_number);
+			    exec_extension (filename_copy, r + 1, drops, move_dir, view_at_line_number);
 			    free (filename_copy);
 			    
     	                    to_return = "Success";
