@@ -17,7 +17,7 @@ extern char *home_dir;
 void refresh_screen (void *);
 
 #ifndef HAVE_STRDUP
-char *strdup (char *);
+char *strdup (const char *);
 #endif
 
 /* AIX compiler doesn't understand '\e' */
