@@ -61,19 +61,11 @@
 #    include "tkmain.h"
 #endif
 
-#ifndef USE_NCURSES
-#   define ACS_MAP(x) '*'
-#else
-#   ifndef ACS_MAP
-#       define ACS_MAP(x) acs_map [x]
-#   endif
-#endif
-
 #define MAXLINKNAME 80
 #define HISTORY_SIZE 20
 #define HELP_WINDOW_WIDTH 62
 
-/* "$Id: help.c,v 1.15 1995/02/21 19:06:03 miguel Exp $" */
+/* "$Id: help.c,v 1.4 1998/03/25 05:16:41 unammx Exp $" */
 
 static char *data;		/* Pointer to the loaded data file */
 static int help_lines = 18;	/* Lines in help viewer */
@@ -104,6 +96,37 @@ static int inside_link_area = 0;
 
 static int help_callback (struct Dlg_head *h, int id, int msg);
 
+#ifdef OS2_NT
+struct {
+    int acscode;
+    int pccode;
+} acs2pc_table [] = {
+    { 'q',  0xC4 },
+    { 'x',  0xB3 },
+    { 'l',  0xDA },
+    { 'k',  0xBF },
+    { 'm',  0xC0 },
+    { 'j',  0xD9 },
+    { 'a',  0xB0 },
+    { 'u',  0xB4 },
+    { 't',  0xC3 },
+    { 'w',  0xC2 },
+    { 'v',  0xC1 },
+    { 'n',  0xC5 },
+    { 0, 0 } };
+
+static int acs2pc (int acscode)
+{
+    int i;
+
+    for (i = 0; acs2pc_table[i].acscode != 0; i++)
+	if (acscode == acs2pc_table[i].acscode) {
+	    return acs2pc_table[i].pccode;
+	}
+    return 0;
+}
+#endif
+
 /* returns the position where text was found in the start buffer */
 /* or 0 if not found */
 char *search_string (char *start, char *text)
@@ -424,11 +447,15 @@ static void show (Dlg_head *h, char *paint_start)
 		    if (c == ' ' || c == '.')
 			addch (c);
 		    else
+#ifndef OS2_NT
 #ifndef HAVE_SLANG
-			addch (ACS_MAP(c));
+			addch (acs_map [c]);
 #else
-		    SLsmg_draw_object (h->y + line + 2, h->x + col + 2, c);
-#endif		    
+			SLsmg_draw_object (h->y + line + 2, h->x + col + 2, c);
+#endif
+#else
+			addch (acs2pc (c));
+#endif /* OS2_NT */
 		} else
 		    addch (c);
 		col++;
@@ -544,7 +571,7 @@ void help_index_cmd (Dlg_head *h)
     currentpoint = startpoint = search_string (data, "[Help]") + 1;
 
     if (!(new_item = search_string (data, "[Contents]")))
-	message (1, _(" Error "), _(" Can't find node [Contents] in help file "));
+	message (1, MSG_ERROR, _(" Can't find node [Contents] in help file "));
     else
 	currentpoint = startpoint = new_item + 1;
     selected_item = NULL;
@@ -557,7 +584,7 @@ static void quit_cmd (void *x)
 {
     Dlg_head *h = (Dlg_head *) x;
     
-    h->running = 0;
+    dlg_stop (x);
 }
 
 static void prev_node_cmd (Dlg_head *h)
@@ -699,7 +726,7 @@ static int help_handle_key (struct Dlg_head *h, int c)
 	
     case ESC_CHAR:
     case XCTRL('g'):
-	h->running = 0;
+	dlg_stop (h);
 	break;
 
     default:
@@ -719,7 +746,7 @@ static int help_callback (struct Dlg_head *h, int id, int msg)
 	draw_box (h, 1, 1, help_lines+2, HELP_WINDOW_WIDTH+2);
 	attrset (COLOR_HOT_NORMAL);
 	dlg_move (h, 1, (HELP_WINDOW_WIDTH - 1) / 2);
-	addstr (" Help ");
+	addstr (_(" Help "));
 	attrset (HELP_NORMAL_COLOR);
 	show (h, currentpoint);
 	break;
@@ -742,12 +769,12 @@ void interactive_display (char *filename, char *node)
     Widget     *md;
     
     if ((data = load_file (filename)) == 0){
-	message (1, _(" Error "), _(" Can't open file %s \n %s "),
+	message (1, MSG_ERROR, _(" Can't open file %s \n %s "),
 		 filename, unix_error_string (errno));
 	return;
     }
     if (!(main = search_string (data, node))){
-	message (1, _(" Error "), _(" Can't find node %s in help file "), node);
+	message (1, MSG_ERROR, _(" Can't find node %s in help file "), node);
 	interactive_display_finish ();
 	return;
     }
