@@ -46,7 +46,6 @@
 #    include <pwd.h>
 #else
 #    include <io.h>
-#    include <direct.h>
 #endif
 #include "tty.h"
 #include "mad.h"
@@ -132,16 +131,16 @@ struct _hotlist_but {
     char *tkname;
     int   type;
 } hotlist_but[] = {
-    { B_MOVE, NORMAL_BUTTON,         1,   42, "&Move",       "move", LIST_HOTLIST},
-    { B_REMOVE, NORMAL_BUTTON,       1,   30, "&Remove",     "r",    LIST_HOTLIST},
-    { B_APPEND, NORMAL_BUTTON,       1,   15, "&Append",     "e",    LIST_MOVELIST},
-    { B_INSERT, NORMAL_BUTTON,       1,    0, "&Insert",     "g",    LIST_MOVELIST},
-    { B_NEW_ENTRY, NORMAL_BUTTON,    1,   15, "New &Entry",  "e",    LIST_HOTLIST},
-    { B_NEW_GROUP, NORMAL_BUTTON,    1,    0, "New &Group",  "g",    LIST_HOTLIST},
-    { B_CANCEL, NORMAL_BUTTON,       0,   53, "&Cancel",     "cc",   LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
-    { B_UP_GROUP, NORMAL_BUTTON,     0,   42, "&Up",         "up",   LIST_HOTLIST|LIST_MOVELIST},
-    { B_ADD_CURRENT, NORMAL_BUTTON,  0,   20, "&Add current","ad",   LIST_HOTLIST},
-    { B_ENTER, DEFPUSH_BUTTON,       0,    0, "Change &To",  "ct",   LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
+    { B_MOVE, NORMAL_BUTTON,         1,   42, N_("&Move"),       "move", LIST_HOTLIST},
+    { B_REMOVE, NORMAL_BUTTON,       1,   30, N_("&Remove"),     "r",    LIST_HOTLIST},
+    { B_APPEND, NORMAL_BUTTON,       1,   15, N_("&Append"),     "e",    LIST_MOVELIST},
+    { B_INSERT, NORMAL_BUTTON,       1,    0, N_("&Insert"),     "g",    LIST_MOVELIST},
+    { B_NEW_ENTRY, NORMAL_BUTTON,    1,   15, N_("New &Entry"),  "e",    LIST_HOTLIST},
+    { B_NEW_GROUP, NORMAL_BUTTON,    1,    0, N_("New &Group"),  "g",    LIST_HOTLIST},
+    { B_CANCEL, NORMAL_BUTTON,       0,   53, N_("&Cancel"),     "cc",   LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
+    { B_UP_GROUP, NORMAL_BUTTON,     0,   42, N_("&Up"),         "up",   LIST_HOTLIST|LIST_MOVELIST},
+    { B_ADD_CURRENT, NORMAL_BUTTON,  0,   20, N_("&Add current"),"ad",   LIST_HOTLIST},
+    { B_ENTER, DEFPUSH_BUTTON,       0,    0, N_("Change &To"),  "ct",   LIST_HOTLIST|LIST_VFSLIST|LIST_MOVELIST},
 };
 
 /* Directory hotlist */
@@ -179,9 +178,11 @@ static void hotlist_refresh (Dlg_head *dlg)
 {
     dialog_repaint (dlg, COLOR_NORMAL, COLOR_HOT_NORMAL);
     attrset (COLOR_NORMAL);
-    draw_box (dlg, UY, UX, UY+(LINES-14), COLS-2*UX-6);
+    draw_box (dlg, 2, 5, 
+		dlg->lines - (hotlist_state.moving ? 6 : 10),
+		dlg->cols - (UX*2));
     if (!hotlist_state.moving)
-	draw_box (dlg, UY+(LINES-12), UX, 3, COLS-2*UX-6);
+	draw_box (dlg, dlg->lines-8, 5, 3, dlg->cols - (UX*2));
 }
 #endif
 
@@ -199,14 +200,14 @@ static INLINE void update_path_name ()
 	    if (hlp->type == HL_TYPE_ENTRY)
 		text = hlp->directory;
 	    else
-		text = "Subgroup - press ENTER to see list";
+		text = _("Subgroup - press ENTER to see list");
 	    
 #ifndef HAVE_X
 	    p = copy_strings (" ", current_group->label, " ", (char *)0);
 	    if (!hotlist_state.moving)
-		label_set_text (pname_group, name_trunc (p, COLS-2*UX-8));
+		label_set_text (pname_group, name_trunc (p, dlg->cols - (UX*2+4)));
 	    else
-		label_set_text (movelist_group, name_trunc (p, COLS-2*UX-8));
+		label_set_text (movelist_group, name_trunc (p, dlg->cols - (UX*2+4)));
 	    free (p);
 #endif
 	} else {
@@ -216,7 +217,7 @@ static INLINE void update_path_name ()
 	text = "";
     }
     if (!hotlist_state.moving)
-	label_set_text (pname, name_trunc (text, COLS-2*UX-10));
+	label_set_text (pname, name_trunc (text, dlg->cols - (UX*2+4)));
     dlg_redraw (dlg);
 }
 
@@ -433,7 +434,7 @@ static int hotlist_callback (Dlg_head * h, int Par, int Msg)
 	case KEY_RIGHT:
 	    if (hotlist_button_callback (B_ENTER, 0)) {
 		h->ret_value = B_ENTER;
-		h->running = 0;
+		dlg_stop (h);
 	    };
 	    return 1;
 	    break;
@@ -455,7 +456,7 @@ l1:
 			    char *tmp = copy_strings( "cd ", hlp->directory, NULL);
 			    stuff (input_w (cmdline), tmp, 0);
 			    free (tmp);
-			    h->running = 0;
+			    dlg_stop (h);
 			    h->ret_value = B_CANCEL;
 			    return 1;
 		        }
@@ -493,8 +494,8 @@ static int l_call (void *l)
 	if (list->current->data) {
 	    struct hotlist *hlp = (struct hotlist*) list->current->data;
 	    if (hlp->type == HL_TYPE_ENTRY) {
-		dlg->running = 0;
 		dlg->ret_value = B_ENTER;
+		dlg_stop (dlg);
 		return listbox_finish;
 	    } else {
 		hotlist_button_callback (B_ENTER, (void *)0);
@@ -502,8 +503,8 @@ static int l_call (void *l)
 		return listbox_nothing;
 	    }
 	} else {
-	    dlg->running = 0;
 	    dlg->ret_value = B_ENTER;
+	    dlg_stop (dlg);
 	    return listbox_finish;
 	}
     }
@@ -518,22 +519,105 @@ static void add_name_to_list (char *path)
     listbox_add_item (l_hotlist, 0, 0, path, 0);
 }
 
+/*
+ * Expands all button names (once) and recalculates button positions.
+ * returns number of columns in the dialog box, which is 10 chars longer
+ * then buttonbar.
+ *
+ * If common width of the window (i.e. in xterm) is less than returned
+ * width - sorry :)  (anyway this did not handled in previous version too)
+ */
+static int
+init_i18n_stuff(int list_type, int cols)
+{
+	register int i;
+	static char* cancel_but = "&Cancel";
+
+#ifdef ENABLE_NLS
+	static int hotlist_i18n_flag = 0;
+
+	if (!hotlist_i18n_flag)
+	{
+		i = sizeof (hotlist_but) / sizeof (hotlist_but [0]);
+		while (i--)
+			hotlist_but [i].text = _(hotlist_but [i].text);
+
+		cancel_but = _(cancel_but);
+		hotlist_i18n_flag = 1;
+	}
+#endif /* ENABLE_NLS */
+
+	/* Dynamic resizing of buttonbars */
+	{
+		int len[2], count[2]; /* at most two lines of buttons */
+		int cur_x[2], row;
+
+		i = sizeof (hotlist_but) / sizeof (hotlist_but [0]);
+		len[0] = len[1] = count[0] = count[1] = 0;
+
+		/* Count len of buttonbars, assuming 2 extra space between buttons */
+		while (i--)
+		{
+			if (! (hotlist_but[i].type & list_type))
+				continue;
+
+			row = hotlist_but [i].y;
+			++count [row];
+			len [row] += strlen (hotlist_but [i].text) + 5;
+			if (hotlist_but [i].flags == DEFPUSH_BUTTON)
+				len [row] += 2;
+		}
+		len[0] -= 2;
+		len[1] -= 2;
+
+		cols = max(cols, max(len[0], len[1]));
+
+		/* arrange buttons */
+
+		cur_x[0] = cur_x[1] = 0;
+		i = sizeof (hotlist_but) / sizeof (hotlist_but [0]);
+		while (i--)
+		{
+			if (! (hotlist_but[i].type & list_type))
+				continue;
+
+			row = hotlist_but [i].y;
+
+			if (hotlist_but [i].x != 0) 
+			{
+				/* not first int the row */
+				if (!strcmp (hotlist_but [i].text, cancel_but))
+					hotlist_but [i].x = 
+						cols - strlen (hotlist_but [i].text) - 13;
+				else
+					hotlist_but [i].x = cur_x [row];
+			}
+
+			cur_x [row] += strlen (hotlist_but [i].text) + 2
+				+ (hotlist_but [i].flags == DEFPUSH_BUTTON ? 5 : 3);
+		}
+	}
+	
+	return cols;
+}
+
 static void init_hotlist (int list_type)
 {
     int i;
+	int hotlist_cols = init_i18n_stuff (list_type, COLS - 6);
 
     do_refresh ();
 
     hotlist_state.expanded = GetPrivateProfileInt ("HotlistConfig",
 			    "expanded_view_of_groups", 0, profile_name);
 
-    hotlist_dlg = create_dlg (0, 0, LINES-2, COLS-6, dialog_colors,
+    hotlist_dlg = create_dlg (0, 0, LINES-2, hotlist_cols, dialog_colors,
 			      hotlist_callback, 
 			      list_type == LIST_VFSLIST ? "[vfshot]" : "[Hotlist]",
 			      list_type == LIST_VFSLIST ? "vfshot" : "hotlist",
 			      DLG_CENTER|DLG_GRID);
     x_set_dialog_title (hotlist_dlg,
-        list_type == LIST_VFSLIST ? "Active VFS directories" : "Directory hotlist");
+        list_type == LIST_VFSLIST ? _("Active VFS directories") : _("Directory hotlist"));
 
 #define XTRACT(i) BY+hotlist_but[i].y, BX+hotlist_but[i].x, hotlist_but[i].ret_cmd, hotlist_but[i].flags, hotlist_but[i].text, hotlist_button_callback, 0, hotlist_but[i].tkname
 
@@ -552,10 +636,10 @@ static void init_hotlist (int list_type)
     add_widget (hotlist_dlg, pname);
 #ifndef HAVE_X
     if (!hotlist_state.moving) {
-	add_widget (hotlist_dlg, label_new (UY-12+LINES, UX+1, " Directory path ", NULL));
+	add_widget (hotlist_dlg, label_new (UY-12+LINES, UX+1, _(" Directory path "), NULL));
 
 	/* This one holds the displayed pathname */
-	pname_group = label_new (UY, UX+1, " Directory label ", NULL);
+	pname_group = label_new (UY, UX+1, _(" Directory label "), NULL);
 	add_widget (hotlist_dlg, pname_group);
     }
 #endif
@@ -576,11 +660,12 @@ static void init_hotlist (int list_type)
 static void init_movelist (int list_type, struct hotlist *item)
 {
     int i;
-    char *hdr = copy_strings ("Moving ", item->label, 0);
+    char *hdr = copy_strings (_("Moving "), item->label, 0);
+	int movelist_cols = init_i18n_stuff (list_type, COLS - 6);
 
     do_refresh ();
 
-    movelist_dlg = create_dlg (0, 0, LINES-6, COLS-6, dialog_colors,
+    movelist_dlg = create_dlg (0, 0, LINES-6, movelist_cols, dialog_colors,
 			      hotlist_callback, "[Hotlist]",
 			      "movelist",
 			      DLG_CENTER|DLG_GRID);
@@ -601,11 +686,13 @@ static void init_movelist (int list_type, struct hotlist *item)
      * that one will hold the path name label
      */
 #ifndef HAVE_X
-    movelist_group = label_new (UY, UX+1, " Directory label ", NULL);
+    movelist_group = label_new (UY, UX+1, _(" Directory label "), NULL);
     add_widget (movelist_dlg, movelist_group);
 #endif
     /* get new listbox */
-    l_movelist = listbox_new (UY + 1, UX + 1, COLS-2*UX-8, LINES-14, listbox_cback, l_call, "listbox");
+    l_movelist = listbox_new (UY + 1, UX + 1, 
+		movelist_dlg->cols - 2*UX - 2, movelist_dlg->lines - 8,
+		listbox_cback, l_call, "listbox");
 
     fill_listbox ();
 
@@ -617,7 +704,7 @@ static void hotlist_done (void)
 {
     destroy_dlg (hotlist_dlg);
     if (0)
-	update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
     repaint_screen ();
 }
 
@@ -690,15 +777,41 @@ add2hotlist (char *label, char *directory, enum HotListType type, int pos)
 
 }
 
+/*
+ * Support routine for add_new_entry_input()/add_new_group_input()
+ * Change positions of buttons (first three widgets).
+ *
+ * This is just a quick hack. Accurate procedure must take care of
+ * internationalized label lengths and total buttonbar length...assume
+ * 64 is longer anyway.
+ */
+static void add_widgets_i18n(QuickWidget* qw, int len)
+{
+	int i, l[3], space, cur_x;
+
+	for (i = 0; i < 3; i++)
+	{
+		qw [i].text = _(qw [i].text);
+		l[i] = strlen (qw [i].text) + 3;
+	}
+	space = (len - 4 - l[0] - l[1] - l[2]) / 4;
+
+	for (cur_x = 2 + space, i = 3; i--; cur_x += l[i] + space)
+	{
+		qw [i].relative_x = cur_x;
+		qw [i].x_divisions = len;
+	}
+}
+
 static int add_new_entry_input (char *header, char *text1, char *text2, char *help, char **r1, char **r2)
 {
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 55, 80, 4, 0, "&Cancel", 0, B_CANCEL, 0, 0,
+    { quick_button, 55, 80, 4, 0, N_("&Cancel"), 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "button-cancel" },
-    { quick_button, 30, 80, 4, 0, "&Insert", 0, B_INSERT, 0, 0,
+    { quick_button, 30, 80, 4, 0, N_("&Insert"), 0, B_INSERT, 0, 0,
 	  XV_WLAY_DONTCARE, "button-insert" },
-    { quick_button, 10, 80, 4, 0, "&Append", 0, B_APPEND, 0, 0,
+    { quick_button, 10, 80, 4, 0, N_("&Append"), 0, B_APPEND, 0, 0,
 	  XV_WLAY_DONTCARE, "button-append" },
     { quick_input,  4, 80, 4, 0, "",58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input-pth" },
     { quick_label,  3, 80, 3, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label-pth" },
@@ -711,17 +824,32 @@ static int add_new_entry_input (char *header, char *text1, char *text2, char *he
     int lines1, lines2;
     char *my_str1, *my_str2;
     
+#ifdef ENABLE_NLS
+	static int i18n_flag = 0;
+#endif /* ENABLE_NLS */
+
     len = max (strlen (header), msglen (text1, &lines1));
     len = max (len, msglen (text2, &lines2)) + 4;
     len = max (len, 64);
 
+#ifdef ENABLE_NLS
+	if (!i18n_flag)
+	{
+		add_widgets_i18n(quick_widgets, len);
+		i18n_flag = 1;
+	}
+#endif /* ENABLE_NLS */
+
     Quick_input.xlen  = len;
     Quick_input.xpos  = -1;
     Quick_input.title = header;
     Quick_input.help  = help;
     Quick_input.class = "hotlist_new_entry";
+    Quick_input.i18n  = 0;
     quick_widgets [6].text = text1;
     quick_widgets [4].text = text2;
+    quick_widgets [5].text = *r1;
+    quick_widgets [3].text = *r2;
 
     for (i = 0; i < 7; i++)
 	quick_widgets [i].y_divisions = lines1+lines2+7;
@@ -750,7 +878,10 @@ static void add_new_entry_cmd (void)
     char *title = 0, *url = 0;
     int ret;
 
-    ret = add_new_entry_input ("New hotlist entry", "Directory label", "Directory path",
+    /* Take current directory as default value for input fields */
+    title = url = cpanel->cwd;
+
+    ret = add_new_entry_input (_("New hotlist entry"), _("Directory label"), _("Directory path"),
 	 "[Hotlist]", &title, &url);
 
     if (!ret || !title || !*title || !url || !*url)
@@ -769,29 +900,42 @@ static int add_new_group_input (char *header, char *label, char **result)
     int		ret;
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
-    { quick_button, 55, 80, 1, 0, "&Cancel", 0, B_CANCEL, 0, 0,
+    { quick_button, 55, 80, 1, 0, N_("&Cancel"), 0, B_CANCEL, 0, 0,
 	  XV_WLAY_DONTCARE, "button-cancel" },
-    { quick_button, 30, 80, 1, 0, "&Insert", 0, B_INSERT, 0, 0,
+    { quick_button, 30, 80, 1, 0, N_("&Insert"), 0, B_INSERT, 0, 0,
 	  XV_WLAY_DONTCARE, "button-insert" },
-    { quick_button, 10, 80, 1, 0, "&Append", 0, B_APPEND, 0, 0,
+    { quick_button, 10, 80, 1, 0, N_("&Append"), 0, B_APPEND, 0, 0,
 	  XV_WLAY_DONTCARE, "button-append" },
     { quick_input,  4, 80,  0, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, "input" },
-    { quick_label,  3, 80, 2, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
+    { quick_label,  3, 80,  2, 0,  0,  0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
     { 0 } };
     
     int len;
     int i;
     int lines;
     char *my_str;
+
+#ifdef ENABLE_NLS
+	static int i18n_flag = 0;
+#endif /* ENABLE_NLS */
     
     len = max (strlen (header), msglen (label, &lines)) + 4;
     len = max (len, 64);
 
+#ifdef ENABLE_NLS
+	if (!i18n_flag)
+	{
+		add_widgets_i18n(quick_widgets, len);
+		i18n_flag = 1;
+	}
+#endif /* ENABLE_NLS */
+
     Quick_input.xlen  = len;
     Quick_input.xpos  = -1;
     Quick_input.title = header;
     Quick_input.help  = "[Hotlist]";
     Quick_input.class = "hotlist_new_group";
+    Quick_input.i18n  = 0;
     quick_widgets [4].text = label;
 
     for (i = 0; i < 5; i++)
@@ -817,7 +961,7 @@ void add_new_group_cmd (void)
     char   *label;
     int		ret;
 
-    ret = add_new_group_input (" New hotlist group ", "Name of new group", &label);
+    ret = add_new_group_input (_(" New hotlist group "), _("Name of new group"), &label);
     if (!ret || !label || !*label)
 	return;
 
@@ -832,10 +976,12 @@ void add_new_group_cmd (void)
 void add2hotlist_cmd (void)
 {
     char *prompt, *label;
+	char* cp = _("Label for \"%s\":");
+	int l = strlen(cp);
 
-    prompt = xmalloc (strlen (cpanel->cwd) + 20, "add2hotlist_cmd");
-    sprintf (prompt, "Label for \"%s\":", name_trunc (cpanel->cwd, COLS-2*UX-23));
-    label = input_dialog (" Add to hotlist ", prompt, "");
+    prompt = xmalloc (strlen (cpanel->cwd) + l, "add2hotlist_cmd");
+    sprintf (prompt, cp, name_trunc (cpanel->cwd, COLS-2*UX-(l+8)));
+    label = input_dialog (_(" Add to hotlist "), prompt, cpanel->cwd);
     free (prompt);
     if (!label || !*label)
 	return;
@@ -872,13 +1018,13 @@ static void remove_from_hotlist (struct hotlist *entry)
 	    char *header;
 	    int   result;
 
-	    header = copy_strings (" Remove: ",
+	    header = copy_strings (_(" Remove: "),
 				   name_trunc (entry->label, 30),
 				   " ",
 				   0);
-	    result = query_dialog (header, "\n Group not empty.\n Remove it?",
+	    result = query_dialog (header, _("\n Group not empty.\n Remove it?"),
 				   D_ERROR, 2,
-				   "&No", "&Yes");
+				   _("&No"), _("&Yes"));
 	    free (header);
 
 	    if (!result)
@@ -1220,7 +1366,7 @@ void load_hotlist (void)
     
     hotlist	       = new_hotlist ();
     hotlist->type      = HL_TYPE_GROUP;
-    hotlist->label     = strdup (" Top level group ");
+    hotlist->label     = strdup (_(" Top level group "));
     hotlist->up        = hotlist;
     /*
      * compatibility :-(
@@ -1234,12 +1380,14 @@ void load_hotlist (void)
 
     if ((hotlist_file = fopen (hotlist_file_name, "r")) == 0) {
 	int	result;
+	char    *msg;
 
-	message (0, " Hotlist Load ",
-		 "Hotlist is now kept in file ~/" HOTLIST_FILENAME,
-		 "MC will load hotlist from ~/" PROFILE_NAME " and then",
-		 "delete [Hotlist] section there"
-	);
+	msg = copy_strings (_("Hotlist is now kept in file ~/"), HOTLIST_FILENAME,
+			    _("MC will load hotlist from ~/"),  PROFILE_NAME,
+			    _(" and then delete [Hotlist] section there"), NULL);
+	message (0, _(" Hotlist Load "), msg);
+	free (msg);
+	
 	load_group (hotlist);
 	hotlist_state.loaded   = 1;
 	/*
@@ -1251,9 +1399,13 @@ void load_hotlist (void)
 	if (result) {
 	    remove_old_list = 1;
 	} else {
-	    message (D_ERROR, " Hotlist Load ",
-		     "MC was unable to write ~/" HOTLIST_FILENAME " file"
-		     "your old hotlist entries were not deleted");
+	    char *msg;
+
+	    msg = copy_strings (_("MC was unable to write ~/"), HOTLIST_FILENAME,
+				_(" file, your old hotlist entries were not deleted"), NULL);
+
+	    message (D_ERROR, _(" Hotlist Load "), msg);
+	    free (msg);
 	}
     } else {
 	hot_load_file (hotlist);
@@ -1261,19 +1413,22 @@ void load_hotlist (void)
 	hotlist_state.loaded = 1;
 	if (has_old_list) {
 	    int		result;
+	    char        *msg;
 
-	    result = query_dialog (" Hotlist Load ", 
-	   "You have ~/" HOTLIST_FILENAME " file and [Hotlist] section in ~/" PROFILE_NAME "\n" /**/
-	   "Your ~/" HOTLIST_FILENAME " most probably was created\n" /**/
-	   "by an earlier development version of MC\n" /**/
-	   "and is more actual than ~/" PROFILE_NAME " entries\n\n" /**/
-	   "You can choose between\n\n" /**/
-	   "  Remove - remove old hotlist entries from ~/" PROFILE_NAME "\n" /**/
-	   "  Keep   - keep your old entries; you will be asked\n" /**/
-	   "           the same question next time\n" /**/
-	   "  Merge  - add old entries to hotlist as group \"Entries from ~/" PROFILE_NAME "\"\n\n",
-				   D_ERROR, 3, "&Remove", "&Keep", "&Merge"
-		      );
+	    msg = copy_strings (
+		    _("You have ~/"), HOTLIST_FILENAME, _(" file and [Hotlist] section in ~/"), PROFILE_NAME, "\n",
+		    _("Your ~/"), HOTLIST_FILENAME, _(" most probably was created\n"),
+		    _("by an earlier development version of MC\nand is more actual than ~/"),
+		    PROFILE_NAME, _(" entries\n\n"),
+		    _("You can choose between\n\n"
+		      "  Remove - remove old hotlist entries from ~/"), PROFILE_NAME, "\n",
+		    _("  Keep   - keep your old entries; you will be asked\n"
+		      "           the same question next time\n"
+		      "  Merge  - add old entries to hotlist as group \"Entries from ~/"),
+		    PROFILE_NAME, "\"\n\n", NULL);
+
+	    result = query_dialog (_(" Hotlist Load "),
+				   msg, D_ERROR, 3, _("&Remove"), _("&Keep"), _("&Merge"));
 	    if (result == 0)
 		remove_old_list = 1;
 	    else if (result == 2) {
@@ -1285,17 +1440,21 @@ void load_hotlist (void)
 
 		old            = new_hotlist ();
 		old->type      = HL_TYPE_GROUP;
-		old->label     = strdup (" Entries from ~/" PROFILE_NAME);
+		old->label     = copy_strings (_(" Entries from ~/"), PROFILE_NAME, NULL);
 		old->up	       = hotlist;
 		old->head      = hotlist->head;
 		old->next      = grp;
 		hotlist->head  = old;
 		hotlist_state.modified = 1;
-		if (!save_hotlist ())
-		    message (D_ERROR, " Hotlist Load ",
-			 "MC was unable to write ~/" HOTLIST_FILENAME " file"
-			 "your old hotlist entries were not deleted");
-		else
+		if (!save_hotlist ()){
+		    char *str;
+
+		    str = copy_strings (_("MC was unable to write ~/"), HOTLIST_FILENAME,
+					_(" file your old hotlist entries were not deleted"), NULL);
+
+		    message (D_ERROR, _(" Hotlist Load "), str);
+		    free (str);
+		} else
 		    remove_old_list = 1;
 		hotlist_state.modified = 0;
 	    }
