@@ -48,6 +48,8 @@
 #include "key.h"
 #include "main.h"
 #include "file.h"
+#include "win.h"
+#include "cons.saver.h"
 #include "../vfs/vfs.h"
 
 #ifdef __linux__
@@ -61,7 +63,7 @@
 
 #include "x.h"
 
-/* "$Id: key.c,v 1.18 1995/02/21 19:06:13 miguel Exp $" */
+/* "$Id: key.c,v 1.6 1998/05/20 02:20:02 unammx Exp $" */
 
 /* This macros were stolen from gpm 0.15 */
 #define GET_TIME(tv) (gettimeofday(&tv, (struct timezone *)NULL))
@@ -250,6 +252,28 @@ void init_key (void)
 	define_sequences (xterm_key_defines);
     
     define_sequences (mc_bindings);
+    
+    /* load some additional keys (e.g. direct Alt-? support) */
+    load_xtra_key_defines();
+    
+#ifdef __QNX__
+    if (strncmp(term, "qnx", 3) == 0){
+	/* Modify the default value of use_8th_bit_as_meta: we would
+	 * like to provide a working mc for a newbie who knows nothing
+	 * about [Options|Display bits|Full 8 bits input]...
+	 * 
+	 * Don't use 'meta'-bit, when we are dealing with a 
+	 * 'qnx*'-type terminal: clear the default value!
+	 * These terminal types use 0xFF as an escape character,
+	 * so use_8th_bit_as_meta==1 must not be enabled!
+	 * 
+	 * [mc-4.1.21+,slint.c/getch(): the DEC_8BIT_HACK stuff
+	 * is not used now (doesn't even depend on use_8th_bit_as_meta
+	 * as in mc-3.1.2)...GREAT!...no additional code is required!]
+	 */
+	use_8th_bit_as_meta = 0;
+    }
+#endif /* __QNX__ */
 #endif /* !HAVE_X */
 }
 
@@ -407,12 +431,13 @@ int correct_key_code (int c)
 
     if (c == KEY_F(0))
 	return KEY_F(10);
-	
-    switch (c) {
-        case KEY_KP_ADD: c = alternate_plus_minus ? ALT('+') : '+'; break;
-        case KEY_KP_SUBTRACT: c = alternate_plus_minus ? ALT('-') : '-'; break;
-        case KEY_KP_MULTIPLY: c = alternate_plus_minus ? ALT('*') : '*'; break;
-    }
+
+    if (!alternate_plus_minus) 
+        switch (c) {
+            case KEY_KP_ADD: c = '+'; break;
+            case KEY_KP_SUBTRACT: c = '-'; break;
+            case KEY_KP_MULTIPLY: c = '*'; break;
+        }
 
     return c;
 }
@@ -559,7 +584,7 @@ int get_key_code (int no_delay)
 	    else {
 	        if (parent != NULL && parent->action == MCKEY_ESCAPE) {
 	            /* This is just to save a lot of define_sequences */
-	            if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')
+	            if (isalpha(c)
 			|| (c == '\n') || (c == '\t') || (c == XCTRL('h'))
 			|| (c == KEY_BACKSPACE) || (c == '!') || (c == '\r')
 			|| c == 127 || c == '+' || c == '-' || c == '\\' 
@@ -790,7 +815,8 @@ int mi_getch ()
 {
     Gpm_Event ev;
     int       key;
-    
+
+    ev.x = -1;
     while ((key = get_event (&ev, 0, 1)) == 0)
 	;
     return key;
@@ -892,6 +918,28 @@ char *learn_key (void)
     *p = 0;
     return strdup (buffer);
 }
+
+/* xterm and linux console only: set keypad to numeric or application
+   mode. Only in application keypad mode it's possible to distinguish
+   the '+' key and the '+' on the keypad ('*' and '-' ditto)*/
+void
+numeric_keypad_mode (void)
+{
+    if (console_flag || xterm_flag) {
+        fprintf (stdout, "\033>");
+        fflush (stdout);
+    }
+}
+
+void
+application_keypad_mode (void)
+{
+    if (console_flag || xterm_flag) {
+        fprintf (stdout, "\033="); 
+        fflush (stdout);
+    }
+}
+
 #endif /* !HAVE_X */
 
 /* A function to check if we're idle.
