@@ -17,7 +17,13 @@ int mi_getch (void);
 
 /* Used to get the modifier information          */
 /* Currently, it just works on the Linux console */
-#define SHIFT_PRESSED 1
+#ifdef _OS_NT
+#   ifndef SHIFT_PRESSED
+#   define SHIFT_PRESSED 0x0010
+#   endif
+#else
+#   define SHIFT_PRESSED 1
+#endif
 #define ALTR_PRESSED 2
 #define CONTROL_PRESSED 4
 #define ALTL_PRESSED 8
@@ -46,7 +52,7 @@ int is_abort_char (int c);
 int is_quit_char (int c);
 
 #define XCTRL(x) ((x) & 31)
-#define ALT(x) (0x200 | (x))
+#define ALT(x) (0x200 | (unsigned int)(x))
 
 /* To define sequences and return codes */
 #define MCKEY_NOACTION	0
@@ -58,6 +64,9 @@ int is_quit_char (int c);
 void do_define_key (int code, char *strcap);
 void define_sequence (int code, char *seq, int action);
 
+/* internally used in key.c, defined in keyxtra.c */
+void load_xtra_key_defines (void);
+
 /* Learn a single key */
 char *learn_key (void);
 
@@ -73,7 +82,10 @@ typedef struct {
 extern key_code_name_t key_name_conv_tab [];
 extern int we_are_background;
 
-#ifdef HAVE_GNOME
+/* Set keypad mode (xterm and linux console only) */
+#ifndef HAVE_X
+    void numeric_keypad_mode (void);
+    void application_keypad_mode (void);
 #endif
 
 #endif	/* __KEY_H */
