@@ -29,6 +29,13 @@
 #include <sys/stat.h>
 #if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(OS2_NT)
 #    include <termios.h>
+     /*
+      * If TIOCGWINSZ supported, make it available here, because window-
+      * resizing code depends on it...
+      */
+#    ifdef __QNX__              /* Maybe not only QNX-specific... */
+#	include <sys/ioctl.h>
+#    endif
 #endif
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
@@ -68,7 +75,7 @@
 
 #include "x.h"
 
-/* "$Id: layout.c,v 1.2 1995/02/21 19:06:17 miguel Exp $" */
+/* "$Id: layout.c,v 1.8 1998/05/20 02:20:09 unammx Exp $" */
 
 /* Controls the display of the rotating dash on the verbose mode */
 int nice_rotating_dash = 1;
@@ -140,7 +147,6 @@ static int height;
 #define MINWIDTH 10
 #define MINHEIGHT 5
 
-#define BX      12
 #define BY      12
 
 #define B_2LEFT B_USER
@@ -150,7 +156,10 @@ static int height;
 
 static Dlg_head *layout_dlg;
 
-static char *s_split_direction [2] = {"Vertical", "Horizontal"};
+static char *s_split_direction [2] = {
+    N_("&Vertical"), 
+    N_("&Horizontal")
+};
 WRadio *radio_widget;
 
 static struct {
@@ -159,18 +168,21 @@ static struct {
     WCheck *widget;
     char   *tkname;
 } check_options [] = {
-    { "&Xterm hintbar",	   &xterm_hintbar,   0, "h" },
-    { "h&Intbar visible",  &message_visible, 0, "v" },
-    { "&Keybar visible",   &keybar_visible,  0, "k" },
-    { "command &Prompt",   &command_prompt,  0, "p" },
-    { "show &Mini status", &show_mini_info,  0, "m" },
-    { "menu&Bar visible",  &menubar_visible, 0, "me" },
-    { "&Equal split",      &equal_split,     0, "eq" },
-    { "pe&Rmissions",      &permission_mode, 0, "pr" },
-    { "&File types",	   &filetype_mode,   0, "ft" },
+    { N_("&Xterm hintbar"),    &xterm_hintbar,   0, "h" },
+    { N_("h&Intbar visible"),  &message_visible, 0, "v" },
+    { N_("&Keybar visible"),   &keybar_visible,  0, "k" },
+    { N_("command &Prompt"),   &command_prompt,  0, "p" },
+    { N_("show &Mini status"), &show_mini_info,  0, "m" },
+    { N_("menu&Bar visible"),  &menubar_visible, 0, "me" },
+    { N_("&Equal split"),      &equal_split,     0, "eq" },
+    { N_("pe&Rmissions"),      &permission_mode, 0, "pr" },
+    { N_("&File types"),       &filetype_mode,   0, "ft" },
     { 0, 0, 0, 0 }
 };
 
+static int first_width, second_width;
+static char *layout_title, *title1, *title2, *title3, *output_lines_label;
+
 static WButton *bleft_widget, *bright_widget;
 
 static void _check_split (void)
@@ -269,20 +281,20 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
     	old_output_lines     = -1;
 	attrset (COLOR_NORMAL);
 	dlg_erase (h);
-	draw_box (h, 1, 1, 13, 56);
-	draw_box (h, 2, 4, 6, 23);
-	draw_box (h, 2, 29, 10, 25);
-	draw_box (h, 8, 4, 4, 23);
+	draw_box (h, 1, 2, h->lines - 2, h->cols - 4);
+	draw_box (h, 2, 4, 6, first_width);
+	draw_box (h, 8, 4, 4, first_width);
+	draw_box (h, 2, 5 + first_width, 10, second_width);
 
 	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, 25);
-	addstr (" Layout ");
+	dlg_move (h, 1, (h->cols - strlen(layout_title))/2);
+	addstr (layout_title);
 	dlg_move (h, 2, 5);
-	addstr (" Panel split ");
+	addstr (title1);
 	dlg_move (h, 8, 5);
-	addstr (" Highlight... ");
-	dlg_move (h, 2, 30);
-	addstr (" Other options ");
+	addstr (title2);
+	dlg_move (h, 2, 6 + first_width);
+	addstr (title3);
 	update_split ();
 	dlg_move (h, 6, 13);
 	addch ('=');
@@ -290,9 +302,9 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 	    if (old_output_lines != _output_lines){
 		old_output_lines = _output_lines;
 		attrset (COLOR_NORMAL);
-		dlg_move (h, 9, 40);
-		addstr ("output lines");
-		dlg_move (h, 9, 34);
+		dlg_move (h, 9, 16 + first_width);
+		addstr (output_lines_label);
+		dlg_move (h, 9, 10 + first_width);
 		printw ("%02d", _output_lines);
 	    }
 	}
@@ -339,7 +351,7 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 	    if (old_output_lines != _output_lines){
 		old_output_lines = _output_lines;
 		attrset (COLOR_NORMAL);
-		dlg_move (h, 9, 34);
+		dlg_move (h, 9, 10 + first_width);
 		printw ("%02d", _output_lines);
 	    }
 	}
@@ -353,43 +365,124 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 
 static void init_layout (void)
 {
-    int i;
+    static int i18n_layt_flag = 0;
+	static int b1, b2, b3;
+    int i = sizeof (s_split_direction) / sizeof(char*) ;
+	char* ok_button = _("&Ok");
+	char* cancel_button = _("&Cancel");
+	char* save_button = _("&Save");
+    
+    if (!i18n_layt_flag)
+    {
+		register int l1;
+		
+		first_width = 19; /* length of line with '<' '>' buttons */
+		
+		layout_title = _(" Layout ");
+		title1 = _(" Panel split ");
+		title2 = _(" Highlight... ");
+		title3 = _(" Other options ");
+		output_lines_label = _("output lines");
+		
+		while (i--)
+		{
+		    s_split_direction [i] = _(s_split_direction [i]);
+			l1 = strlen (s_split_direction [i]) + 7;
+			if (l1 > first_width)
+				first_width = l1;
+		}
 
-    layout_dlg = create_dlg (0, 0, 15, 58, dialog_colors, layout_callback,
-			     "[Layout]", "layout", DLG_CENTER | DLG_GRID);
+		for (i = 0; i <= 8; i++)
+		{
+			check_options[i].text = _(check_options[i].text);
+			l1 = strlen (check_options[i].text) + 7;
+			if (l1 > first_width)
+				first_width = l1;
+		}
+
+		l1 = strlen (title1) + 1;
+		if (l1 > first_width)
+			first_width = l1;
+		
+		l1 = strlen (title2) + 1;
+		if (l1 > first_width)
+			first_width = l1;
+		
+
+		second_width = strlen (title3) + 1;
+		for (i = 0; i < 6; i++)
+		{
+			check_options[i].text = _(check_options[i].text);
+			l1 = strlen (check_options[i].text) + 7;
+			if (l1 > second_width)
+				second_width = l1;
+		}
+		if (console_flag)
+		{
+			l1 = strlen (output_lines_label) + 13;
+			if (l1 > second_width)
+				second_width = l1;
+		}
+
+		/* 
+		 * alex@bcs.zp.ua:
+		 * To be completely correct, one need to check if layout_title
+		 * does not exceed dialog length and total length of 3 buttons
+		 * allows their placement in one row. But assuming this dialog
+		 * is wide enough, I don't include such a tests.
+		 *
+		 * Now the last thing to do - properly space buttons...
+		 */
+		l1 = 11 + strlen (ok_button)   /* 14 - all brackets and inner space */
+		 	+ strlen (save_button)     /* notice: it is 3 char less because */
+			+ strlen (cancel_button);  /* of '&' char in button text */
+		
+		i = (first_width + second_width - l1) / 4;
+		b1 = 5 + i;
+		b2 = b1 + strlen(ok_button) + i + 6;
+		b3 = b2 + strlen(save_button) + i + 4;
+
+		i18n_layt_flag = 1;
+    }
+
+    layout_dlg = create_dlg (0, 0, 15, first_width + second_width + 9, 
+		dialog_colors, layout_callback,
+		"[Layout]", "layout", DLG_CENTER | DLG_GRID);
 			     
-    x_set_dialog_title (layout_dlg, "Layout");
+    x_set_dialog_title (layout_dlg, _("Layout"));
 
     add_widgetl (layout_dlg,
-		button_new (BY, BX+25, B_CANCEL, NORMAL_BUTTON, "&Cancel", 0, 0, "c"), 
+		button_new (BY, b3, B_CANCEL, NORMAL_BUTTON, cancel_button, 0, 0, "c"), 
 		XV_WLAY_RIGHTOF);
     add_widgetl (layout_dlg,
-		button_new (BY, BX+12, B_EXIT, NORMAL_BUTTON, "&Save", 0, 0, "s"),
+		button_new (BY, b2, B_EXIT, NORMAL_BUTTON, save_button, 0, 0, "s"),
 		XV_WLAY_RIGHTOF);
     add_widgetl (layout_dlg,
-		button_new (BY, BX, B_ENTER, DEFPUSH_BUTTON, "&Ok", 0, 0, "o"),
+		button_new (BY, b1, B_ENTER, DEFPUSH_BUTTON, ok_button, 0, 0, "o"),
 		XV_WLAY_CENTERROW);
 #ifndef HAVE_X
     if (console_flag){
 	add_widget (layout_dlg,
-		    button_new (9, 36, B_MINUS, NARROW_BUTTON, "&-", bminus_cback, 0, NULL));
+		    button_new (9, 12 + first_width, B_MINUS, NARROW_BUTTON, "&-",
+			bminus_cback, 0, NULL));
 	add_widget (layout_dlg,
-		    button_new (9, 31, B_PLUS, NARROW_BUTTON, "&+", bplus_cback, 0, NULL));
+		    button_new (9, 7 + first_width, B_PLUS, NARROW_BUTTON, "&+", 
+			bplus_cback, 0, NULL));
     }
 #endif    
 
 #define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options[i].tkname
 
     for (i = 0; i < 6; i++){
-	check_options [i].widget = check_new (8 - i, 31, XTRACT(i));
+	check_options [i].widget = check_new (8 - i, 7 + first_width, XTRACT(i));
 	add_widgetl (layout_dlg, check_options [i].widget, XV_WLAY_BELOWCLOSE);
     }
 #ifdef HAVE_XVIEW
-    add_widgetl (layout_dlg, label_new (2, 30, "Other options", "oo"),
+    add_widgetl (layout_dlg, label_new (2, 7 + first_width, _("Other options"), "oo"),
         XV_WLAY_NEXTCOLUMN);
-    add_widgetl (layout_dlg, label_new (8, 5, "Highlight...", "hl"),
+    add_widgetl (layout_dlg, label_new (8, 5, _("Highlight..."), "hl"),
         XV_WLAY_NEXTCOLUMN);
-    add_widgetl (layout_dlg, label_new (2, 5, "Panel split", "ps"),
+    add_widgetl (layout_dlg, label_new (2, 5, _("Panel split"), "ps"),
         XV_WLAY_NEXTCOLUMN);
 #endif        
     check_options [8].widget = check_new (10, 6, XTRACT(8));
@@ -435,6 +528,7 @@ void layout_change (void)
 	how the panel are split (horizontal/vertical). */
     done_menu();
     init_menu();
+    menubar_arrange(the_menubar);
 #endif
 }
 
@@ -665,21 +759,14 @@ void flag_winch (int dummy)
 
 void edit_adjust_size (Dlg_head * h);
 
-void change_screen_size (void)
+#ifdef PORT_NEEDS_CHANGE_SCREEN_SIZE
+void low_level_change_screen_size (void)
 {
-#ifndef HAVE_X
 #if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
 #if defined TIOCGWINSZ && !defined SCO_FLAVOR
     struct winsize winsz;
-    extern Dlg_head *view_dlg;
-    extern Dlg_head *edit_dlg;
-    
-#ifndef NCURSES_VERSION
-    mc_noraw_mode ();
-    endwin ();
-#endif
-    winsz.ws_col = winsz.ws_row = 0;
 
+    winsz.ws_col = winsz.ws_row = 0;
     /* Ioctl on the STDIN_FILENO */
     ioctl (0, TIOCGWINSZ, &winsz);
     if (winsz.ws_col && winsz.ws_row){
@@ -694,7 +781,22 @@ void change_screen_size (void)
 	    resize_subshell ();
 #endif
     }
+#endif /* TIOCGWINSZ && !SCO_FLAVOR */
+#endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
+}
 
+void change_screen_size (void)
+{
+#if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
+#if defined TIOCGWINSZ && !defined SCO_FLAVOR
+    extern Dlg_head *view_dlg;
+    extern Dlg_head *edit_dlg;
+    
+#ifndef NCURSES_VERSION
+    mc_noraw_mode ();
+    endwin ();
+#endif
+    low_level_change_screen_size ();
     check_split ();
 #ifndef NCURSES_VERSION
     /* XSI Curses spec states that portable applications shall not invoke
@@ -713,14 +815,18 @@ void change_screen_size (void)
 	edit_adjust_size (edit_dlg);
 #endif
     
+#ifdef RESIZABLE_MENUBAR
+	menubar_arrange(the_menubar);
+#endif
+		
     /* Now, force the redraw */
     do_refresh ();
     touchwin (stdscr);
 #endif /* TIOCGWINSZ && !SCO_FLAVOR */
 #endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
-#endif /* HAVE_X */
     winch_flag = 0;
 }
+#endif /* HAVE_X */
 
 extern int verbose;
 static int ok_to_refresh = 1;
