@@ -43,7 +43,6 @@
 #define UX		4
 #define UY		3
 
-#define BX		25
 #define BY		UY + 17
 
 #define ROWS		13
@@ -53,10 +52,11 @@
 
 struct {
     int ret_cmd, flags, y, x;
+	unsigned int hotkey;
     char *text;
 } learn_but[BUTTONS] = {
-    { B_CANCEL, NORMAL_BUTTON, 0, 14, "&Cancel" },
-    { B_ENTER, DEFPUSH_BUTTON, 0,  0, "&Save" }
+    { B_CANCEL, NORMAL_BUTTON, 0, 39, 'C', N_("&Cancel") },
+    { B_ENTER, DEFPUSH_BUTTON, 0, 25, 'S', N_("&Save") }
 };
 
 static Dlg_head *learn_dlg;
@@ -70,6 +70,7 @@ static learnkey *learnkeys = NULL;
 static int learn_total;
 static int learnok;
 static int learnchanged;
+static char* learn_title = N_(" Learn keys ");
 
 #ifndef HAVE_X
 static void learn_refresh (void)
@@ -77,25 +78,25 @@ static void learn_refresh (void)
     attrset (COLOR_NORMAL);
     dlg_erase (learn_dlg);
     
-    draw_box (learn_dlg, 1, 2, 21, 73);
+    draw_box (learn_dlg, 1, 2, learn_dlg->lines - 2, learn_dlg->cols - 4);
     
     attrset (COLOR_HOT_NORMAL);
-    dlg_move (learn_dlg, 1, (72 - 12) / 2);
-    addstr (" Learn keys ");
+    dlg_move (learn_dlg, 1, (learn_dlg->cols - strlen (learn_title)) / 2);
+    addstr (learn_title);
 }
 #endif
 
 static int learn_button (int action, void *param)
 {
     unsigned char *seq;
-    Dlg_head *d = message (D_INSERT | 1, " Teach me a key ",
-"Please press the %s\n"
+    Dlg_head *d = message (D_INSERT | 1, _(" Teach me a key "),
+_("Please press the %s\n"
 "and then wait until this message disappears.\n\n"
 "Then, press it again to see if OK appears\n"
 "next to its button.\n\n"
 "If you want to escape, press a single Escape key\n"
-"and wait as well.", 
-        key_name_conv_tab [action - B_USER].longname);
+"and wait as well."), 
+        _(key_name_conv_tab [action - B_USER].longname));
     mc_refresh ();
     if (learnkeys [action - B_USER].sequence != NULL) {
 	free (learnkeys [action - B_USER].sequence);
@@ -117,8 +118,8 @@ static int learn_button (int action, void *param)
 	    define_sequence (key_name_conv_tab [action - B_USER].code, seq, 
 			     MCKEY_NOACTION);
 	} else {
-	    message (0, " Cannot accept this key ",
-		" You have entered \"%s\"", seq);
+	    message (0, _(" Cannot accept this key "),
+		_(" You have entered \"%s\""), seq);
 	}
 	
     	free (seq);
@@ -165,23 +166,23 @@ static int learn_check_key (int c)
 	    if (!learnkeys [i].ok) {
 	    	dlg_select_widget (learn_dlg, learnkeys [i].button);
 	        label_set_text ((WLabel *) learnkeys [i].label,
-	            "OK");
+	            _("OK"));
 	        learnkeys [i].ok = 1;
 	        learnok++;
 	        if (learnok >= learn_total) {
 	            learn_dlg->ret_value = B_CANCEL;
 	            if (learnchanged) {
-	                if (query_dialog (" Learn keys ", "\
-			    It seems that all your keys already\n\
-			    work fine. That's great.",
-	                    1, 2, "&Save", "&Discard") == 0)
+	                if (query_dialog (learn_title, 
+			    _("It seems that all your keys already\n"
+			      "work fine. That's great."),
+	                    1, 2, _("&Save"), _("&Discard")) == 0)
 	                    learn_dlg->ret_value = B_ENTER;
 	            } else {
-	            	message (1, " Learn keys ", "\
-			Great! You have a complete terminal database!\n\
-			All your keys work well.");
+	            	message (1, learn_title,
+			_("Great! You have a complete terminal database!\n"
+			  "All your keys work well."));
 	            }
-	            learn_dlg->running = 0;
+		    dlg_stop (learn_dlg);
 	        }
 	        return 1;
 	    }
@@ -200,14 +201,18 @@ static int learn_check_key (int c)
         case 'k':
             dlg_one_up (learn_dlg);
             return 1;
-        case 's':
-        case 'S':
-        case 'c':
-        case 'C':
-            /* Prevent from disappearing if a non-defined sequence is pressed
-               and contains s or c. Use ALT('s') or ALT('c'). */
-            return 1;
     }    
+
+    /* Prevent from disappearing if a non-defined sequence is pressed
+       and contains s or c. Use ALT('s') or ALT('c'). */
+	if (c < 255 && isalpha(c))
+	{
+		c = toupper(c);
+		for (i = 0; i < BUTTONS; i++)
+			if (c == learn_but [i].hotkey)
+				return 1;
+	}
+
     return 0;
 }
 
@@ -228,15 +233,39 @@ static void init_learn (void)
     int x, y, i, j;
     key_code_name_t *key;
     char buffer [22];
+	static int i18n_flag = 0;
 
     do_refresh ();
 
+#ifdef ENABLE_NLS
+	if (!i18n_flag)
+	{
+		char* cp;
+		
+		learn_but [0].text = _(learn_but [0].text);
+		learn_but [0].x = 78 / 2 + 4;
+
+		learn_but [1].text = _(learn_but [1].text);
+		learn_but [1].x = 78 / 2 - (strlen (learn_but [1].text) + 9);
+
+		for (i = 0; i < BUTTONS; i++)
+		{
+			cp = strchr(learn_but [i].text, '&');
+			if (cp != NULL && *++cp != '\0')
+				learn_but [i].hotkey = toupper(*cp);
+		}
+
+		learn_title = _(learn_title);
+		i18n_flag = 1;
+	}
+#endif /* ENABLE_NLS */
+
     learn_dlg = create_dlg (0, 0, 23, 78, dialog_colors,
 			      learn_callback, "[Learn keys]", "Learn keys",
 			      DLG_CENTER);
-    x_set_dialog_title (learn_dlg, "Learn keys");
+    x_set_dialog_title (learn_dlg, _("Learn keys"));
 
-#define XTRACT(i) BY+learn_but[i].y, BX+learn_but[i].x, learn_but[i].ret_cmd, learn_but[i].flags, learn_but[i].text, 0, 0, NULL
+#define XTRACT(i) BY+learn_but[i].y, learn_but[i].x, learn_but[i].ret_cmd, learn_but[i].flags, _(learn_but[i].text), 0, 0, NULL
 
     for (i = 0; i < BUTTONS; i++)
 	add_widget (learn_dlg, button_new (XTRACT (i)));
@@ -254,7 +283,7 @@ static void init_learn (void)
     for (i = j - 1, key = key_name_conv_tab + j - 1; i >= 0; i--, key--) {
     	learnkeys [i].ok = 0;
     	learnkeys [i].sequence = NULL;
-        sprintf (buffer, "%-16s", key->longname);
+        sprintf (buffer, "%-16s", _(key->longname));
 	add_widget (learn_dlg, learnkeys [i].button = (Widget *)
 	    button_new (y, x, B_USER + i, NARROW_BUTTON, buffer, learn_button, 0, NULL));
 	add_widget (learn_dlg, learnkeys [i].label = (Widget *)
@@ -266,9 +295,12 @@ static void init_learn (void)
 	    y = UY + ROWS - 1;
 	}
     }
-    add_widget (learn_dlg, label_new (UY+14, 5, "Press all the keys mentioned here. After you have done it, check", NULL));
-    add_widget (learn_dlg, label_new (UY+15, 5, "which keys are not marked with OK.  Press space on the missing", NULL));
-    add_widget (learn_dlg, label_new (UY+16, 5, "key, or click with the mouse to define it. Move around with Tab.", NULL));
+    add_widget (learn_dlg,
+		label_new (UY+14, 5, _("Press all the keys mentioned here. After you have done it, check"), NULL));
+    add_widget (learn_dlg,
+		label_new (UY+15, 5, _("which keys are not marked with OK.  Press space on the missing"), NULL));
+    add_widget (learn_dlg,
+		label_new (UY+16, 5, _("key, or click with the mouse to define it. Move around with Tab."), NULL));
 }
 
 static void learn_done (void)
@@ -304,13 +336,27 @@ void learn_save (void)
 void learn_keys (void)
 {
     int save_old_esc_mode = old_esc_mode;
+    int save_alternate_plus_minus = alternate_plus_minus;
     
     old_esc_mode = 0; /* old_esc_mode cannot work in learn keys dialog */
+    alternate_plus_minus = 1; /* don't translate KP_ADD, KP_SUBTRACT and
+                                 KP_MULTIPLY to '+', '-' and '*' in
+                                 correct_key_code */
+#ifndef HAVE_X
+    application_keypad_mode ();
+#endif
     init_learn ();
 
     run_dlg (learn_dlg);
     
     old_esc_mode = save_old_esc_mode;
+    alternate_plus_minus = save_alternate_plus_minus;
+
+#ifndef HAVE_X
+    if (!alternate_plus_minus)
+        numeric_keypad_mode ();
+
+#endif
     
     switch (learn_dlg->ret_value) {
     case B_ENTER:
