@@ -230,7 +230,7 @@ static void listmode_done (void)
 {
     destroy_dlg (listmode_dlg);
     if (0)
-	update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
     repaint_screen ();
 }
 
