@@ -35,21 +35,6 @@ void free (void *ptr);
 #include <strings.h>
 #endif
 
-#ifdef __NetBSD__
-#   define xBSD
-#endif
-
-#ifdef __OpenBSD__
-#   define xBSD
-#endif
-
-#if defined(SCO_FLAVOR) && defined(__GNUC__)
-extern char* strdup(const char*);
-#endif
-
-char *strstr (const char *haystack, const char *needle);
-/* void error (void);  FIXME -- needed? */
-
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -99,6 +84,27 @@ char *strstr (const char *haystack, const char *needle);
 #include <sys/vfs.h>
 #endif
 
+#ifdef __bsdi__
+#  ifndef MOUNT_UFS
+#    define xBSD
+#  endif
+#endif
+
+#ifdef __NetBSD__
+#   define xBSD
+#endif
+
+#ifdef __OpenBSD__
+#   define xBSD
+#endif
+
+#if defined(SCO_FLAVOR) && defined(__GNUC__)
+extern char* strdup(const char*);
+#endif
+
+char *strstr (const char *haystack, const char *needle);
+/* void error (void);  FIXME -- needed? */
+
 #ifdef DOLPHIN
 /* So special that it's not worth putting this in autoconf.  */
 #undef MOUNTED_FREAD_FSTYP
@@ -480,7 +486,9 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#    include <unistd.h>
+#endif
 #include <sys/disk.h>
 #include <sys/fsys.h>
 #include <sys/statfs.h>
