@@ -11,7 +11,7 @@ enum {
     KEY_BACKSPACE = 400,
     KEY_END, KEY_UP, KEY_DOWN, KEY_LEFT, KEY_RIGHT,
     KEY_HOME, KEY_A1, KEY_C1, KEY_NPAGE, KEY_PPAGE, KEY_IC,
-    KEY_ENTER, KEY_DC, KEY_SCANCEL
+    KEY_ENTER, KEY_DC, KEY_SCANCEL, KEY_BTAB
 };
 
 #define KEY_F(x) 1000+x
@@ -52,7 +52,6 @@ enum {
 #    define A_NORMAL    0x00
 #endif
 
-#define ACS_MAP(x) '*'
 #define COLOR_PAIR(x) x
 #define ERR -1
 #define TRUE 1
@@ -126,9 +125,9 @@ void init_pair (int, char *, char *);
 
 #define SLsmg_draw_double_box(r,c,dr,dc) SLsmg_draw_box ((r), (c), (dr), (dc))
 
-#if 0
-#   define one_vline() {acs (); addch (ACS_VLINE); noacs ();}
-#   define one_hline() {acs (); addch (ACS_HLINE); noacs ();}
+#ifdef OS2_NT
+#   define one_vline() addch(ACS_VLINE)
+#   define one_hline() addch(ACS_HLINE)
     /* This is fast, but unusefull if ! pc_system - doesn't use
        Alt_Char_Pairs [] :( */
 #else
