@@ -23,9 +23,6 @@
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #endif
-#ifdef __os2__
-#   include <direct.h>
-#endif
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>		/* For malloc() */
@@ -89,10 +86,10 @@ struct {
     char *text;
     char *tkname;
 } panelize_but[BUTTONS] = {
-    { B_CANCEL, NORMAL_BUTTON, 0, 53, "&Cancel",  "c"},
-    { B_ADD, NORMAL_BUTTON,    0, 28, "&Add new", "a"},
-    { B_REMOVE, NORMAL_BUTTON, 0, 16, "&Remove",  "r"},
-    { B_ENTER, DEFPUSH_BUTTON, 0,  0, "&Panelize","p"},
+    { B_CANCEL, NORMAL_BUTTON, 0, 53, N_("&Cancel"),  "c"},
+    { B_ADD, NORMAL_BUTTON,    0, 28, N_("&Add new"), "a"},
+    { B_REMOVE, NORMAL_BUTTON, 0, 16, N_("&Remove"),  "r"},
+    { B_ENTER, DEFPUSH_BUTTON, 0,  0, N_("Pane&lize"),"l"},
 };
 
 /* Directory panelize */
@@ -102,33 +99,38 @@ static struct panelize{
     struct panelize *next;
 } *panelize = NULL;
 
+static char* panelize_title = N_(" External panelize ");
+
 #ifndef HAVE_X
-static void panelize_refresh (void)
+static void
+panelize_refresh (void)
 {
     attrset (COLOR_NORMAL);
     dlg_erase (panelize_dlg);
     
-    draw_box (panelize_dlg, 1, 2, 20, 70);
-    draw_box (panelize_dlg, UY, UX, 12, 63);
+    draw_box (panelize_dlg, 1, 2, panelize_dlg->lines-2, panelize_dlg->cols-4);
+    draw_box (panelize_dlg, UY, UX, panelize_dlg->lines-10, panelize_dlg->cols-10);
     
     attrset (COLOR_HOT_NORMAL);
-    dlg_move (panelize_dlg, 1, (72 - 19) / 2);
-    addstr (" External panelize ");
+    dlg_move (panelize_dlg, 1, (panelize_dlg->cols - strlen(panelize_title)) / 2);
+    addstr (panelize_title);
 }
 #endif
 
-static INLINE void update_command ()
+static void
+update_command ()
 {
     if (l_panelize->pos != last_listitem) {
     	last_listitem = l_panelize->pos;
         assign_text (pname, 
             ((struct panelize *) l_panelize->current->data)->command);
 	pname->point = 0;
-        update_input (pname);
+        update_input (pname, 1);
     }
 }
 
-static int panelize_callback (Dlg_head * h, int Par, int Msg)
+static int
+panelize_callback (Dlg_head * h, int Par, int Msg)
 {
     switch (Msg) {
 #ifndef HAVE_X    
@@ -149,22 +151,50 @@ static int panelize_callback (Dlg_head * h, int Par, int Msg)
 
 static int l_call (void *data)
 {
-    return 1;
+	return listbox_nothing;
 }
 
 static void init_panelize (void)
 {
-    int i;
+    int i, panelize_cols = COLS - 6;
     struct panelize *current = panelize;
+
+#ifdef ENABLE_NLS
+	static int i18n_flag = 0;
+	static int maxlen = 0;
+	
+	if (!i18n_flag)
+	{
+		i = sizeof(panelize_but) / sizeof(panelize_but[0]);
+		while (i--)
+		{
+			panelize_but [i].text = _(panelize_but [i].text);
+			maxlen += strlen (panelize_but [i].text) + 5;
+		}
+		maxlen += 10;
+		panelize_title = _(panelize_title);
+
+		i18n_flag = 1;
+	}
+	panelize_cols = max(panelize_cols, maxlen);
+		
+	panelize_but [2].x = panelize_but [3].x 
+		+ strlen (panelize_but [3].text) + 7;
+	panelize_but [1].x = panelize_but [2].x 
+		+ strlen (panelize_but [2].text) + 5;
+	panelize_but [0].x = panelize_cols 
+		- strlen (panelize_but[0].text) - 8 - BX;
+
+#endif /* ENABLE_NLS */
     
     last_listitem = 0;
     
     do_refresh ();
 
-    panelize_dlg = create_dlg (0, 0, 22, 74, dialog_colors,
+    panelize_dlg = create_dlg (0, 0, 22, panelize_cols, dialog_colors,
 			      panelize_callback, "[External panelize]", "panelize",
 			      DLG_CENTER|DLG_GRID);
-    x_set_dialog_title (panelize_dlg, "External panelize");
+    x_set_dialog_title (panelize_dlg, _("External panelize"));
 
 #define XTRACT(i) BY+panelize_but[i].y, BX+panelize_but[i].x, panelize_but[i].ret_cmd, panelize_but[i].flags, panelize_but[i].text, 0, 0, panelize_but[i].tkname
 
@@ -172,13 +202,13 @@ static void init_panelize (void)
 	add_widgetl (panelize_dlg, button_new (XTRACT (i)), (i == BUTTONS - 1) ?
 	    XV_WLAY_CENTERROW : XV_WLAY_RIGHTOF);
 
-    pname = input_new (UY+14, UX, INPUT_COLOR, 63, "", "in");
+    pname = input_new (UY+14, UX, INPUT_COLOR, panelize_dlg->cols-10, "", "in");
     add_widgetl (panelize_dlg, pname, XV_WLAY_RIGHTOF);
 
-    add_widgetl (panelize_dlg, label_new (UY+13, UX, "Command", "label-command"), XV_WLAY_NEXTROW);
+    add_widgetl (panelize_dlg, label_new (UY+13, UX, _("Command"), "label-command"), XV_WLAY_NEXTROW);
 
     /* get new listbox */
-    l_panelize = listbox_new (UY + 1, UX + 1, 61, 10, 0, l_call, "li");
+    l_panelize = listbox_new (UY + 1, UX + 1, panelize_dlg->cols-12, 10, 0, l_call, "li");
 
     while (current){
 	listbox_add_item (l_panelize, 0, 0, current->label, current);
@@ -189,7 +219,7 @@ static void init_panelize (void)
     add_widgetl (panelize_dlg, l_panelize, XV_WLAY_EXTENDWIDTH); 
 
     listbox_select_entry (l_panelize, 
-        listbox_search_text (l_panelize, "Other command"));
+        listbox_search_text (l_panelize, _("Other command")));
 }
 
 static void panelize_done (void)
@@ -229,7 +259,8 @@ void add2panelize_cmd (void)
     char *label;
 
     if (pname->buffer && (*pname->buffer)) {
-	label = input_dialog (" Add to external panelize ", "Unnamed find", 
+	label = input_dialog (_(" Add to external panelize "), 
+		_(" Enter command label: "), 
 			      "");
 	if (!label)
 	    return;
@@ -244,7 +275,7 @@ void add2panelize_cmd (void)
 
 static void remove_from_panelize (struct panelize *entry)
 {
-    if (strcmp (entry->label, "Other command") != 0) {
+    if (strcmp (entry->label, _("Other command")) != 0) {
 	if (entry == panelize) {
 	    panelize = panelize->next;
 	} else {
@@ -267,13 +298,13 @@ void external_panelize (void)
     char *target = NULL;
 
     if (!vfs_current_is_local ()){
-	message (1, " Oops... ",
-	    " I can't run external panalize while logged on a non local directory ");
+	message (1, _(" Oops... "),
+		 _(" I can't run external panelize while logged on a non local directory "));
 	return;
     }
 
     init_panelize ();
-
+    
     /* display file info */
     attrset (SELECTED_COLOR);
 
@@ -314,12 +345,12 @@ void load_panelize (void)
     
     profile_keys = profile_init_iterator (panelize_section, profile_name);
     
-    add2panelize (strdup ("Other command"), strdup (""));
+    add2panelize (strdup (_("Other command")), strdup (""));
 
     if (!profile_keys){
-	add2panelize (strdup ("Find rejects after patching"), strdup ("find . -name \\*.rej -print"));
-	add2panelize (strdup ("Find *.orig after patching"), strdup ("find . -name \\*.orig -print"));
-	add2panelize (strdup ("Find SUID and SGID programs"), strdup ("find . \\( \\( -perm -04000 -a -perm +011 \\) -o \\( -perm -02000 -a -perm +01 \\) \\) -print"));
+	add2panelize (strdup (_("Find rejects after patching")), strdup ("find . -name \\*.rej -print"));
+	add2panelize (strdup (_("Find *.orig after patching")), strdup ("find . -name \\*.orig -print"));
+	add2panelize (strdup (_("Find SUID and SGID programs")), strdup ("find . \\( \\( -perm -04000 -a -perm +011 \\) -o \\( -perm -02000 -a -perm +01 \\) \\) -print"));
 	return;
     }
     
@@ -335,7 +366,7 @@ void save_panelize (void)
     
     profile_clean_section (panelize_section, profile_name);
     for (;current; current = current->next){
-    	if (strcmp (current->label, "Other command"))
+    	if (strcmp (current->label, _("Other command")))
 	    WritePrivateProfileString (panelize_section,
 				       current->label,
 				       current->command,
@@ -370,7 +401,7 @@ void do_external_panelize (char *command)
     open_error_pipe ();
     external = popen (command, "r");
     if (!external){
-	close_error_pipe (1, "Cannot invoke command.");
+	close_error_pipe (1, _("Cannot invoke command."));
 	return;
     }
     clean_dir (list, cpanel->count);
@@ -425,7 +456,7 @@ void do_external_panelize (char *command)
 #else /* SCO_FLAVOR */
     if (WEXITSTATUS(pclose (external)) < 0)
 #endif /* SCO_FLAVOR */
-	message (0, "External panelize", "Pipe close failed");
+	message (0, _("External panelize"), _("Pipe close failed"));
     close_error_pipe (0, 0);
     try_to_select (cpanel, NULL);
     paint_panel (cpanel);
