@@ -20,6 +20,9 @@
 #ifdef HAVE_UNISTD_H
 #    include <unistd.h>
 #endif
+#ifdef OS2_NT
+#    include <io.h>
+#endif
 
 #include "popt.h"
 
@@ -395,7 +398,8 @@ int poptParseArgvString(char * s, int * argcPtr, char *** argvPtr) {
     }
 
     if (strlen(argv[argc])) {
-	argc++, buf++;
+	argc++;
+	buf++;
     }
 
     argv2 = malloc(argc * sizeof(*argv) + (buf - bufStart));
