@@ -21,9 +21,6 @@
 #include <config.h>
 #include "tty.h"
 #include "fs.h"
-#ifdef __os2__
-#    include <direct.h>
-#endif
 #include <sys/param.h>
 #include <string.h>
 #include <stdlib.h>	/* For malloc() and free() */
@@ -62,7 +59,7 @@
 
 #include "x.h"
 
-/* "$Id: screen.c,v 1.19 1995/02/21 19:07:10 miguel Exp $" */
+/* "$Id: screen.c,v 1.26 1998/05/12 04:26:30 unammx Exp $" */
 #define ELEMENTS(arr) ( sizeof(arr) / sizeof((arr)[0]) )
 
 /* If true, show the mini-info on the panel */
@@ -94,11 +91,14 @@ WPanel *the_info_panel = 0;
 /* The hook list for the select file function */
 Hook *select_file_hook = 0;
 
-int panel_callback (Dlg_head *h, WPanel *p, int Msg, int Par);
+static int panel_callback (Dlg_head *h, WPanel *p, int Msg, int Par);
 int panel_event (Gpm_Event *event, WPanel *panel);
 
-#ifndef HAVE_TK
+#ifndef PORT_HAS_PANEL_ADJUST_TOP_FILE
 #   define x_adjust_top_file(p)
+#endif
+
+#ifndef PORT_HAS_PANEL_RESET_SORT_LABELS
 #   define x_reset_sort_labels(x) 
 #endif
 
@@ -109,13 +109,25 @@ int panel_event (Gpm_Event *event, WPanel *panel);
 #define llines(p) (p->widget.lines)
 #endif
 
+#ifdef PORT_NOT_FOCUS_SELECT_ITEM
+#   define focus_select_item(x)
+#else
+#   define focus_select_item(x) select_item(x)
+#endif
+
+#ifdef PORT_NOT_UNFOCUS_UNSELECT_ITEM
+#    define unfocus_unselect_item(x)
+#else
+#    define unfocus_unselect_item(x) unselect_item(x)
+#endif
+
 #ifdef HAVE_X
 #   define set_colors(x)
 #else
 #   define x_create_panel(x,y,z) 1;
 #   define x_panel_load_index(p,x)
 #   define x_panel_select_item(a,b,c)
-
+#   define x_panel_destroy(p)
 
 void
 set_colors (WPanel *panel)
@@ -126,6 +138,10 @@ set_colors (WPanel *panel)
 }
 #endif
 
+#ifndef ICONS_PER_ROW
+#    define ICONS_PER_ROW(x) 1
+#endif
+
 /* Delete format string, it is a linked list */
 void
 delete_format (format_e *format)
@@ -193,7 +209,7 @@ file_entry_color (file_entry *fe)
             return (SPECIAL_COLOR);
         else if (is_exe (fe->buf.st_mode))
             return (EXECUTABLE_COLOR);
-	else if (strcmp (fe->fname, "core") == 0)
+	else if (fe->fname && (strcmp (fe->fname, "core") == 0))
 	    return (CORE_COLOR);
     }
     return (NORMAL_COLOR);
@@ -252,7 +268,7 @@ string_file_permission (file_entry *fe, int len)
 char *
 string_file_nlinks (file_entry *fe, int len)
 {
-    static char buffer [16];
+    static char buffer [20];
 
     sprintf (buffer, "%16d", fe->buf.st_nlink);
     return buffer;
@@ -315,6 +331,14 @@ string_file_ctime (file_entry *fe, int len)
     return file_date (fe->buf.st_ctime);
 }
 
+#ifdef HAVE_GNOME
+/* In GNOME, the CList truncates the names */
+char *
+string_file_name (file_entry *fe, int len)
+{
+	return fe->fname;
+}
+#else
 char *
 string_file_name (file_entry *fe, int len)
 {
@@ -323,6 +347,7 @@ string_file_name (file_entry *fe, int len)
     else
 	return fe->fname;
 }
+#endif
 
 char *
 string_space (file_entry *fe, int len)
@@ -356,7 +381,7 @@ string_inode (file_entry *fe, int len)
 {
     static char buffer [9];
 
-    sprintf (buffer, "%ld", fe->buf.st_ino);
+    sprintf (buffer, "%ld", (long) fe->buf.st_ino);
     return buffer;
 }
 
@@ -378,6 +403,12 @@ string_file_nuid (file_entry *fe, int len)
     return buffer;
 }
 
+#ifdef HAVE_GNOME
+#    define GT 2
+#else
+#    define GT 1
+#endif
+
 static struct {
     char *id;
     int  min_size;
@@ -388,25 +419,25 @@ static struct {
     char *(*string_fn)(file_entry *, int);
     sortfn *sort_routine;
 } formats [] = {
-{ "name",  12, 1, J_LEFT,  "Name",       1, string_file_name,       (sortfn *) sort_name },
-{ "size",  7,  0, J_RIGHT, "Size",       1, string_file_size,       (sortfn *) sort_size },
-{ "type",  1,  0, J_LEFT,  "",           1, string_file_type,       (sortfn *) sort_type },
-{ "mtime", 12, 0, J_RIGHT, "MTime",      1, string_file_mtime,      (sortfn *) sort_time },
-{ "bsize", 7,  0, J_RIGHT, "Size",       1, string_file_size_brief, (sortfn *) sort_size },
-{ "perm",  10, 0, J_LEFT,  "Permission", 1, string_file_permission, NULL },
-{ "mode",  6,  0, J_RIGHT, "Perm",       1, string_file_perm_octal, NULL },
-{ "|",     1,  0, J_RIGHT, "|",          0, 0,                      NULL },
-{ "nlink", 2,  0, J_RIGHT, "Nl",         1, string_file_nlinks,     (sortfn *) sort_links },
-{ "ngid",  5,  0, J_RIGHT, "GID",        1, string_file_ngid,       (sortfn *) sort_ngid },
-{ "nuid",  5,  0, J_RIGHT, "UID",        1, string_file_nuid,       (sortfn *) sort_nuid },
-{ "owner", 8,  0, J_LEFT,  "Owner",      1, string_file_owner,      (sortfn *) sort_owner },
-{ "group", 8,  0, J_LEFT,  "Group",      1, string_file_group,      (sortfn *) sort_group },
-{ "atime", 12, 0, J_RIGHT, "ATime",      1, string_file_atime,      (sortfn *) sort_atime },
-{ "ctime", 12, 0, J_RIGHT, "CTime",      1, string_file_ctime,      (sortfn *) sort_ctime },
-{ "space", 1,  0, J_RIGHT, " ",          0, string_space,           NULL },
-{ "dot",   1,  0, J_RIGHT, " ",          0, string_dot,             NULL },
-{ "mark",  1,  0, J_RIGHT, " ",          1, string_marked,          NULL },
-{ "inode", 5,  0, J_RIGHT, "Inode",      1, string_inode,           (sortfn *) sort_inode },
+{ "name",  12, 1, J_LEFT,  N_("Name"),       1, string_file_name,       (sortfn *) sort_name },
+{ "size",  7,  0, J_RIGHT, N_("Size"),       1, string_file_size,       (sortfn *) sort_size },
+{ "type",  GT, 0, J_LEFT,     "",            1, string_file_type,       (sortfn *) sort_type },
+{ "mtime", 12, 0, J_RIGHT, N_("MTime"),      1, string_file_mtime,      (sortfn *) sort_time },
+{ "bsize", 7,  0, J_RIGHT, N_("Size"),       1, string_file_size_brief, (sortfn *) sort_size },
+{ "perm",  10, 0, J_LEFT,  N_("Permission"), 1, string_file_permission, NULL },
+{ "mode",  6,  0, J_RIGHT, N_("Perm"),       1, string_file_perm_octal, NULL },
+{ "|",     1,  0, J_RIGHT, N_("|"),          0, 0,                      NULL },
+{ "nlink", 2,  0, J_RIGHT, N_("Nl"),         1, string_file_nlinks,     (sortfn *) sort_links },
+{ "ngid",  5,  0, J_RIGHT, N_("GID"),        1, string_file_ngid,       (sortfn *) sort_ngid },
+{ "nuid",  5,  0, J_RIGHT, N_("UID"),        1, string_file_nuid,       (sortfn *) sort_nuid },
+{ "owner", 8,  0, J_LEFT,  N_("Owner"),      1, string_file_owner,      (sortfn *) sort_owner },
+{ "group", 8,  0, J_LEFT,  N_("Group"),      1, string_file_group,      (sortfn *) sort_group },
+{ "atime", 12, 0, J_RIGHT, N_("ATime"),      1, string_file_atime,      (sortfn *) sort_atime },
+{ "ctime", 12, 0, J_RIGHT, N_("CTime"),      1, string_file_ctime,      (sortfn *) sort_ctime },
+{ "space", 1,  0, J_RIGHT,    " ",          0, string_space,           NULL },
+{ "dot",   1,  0, J_RIGHT,    " ",          0, string_dot,             NULL },
+{ "mark",  1,  0, J_RIGHT,    " ",          1, string_marked,          NULL },
+{ "inode", 5,  0, J_RIGHT, N_("Inode"),      1, string_inode,           (sortfn *) sort_inode },
 };
 
 static char *
@@ -483,16 +514,25 @@ format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int
 	color = NORMAL_COLOR;
     for (format = home; format; format = format->next){
 	if (format->string_fn){
+	    int len;
+	    
 	    if (empty_line)
 		txt = " ";
 	    else
 		txt = (*format->string_fn)(fe, format->field_len);
 
 	    old_pos = cdest;
-	    cdest = to_buffer (cdest, format->just_mode, format->field_len, txt);
-	    length += format->field_len;
 
-#ifndef HAVE_X
+	    len = format->field_len;
+	    if (len + length > width)
+		len = width - length;
+	    cdest = to_buffer (cdest, format->just_mode, len, txt);
+	    length += len;
+
+#ifdef HAVE_X
+	    if (length == width)
+		    break;
+#else
             /* What shall we do? Will we color each line according to
 	     * the file type? Any suggestions to mc@timur.kazan.su
 	     */
@@ -545,10 +585,10 @@ repaint_file (WPanel *panel, int file_index, int mv, int attr, int isstatus)
     if (!isstatus && panel->split){
 	
 	second_column = (file_index - panel->top_file) / llines (panel);
-	width = (panel->widget.cols/2 - 2) - 1;
+	width = (panel->widget.cols - 2)/2 - 1;
 	
 	if (second_column){
-	    offset = 2 + width;
+	    offset = 1 + width;
 	    width = (panel->widget.cols-2) - (panel->widget.cols-2)/2 - 1;
 	} 
     } else
@@ -577,10 +617,10 @@ repaint_file (WPanel *panel, int file_index, int mv, int attr, int isstatus)
 }
 #endif
 
+#ifndef PORT_HAS_DISPLAY_MINI_INFO
 void
 display_mini_info (WPanel *panel)
 {
-#ifndef HAVE_X
     if (!show_mini_info)
 	return;
     
@@ -601,7 +641,7 @@ display_mini_info (WPanel *panel)
 	attrset (MARKED_COLOR);
 	printw  ("%*s", panel->widget.cols-2, " ");
 	widget_move (&panel->widget, llines (panel)+3, 1);
-	sprintf (buffer, "  %s bytes in %d file%s",
+	sprintf (buffer, _("  %s bytes in %d file%s"),
 		 size_trunc_sep (panel->total), panel->marked,
 		 panel->marked == 1 ? "" : "s");
 	p = buffer;
@@ -628,17 +668,15 @@ display_mini_info (WPanel *panel)
 	    printw ("-> %-*s", panel->widget.cols - 5,
 		     name_trunc (link_target, panel->widget.cols - 5));
 	} else 
-	    addstr ("<readlink failed>");
+	    addstr (_("<readlink failed>"));
 	return;
     }
 #endif
     /* Default behaviour */
-#ifndef HAVE_XVIEW
     repaint_file (panel, panel->selected, 0, STATUS, 1);
-#endif	
     return;
-#endif
 }
+#endif
 
 #ifndef HAVE_X
 void
@@ -716,10 +754,16 @@ show_dir (WPanel *panel)
     if (panel->active)
 	attrset (REVERSE_COLOR);
 
-    widget_move (&panel->widget, 0, 1);
+    widget_move (&panel->widget, 0, 3);
 
-    trim (strip_home_and_password (panel->cwd), tmp, panel->widget.cols-5);
+    trim (strip_home_and_password (panel->cwd), tmp, panel->widget.cols-7);
     addstr (tmp);
+    widget_move (&panel->widget, 0, 1);
+    addstr ("<");
+    widget_move (&panel->widget, 0, panel->widget.cols-2);
+    addstr (">");
+    widget_move (&panel->widget, 0, panel->widget.cols-3);
+    addstr ("v");
     
     if (panel->active)
 	standend ();
@@ -864,11 +908,28 @@ static void
 panel_destroy (WPanel *p)
 {
     int i;
-    
+
     char *name = panel_save_name (p);
 
     panel_save_setup (p, name);
+    x_panel_destroy (p);
     clean_dir (&p->dir, p->count);
+    
+/* save and clean history */
+    if (p->dir_history){
+	Hist *current, *old;
+	history_put (p->hist_name, p->dir_history);
+	current = p->dir_history;
+	while (current->next)
+	    current = current->next;
+	while (current){
+	    old = current;
+	    current = current->prev;
+	    free (old->text);
+	    free (old);
+	}
+    }
+    free (p->hist_name);
 
     delete_format (p->format);
     delete_format (p->status_format);
@@ -888,6 +949,13 @@ panel_format_modified (WPanel *panel)
     x_reset_sort_labels (panel);
 }
 
+int
+is_a_panel (Widget *w)
+{
+	return (w->callback == (void *) panel_callback);
+}
+
+void directory_history_add (WPanel * panel, char *s);
 
 /* Panel creation */
 /* The parameter specifies the name of the panel for setup retieving */
@@ -899,7 +967,8 @@ panel_new (char *panel_name)
     int i, err;
 
     panel = xmalloc (sizeof (WPanel), "panel_new");
-
+    memset (panel, 0, sizeof (WPanel));
+    
     /* No know sizes of the panel at startup */
     init_widget (&panel->widget, 0, 0, 0, 0, (callback_fn)
 		 panel_callback, (destroy_fn) panel_destroy,
@@ -911,24 +980,28 @@ panel_new (char *panel_name)
     mc_get_current_wd (panel->cwd, sizeof (panel->cwd)-2);
     strcpy (panel->lwd, ".");
 
-    panel->dir.list    = (file_entry *) malloc (MIN_FILES * sizeof (file_entry));
-    panel->dir.size    = MIN_FILES;
-    panel->active      = 0;
-    panel->filter      = 0;
-    panel->split       = 0;
-    panel->top_file    = 0;
-    panel->selected    = 0;
-    panel->marked      = 0;
-    panel->total       = 0;
-    panel->reverse     = 0;
-    panel->dirty       = 1;
-    panel->searching   = 0;
-    panel->dirs_marked = 0;
-    panel->is_panelized  = 0;
-    panel->has_dir_sizes = 0;
-    panel->format	 = 0;
-    panel->status_format = 0;    
-    panel->format_modified = 1;
+    panel->hist_name = copy_strings ("Dir Hist ", panel_name, 0);
+    panel->dir_history = history_get (panel->hist_name);
+    directory_history_add (panel, panel->cwd);
+
+    panel->dir.list         = (file_entry *) malloc (MIN_FILES * sizeof (file_entry));
+    panel->dir.size         = MIN_FILES;
+    panel->active           = 0;
+    panel->filter           = 0;
+    panel->split            = 0;
+    panel->top_file         = 0;
+    panel->selected         = 0;
+    panel->marked           = 0;
+    panel->total            = 0;
+    panel->reverse          = 0;
+    panel->dirty            = 1;
+    panel->searching        = 0;
+    panel->dirs_marked      = 0;
+    panel->is_panelized     = 0;
+    panel->has_dir_sizes    = 0;
+    panel->format	    = 0;
+    panel->status_format    = 0;    
+    panel->format_modified  = 1;
     
     panel->panel_name = strdup (panel_name);
     panel->user_format = strdup (DEFAULT_USER_FORMAT);
@@ -1176,7 +1249,10 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus, i
 	    darr->use_in_gui          = formats [i].use_in_gui;
             darr->requested_field_len = formats [i].min_size;
             darr->string_fn           = formats [i].string_fn;
-            darr->title               = formats [i].title;
+	    if (formats [i].title [0])
+		    darr->title = _(formats [i].title);
+	    else
+		    darr->title = "";
             darr->id                  = formats [i].id;
 	    darr->expand              = formats [i].expands;
 	    
@@ -1220,7 +1296,7 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus, i
 	    delete_format (home);
 	    old_char = format [pos];
 	    format [pos] = 0;
-	    *error = copy_strings("Unknow tag on display format: ", format, 0);
+	    *error = copy_strings(_("Unknow tag on display format: "), format, 0);
 	    format [pos] = old_char;
 	    return 0;
 	}
@@ -1359,7 +1435,6 @@ panel_format (WPanel *panel)
     case list_brief:
 	return "half 2,type,name";
 
-
     case list_user:
 	return panel->user_format;
 
@@ -1505,10 +1580,8 @@ unselect_item (WPanel *panel)
 }
 #endif
 
-/*                           */
-/* Panel key binded commands */
-/*                           */
-static void move_down (WPanel *panel)
+static void
+do_move_down (WPanel *panel)
 {
     if (panel->selected+1 == panel->count)
 	return;
@@ -1531,7 +1604,7 @@ static void move_down (WPanel *panel)
 }
 
 static void
-move_up (WPanel *panel)
+do_move_up (WPanel *panel)
 {
     if (panel->selected == 0)
 	return;
@@ -1549,6 +1622,45 @@ move_up (WPanel *panel)
     select_item (panel);
 }
 
+static int
+move_rel (WPanel *panel, int rel)
+{
+    unselect_item (panel);
+    
+    if (rel < 0){
+	if (panel->selected + rel < 0)
+	    panel->selected = 0;
+	else
+	    panel->selected = panel->selected + rel;
+    } else {
+	if (panel->selected + rel >= panel->count)
+	    panel->selected = panel->count - 1;
+	else
+	    panel->selected = panel->selected + rel;
+    }
+    select_item (panel);
+}
+
+/*                           */
+/* Panel key binded commands */
+/*                           */
+static void
+move_up (WPanel *panel)
+{
+    if (panel->list_type == list_icons){
+	move_rel (panel, -ICONS_PER_ROW (panel));
+    } else
+	do_move_up (panel);
+}
+
+static void
+move_down (WPanel *panel)
+{
+    if (panel->list_type == list_icons){
+	move_rel (panel, ICONS_PER_ROW (panel));
+    } else
+	do_move_down (panel);
+}
 
 /* Changes the selection by lines (may be negative) */
 static void
@@ -1592,21 +1704,31 @@ move_selection (WPanel *panel, int lines)
 static int
 move_left (WPanel *panel, int c_code)
 {
-    if (panel->split){
-	move_selection (panel, -llines (panel));
+    if (panel->list_type == list_icons){
+	do_move_up (panel);
 	return 1;
-    } else 
-	return maybe_cd (c_code, 0);
+    } else {    
+	if (panel->split){
+	    move_selection (panel, -llines (panel));
+	    return 1;
+	} else 
+	    return maybe_cd (c_code, 0);
+    }
 }
 
 static int
 move_right (WPanel *panel, int c_code)
 {
-    if (panel->split){
-	move_selection (panel, llines (panel));
+    if (panel->list_type == list_icons){
+	do_move_down (panel);
 	return 1;
-    } else
-	return maybe_cd (c_code, 1);
+    } else {
+	if (panel->split){
+	    move_selection (panel, llines (panel));
+	    return 1;
+	} else
+	    return maybe_cd (c_code, 1);
+    }
 }
 
 static void
@@ -1805,6 +1927,19 @@ do_file_mark (WPanel *panel, int idx, int mark)
     }
 }
 
+void
+do_file_mark_range (WPanel *panel, int r1, int r2)
+{
+	const int start = min (r1, r2);
+	const int end   = max (r1, r2);
+	int i, mark;
+
+	mark = !panel->dir.list [start].f.marked;
+	
+	for (i = start; i < end; i++)
+	    do_file_mark (panel, i, mark);
+}
+
 static void
 do_mark_file (WPanel *panel, int do_move)
 {
@@ -1852,12 +1987,7 @@ do_search (WPanel *panel, int c_code)
 		break;
 	    wrapped = 1;
 	}
-#ifdef OS2_NT
-#   define FILENAME_COMPARE strncasecmp
-#else
-#   define FILENAME_COMPARE strncmp
-#endif
-	if (FILENAME_COMPARE (panel->dir.list [i].fname, panel->search_buffer, l) == 0){
+	if (STRNCOMP (panel->dir.list [i].fname, panel->search_buffer, l) == 0){
 	    unselect_item (panel);
 	    panel->selected = i;
 	    select_item (panel);
@@ -1889,12 +2019,13 @@ start_search (WPanel *panel)
     }
 }
 
-void
+int
 do_enter (WPanel *panel)
 {
     if (S_ISDIR (selection (panel)->buf.st_mode)
 	|| link_isdir (selection (panel))){
 	do_cd (selection (panel)->fname, cd_exact);
+	return 1;
     } else { 
 	if (is_exe (selection (panel)->buf.st_mode) &&
 	    if_link_is_exe (selection (panel))) {
@@ -1904,9 +2035,9 @@ do_enter (WPanel *panel)
 	    {
 	        char *tmp = name_quote (selection (panel)->fname, 0);
 	        char *cmd = copy_strings (".", PATH_SEP_STR, tmp, 0);
-	        if (!confirm_execute || (query_dialog (" The Midnight Commander ",
-	                                    " Do you really want to execute? ",
-	                                         0, 2, "&Yes", "&No") == 0))
+	        if (!confirm_execute || (query_dialog (_(" The Midnight Commander "),
+						       _(" Do you really want to execute? "),
+						       0, 2, _("&Yes"), _("&No")) == 0))
 	            execute (cmd);
 	        free (tmp);
 	        free (cmd);
@@ -1921,9 +2052,16 @@ do_enter (WPanel *panel)
 	        free (tmp2);
 	    }
 #endif /* USE_VFS */	    
-	    return;
-	} else
-	    regex_command (selection (panel)->fname, "Open", NULL, 0);
+	    return 1;
+	} else {
+	    char *p;
+	    
+	    p = regex_command (selection (panel)->fname, "Open", NULL, 0);
+	    if (p && (strcmp (p, "Success") == 0))
+		    return 1;
+	    else
+		    return 0;
+	}
     }
 }
 
@@ -1935,10 +2073,10 @@ chdir_other_panel (WPanel *panel)
     if (get_other_type () != view_listing)
 	return;
 
-    if (!S_ISDIR (cpanel->dir.list [cpanel->selected].buf.st_mode))
-	new_dir = concat_dir_and_file (cpanel->cwd, "..");
+    if (!S_ISDIR (panel->dir.list [panel->selected].buf.st_mode))
+	new_dir = concat_dir_and_file (panel->cwd, "..");
     else
-	new_dir = concat_dir_and_file (cpanel->cwd, cpanel->dir.list [cpanel->selected].fname);
+	new_dir = concat_dir_and_file (panel->cwd, panel->dir.list [panel->selected].fname);
 
     change_panel ();
     do_cd (new_dir, cd_exact);
@@ -1957,15 +2095,15 @@ chdir_to_readlink (WPanel *panel)
     if (get_other_type () != view_listing)
 	return;
 
-    if (S_ISLNK (cpanel->dir.list [cpanel->selected].buf.st_mode)) {
+    if (S_ISLNK (panel->dir.list [panel->selected].buf.st_mode)) {
 	char buffer [MC_MAXPATHLEN], *p;
 	int i;
 	struct stat mybuf;
 	
-	i = readlink (selection (cpanel)->fname, buffer, MC_MAXPATHLEN);
+	i = readlink (selection (panel)->fname, buffer, MC_MAXPATHLEN);
 	if (i < 0)
 	    return;
-	if (mc_stat (selection (cpanel)->fname, &mybuf) < 0)
+	if (mc_stat (selection (panel)->fname, &mybuf) < 0)
 	    return;
 	buffer [i] = 0;
 	if (!S_ISDIR (mybuf.st_mode)) {
@@ -1981,7 +2119,7 @@ chdir_to_readlink (WPanel *panel)
 	if (*buffer == PATH_SEP)
 	    new_dir = strdup (buffer);
 	else
-	    new_dir = concat_dir_and_file (cpanel->cwd, buffer);
+	    new_dir = concat_dir_and_file (panel->cwd, buffer);
 
 	change_panel ();
 	do_cd (new_dir, cd_exact);
@@ -2027,21 +2165,36 @@ static key_map panel_keymap [] = {
     { XCTRL('n'), move_down },		/* C-n like emacs */
     { XCTRL('s'), start_search },	/* C-s like emacs */
     { ALT('s'),   start_search },	/* M-s not like emacs */
-
-    /* The functions keys we deal with */
-    { KEY_F(3),   view_cmd },
-    { KEY_F(13),  view_simple_cmd },
-    { KEY_F(4),   edit_cmd },
-    { KEY_F(14),  edit_cmd_new },
-    { KEY_F(5),   copy_cmd },
-    { KEY_F(6),   ren_cmd },
-    { KEY_F(7),   mkdir_cmd },
-    { KEY_F(8),   delete_cmd },
-    { KEY_DC,     delete_cmd },
-	
     { XCTRL('t'), mark_file },
     { ALT('o'),   chdir_other_panel },
     { ALT('l'),   chdir_to_readlink },
+    { KEY_F(13),  view_simple_cmd },
+    { KEY_F(14),  edit_cmd_new },
+    { ALT('y'),   directory_history_prev },
+    { ALT('u'),   directory_history_next },
+    { ALT('H'),   directory_history_list },
+    { ALT('+'),	  select_cmd_panel },
+    { KEY_KP_ADD, select_cmd_panel },
+    { ALT('\\'),  unselect_cmd_panel },
+    { ALT('-'),	  unselect_cmd_panel },
+    { KEY_KP_SUBTRACT, unselect_cmd_panel },
+    { ALT('*'),	  reverse_selection_cmd_panel },
+    { KEY_KP_MULTIPLY, reverse_selection_cmd_panel },
+    
+
+#ifdef HAVE_GNOME
+    { '+',        select_cmd_panel },
+    { '\\',       unselect_cmd_panel },
+    { '-',        unselect_cmd_panel },
+    { '*',	  reverse_selection_cmd_panel },
+    { XCTRL('r'), reread_cmd },
+    { KEY_F(3),   view_panel_cmd },
+    { KEY_F(5),   copy_cmd },
+    { KEY_F(6),   ren_cmd },
+    { KEY_F(7),   mkdir_panel_cmd },
+    { KEY_F(8),   delete_cmd },
+#endif
+    
     { 0, 0 }
 };
     
@@ -2058,8 +2211,10 @@ panel_key (WPanel *panel, int key)
 	    return 1;
 	}
     }
-    if (torben_fj_mode && key == ALT('h'))
+    if (torben_fj_mode && key == ALT('h')) {
 	goto_middle_file (panel);
+	return 1;
+    }
 
     /* We do not want to take a key press if nothing can be done with it */
     /* The command line widget may do something more usefull */
@@ -2067,52 +2222,51 @@ panel_key (WPanel *panel, int key)
 	return move_left (panel, key);
 
     if (key == KEY_RIGHT)
-	move_right (panel, key);
+	return move_right (panel, key);
 
     if (is_abort_char (key)) {
 	panel->searching = 0;
 	display_mini_info (panel);
-    }
-
-    if (panel->searching){
-	do_search (panel, key);
 	return 1;
     }
-    if (!command_prompt)
-	start_search (panel);
-    if (key == -1)
-	return 0;
-    
-    if (panel_keymap [i].key_code == 0){
+
+    /* Do not eat characters not meant for the panel below ' ' (e.g. C-l). */
+    if ((key >= ' '&& key <= 255) || key == 8 || key == KEY_BACKSPACE) {
 	if (panel->searching){
-	    panel->searching = 0;
-	    display_mini_info (panel);
+	    do_search (panel, key);
+	    return 1;
+	}
+
+	if (!command_prompt) {
+	    start_search (panel);
+	    do_search (panel, key);
+	    return 1;
 	}
-	return 0;
     }
-    return 1;
+
+    return 0;
 }
 
-int
+static int
 panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 {
     switch (msg){
     case WIDGET_INIT:
 #ifdef HAVE_X
-	define_label (h, (Widget *)panel, 1, "Help", help_cmd);
-	define_label (h, (Widget *)panel, 2, "Menu", user_menu_cmd);
-	define_label (h, (Widget *)panel, 3, "View", view_panel_cmd);
-	define_label (h, (Widget *)panel, 4, "Edit", edit_panel_cmd);
-	define_label (h, (Widget *)panel, 5, "Copy", copy_cmd);
-	define_label (h, (Widget *)panel, 6, "RenMov", ren_cmd);
-	define_label (h, (Widget *)panel, 7, "Mkdir", mkdir_panel_cmd);
-	define_label (h, (Widget *)panel, 8, "Delete", delete_cmd);
+	define_label (h, (Widget *)panel, 1, _("Help"), help_cmd);
+	define_label (h, (Widget *)panel, 2, _("Menu"), user_menu_cmd);
+	define_label (h, (Widget *)panel, 3, _("View"), view_panel_cmd);
+	define_label (h, (Widget *)panel, 4, _("Edit"), edit_panel_cmd);
+	define_label (h, (Widget *)panel, 5, _("Copy"), copy_cmd);
+	define_label (h, (Widget *)panel, 6, _("RenMov"), ren_cmd);
+	define_label (h, (Widget *)panel, 7, _("Mkdir"), mkdir_panel_cmd);
+	define_label (h, (Widget *)panel, 8, _("Delete"), delete_cmd);
 	x_create_panel (h, h->wdata, panel);
 #endif	
 	return 1;
 	
     case WIDGET_DRAW:
-#ifndef HAVE_XVIEW    
+#ifndef HAVE_XVIEW
 	paint_panel (panel);
 #else
 	show_dir (panel);	
@@ -2131,18 +2285,20 @@ panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 	    subshell_chdir (panel->cwd);
 	    
 	show_dir (panel);
-	select_item (panel);
+	focus_select_item (panel);
 #ifndef HAVE_X	
-	define_label (h, (Widget *)panel, 1, "Help", help_cmd);
-	define_label (h, (Widget *)panel, 2, "Menu", user_menu_cmd);
-	define_label (h, (Widget *)panel, 3, "View", view_panel_cmd);
-	define_label (h, (Widget *)panel, 4, "Edit", edit_panel_cmd);
-	define_label (h, (Widget *)panel, 5, "Copy", copy_cmd);
-	define_label (h, (Widget *)panel, 6, "RenMov", ren_cmd);
-	define_label (h, (Widget *)panel, 7, "Mkdir", mkdir_panel_cmd);
-	define_label (h, (Widget *)panel, 8, "Delete", delete_cmd);
+	define_label (h, (Widget *)panel, 1, _("Help"), help_cmd);
+	define_label (h, (Widget *)panel, 2, _("Menu"), user_menu_cmd);
+	define_label (h, (Widget *)panel, 3, _("View"), view_panel_cmd);
+	define_label (h, (Widget *)panel, 4, _("Edit"), edit_panel_cmd);
+	define_label (h, (Widget *)panel, 5, _("Copy"), copy_cmd);
+	define_label (h, (Widget *)panel, 6, _("RenMov"), ren_cmd);
+	define_label (h, (Widget *)panel, 7, _("Mkdir"), mkdir_panel_cmd);
+	define_label (h, (Widget *)panel, 8, _("Delete"), delete_cmd);
 	redraw_labels (h, (Widget *)panel);
 #endif
+	/* Chain behaviour */
+	default_proc (h, WIDGET_FOCUS, par);
 	return 1;
 	
     case WIDGET_UNFOCUS:
@@ -2153,7 +2309,7 @@ panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 	}
 #ifdef HAVE_X	
 	show_dir (panel);
-	unselect_item (panel);
+	unfocus_unselect_item (panel);
 	panel->active = 0;
 #else
 	panel->active = 0;
@@ -2289,6 +2445,21 @@ panel_event (Gpm_Event *event, WPanel *panel)
     int my_index;
     extern void change_panel (void);
 
+    if (event->type & GPM_DOWN && event->x == 1 + 1 && event->y == 0 + 1) {
+	directory_history_prev (panel);
+	return MOU_NORMAL;
+    }
+
+    if (event->type & GPM_DOWN && event->x == panel->widget.cols - 2 + 1 && event->y == 0 + 1) {
+	directory_history_next (panel);
+	return MOU_NORMAL;
+    }
+
+    if (event->type & GPM_DOWN && event->x == panel->widget.cols - 3 + 1 && event->y == 0 + 1) {
+	directory_history_list (panel);
+	return MOU_NORMAL;
+    }
+
     event->y -= 2;
     if ((event->type & (GPM_DOWN|GPM_DRAG))){
 
@@ -2339,3 +2510,10 @@ panel_event (Gpm_Event *event, WPanel *panel)
 }
 #endif
 
+void
+panel_update_marks (WPanel *panel)
+{
+#ifdef PORT_HAS_UPDATE_MARKS
+	x_panel_update_marks (panel);
+#endif
+}
