@@ -43,7 +43,19 @@
 #include "menu.h"		/* menubar_visible declaration */
 #include "win.h"		/* lookup_key */
 #include "cmd.h"
-#include "dirhist.h"		/* Directory history routines */
+#include "x.h"
+
+#ifdef HAVE_GNOME
+#    include "gdesktop.h"
+#endif
+
+#ifndef PORT_LIST_MODE_NAME
+#     define PORT_LIST_MODE_NAME "list_mode"
+#endif
+
+#ifndef PORT_LIST_MODE_DEFAULT
+#     define PORT_LIST_MODE_DEFAULT "full"
+#endif
 
 #include "../vfs/vfs.h"
 #ifdef USE_NETCODE
@@ -53,7 +65,7 @@ extern int ftpfs_retry_seconds;
 extern int ftpfs_use_passive_connections;
 #endif
 
-/* "$Id: setup.c,v 1.9 1995/02/21 19:07:13 miguel Exp $" */
+/* "$Id: setup.c,v 1.9 1998/05/12 04:26:32 unammx Exp $" */
 
 #ifdef USE_VFS
 extern int vfs_timeout;
@@ -100,10 +112,11 @@ static struct {
     char *key;
     int  list_type;
 } list_types [] = {
-    { "full",  list_full },
+    { "full",  list_full  },
     { "brief", list_brief },
-    { "long",  list_long },
-    { "user",  list_user },
+    { "long",  list_long  },
+    { "user",  list_user  },
+    { "icons", list_icons },
     { 0, 0 }
 };
 
@@ -138,7 +151,12 @@ static struct {
     { 0, 0 }
 };
 
+
+#undef SAVE_CHANGES_OUTSIDE_OPTIONS_MENU
+
+#ifdef SAVE_CHANGES_OUTSIDE_OPTIONS_MENU
 extern int preserve_uidgid;
+#endif
 
 static struct {
     char *opt_name;
@@ -170,14 +188,13 @@ static struct {
     { "full_eight_bits", &full_eight_bits },
     { "use_8th_bit_as_meta", &use_8th_bit_as_meta },
     { "confirm_view_dir", &confirm_view_dir },
-    { "tree_navigation_flag", &tree_navigation_flag },
     { "mouse_move_pages", &mouse_move_pages },
     { "mouse_move_pages_viewer", &mouse_move_pages_viewer },
     { "fast_refresh", &fast_refresh },
     { "navigate_with_arrows", &navigate_with_arrows },
     { "advanced_chown", &advanced_chfns },
     { "drop_menus", &drop_menus },
-    { "wrap_mode",  &wrap_mode},
+    { "wrap_mode",  &global_wrap_mode},
     { "old_esc_mode", &old_esc_mode },
     { "cd_symlinks", &cd_symlinks },
     { "show_all_if_ambiguous", &show_all_if_ambiguous },
@@ -189,10 +206,15 @@ static struct {
     { "only_leading_plus_minus", &only_leading_plus_minus },
     { "show_output_starts_shell", &output_starts_shell },
     { "panel_scroll_pages", &panel_scroll_pages },
-    { "dive_into_subdirs", &dive_into_subdirs },
-    { "preserve_uidgid", &preserve_uidgid },
     { "xtree_mode", &xtree_mode },
     { "num_history_items_recorded", &num_history_items_recorded },
+#ifdef SAVE_CHANGES_OUTSIDE_OPTIONS_MENU
+    { "dive_into_subdirs", &dive_into_subdirs },
+    { "preserve_uidgid", &preserve_uidgid },
+    /* What about the other two options in the copy dialog
+       (follow links, stable symlinks) -Norbert */
+    { "tree_navigation_flag", &tree_navigation_flag },
+#endif
 #ifdef USE_VFS    
     { "tar_gzipped_memlimit", &tar_gzipped_memlimit },
     { "vfs_timeout", &vfs_timeout },
@@ -218,9 +240,14 @@ static struct {
     { "editor_option_auto_para_formatting", &option_auto_para_formatting },
     { "editor_option_typewriter_wrap", &option_typewriter_wrap },
     { "editor_edit_confirm_save", &edit_confirm_save },
+    { "editor_syntax_highlighting", &option_syntax_highlighting },
 #endif
 
-#ifndef HAVE_GNOME
+#ifdef HAVE_GNOME
+    { "want_transparent_text", &want_transparent_text },
+    { "want_transparent_icons", &want_transparent_icons },
+    { "icons_snap_to_grid", &icons_snap_to_grid },
+#else
     { "nice_rotating_dash", &nice_rotating_dash },
     { "horizontal_split", &horizontal_split },
 #endif
@@ -245,11 +272,10 @@ void panel_save_setup (WPanel *panel, char *section)
 
     for (i = 0; list_types [i].key; i++)
 	if (list_types [i].list_type == panel->list_type){
-	    save_string (section, "list_mode",
-				       list_types [i].key, profile_name);
+	    save_string (section, PORT_LIST_MODE_NAME, list_types [i].key, profile_name);
 	    break;
 	}
-
+    
     save_string (section, "user_format",
 			       panel->user_format, profile_name);
 			       
@@ -307,6 +333,7 @@ static void panel_save_type (char *section, int type)
 	}
 }
 
+#ifndef PORT_HAS_SAVE_PANEL_TYPES
 void save_panel_types ()
 {
     int type;
@@ -320,6 +347,7 @@ void save_panel_types ()
     if (type == view_listing)
 	panel_save_setup (right_panel, right_panel->panel_name);
 }
+#endif
 
 void save_setup (void)
 {
@@ -343,7 +371,7 @@ void save_setup (void)
     save_hotlist ();
     save_panelize ();
     save_panel_types ();
-    directory_history_save ();
+/*     directory_history_save (); */
     
 #ifdef USE_VFS
 #ifdef USE_NETCODE
@@ -376,14 +404,17 @@ void panel_load_setup (WPanel *panel, char *section)
 	}
 
     /* Load the listing mode */
-    load_string (section, "list_mode", "full", buffer, sizeof (buffer));
+    load_string (section, PORT_LIST_MODE_NAME, PORT_LIST_MODE_DEFAULT, buffer, sizeof (buffer));
     panel->list_type = list_full;
     for (i = 0; list_types [i].key; i++)
 	if (strcasecmp (list_types [i].key, buffer) == 0){
 	    panel->list_type = list_types [i].list_type;
 	    break;
 	}
-    
+#ifndef PORT_HAS_ICON_VIEW
+    if (panel->list_type == list_icons)
+        panel->list_type == list_full;
+#endif
     /* User formats */
     if (panel->user_format){
 	free (panel->user_format);
@@ -515,7 +546,7 @@ void load_setup (void)
 			     color_terminal_string, sizeof (color_terminal_string));
 
     /* Load the directory history */
-    directory_history_load ();
+/*    directory_history_load (); */
     /* Remove the temporal entries */
     profile_clean_section ("Temporal:New Left Panel", profile_name);
     profile_clean_section ("Temporal:New Right Panel", profile_name);
@@ -546,7 +577,7 @@ void done_setup (void)
     free (profile_name);
     done_hotlist ();
     done_panelize ();
-    directory_history_free ();
+/*    directory_history_free (); */
 }
 
 void load_keys_from_section (char *terminal, char *profile_name)
