@@ -74,8 +74,13 @@ static int no_slang_delay;
 /* {{{  Copied from ../slang/slgetkey.c, removed the DEC_8Bit_HACK, */
 extern unsigned int SLang_Input_Buffer_Len;
 extern unsigned char SLang_Input_Buffer [];
+#if SLANG_VERSION >= 10000
+extern unsigned int _SLsys_getkey (void);
+extern int _SLsys_input_pending (int);
+#else
 extern unsigned int SLsys_getkey (void);
 extern int SLsys_input_pending (int);
+#endif
 
 static unsigned int SLang_getkey2 (void)
 {
@@ -92,7 +97,11 @@ static unsigned int SLang_getkey2 (void)
 		(char *) (SLang_Input_Buffer + 1), imax);
 	return(ch);
      }
-   else return(SLsys_getkey ());
+#if SLANG_VERSION >= 10000
+   else return(_SLsys_getkey ());
+#else
+   else return(SLsys_getkey());
+#endif
 }
 
 static int SLang_input_pending2 (int tsecs)
@@ -100,9 +109,11 @@ static int SLang_input_pending2 (int tsecs)
    int n;
    unsigned char c;
    if (SLang_Input_Buffer_Len) return (int) SLang_Input_Buffer_Len;
-   
+#if SLANG_VERSION >= 10000  
+   n = _SLsys_input_pending (tsecs);
+#else
    n = SLsys_input_pending (tsecs);
-   
+#endif
    if (n <= 0) return 0;
    
    c = (unsigned char) SLang_getkey2 ();
@@ -335,10 +346,57 @@ vline (int character, int len)
     }
 }
 
+int max_index = 0;
+
 void
 init_pair (int index, char *foreground, char *background)
 {
+
     SLtt_set_color (index, "", foreground, background);
+    if (index > max_index)
+	max_index = index;
+}
+
+int
+alloc_color_pair (char *foreground, char *background)
+{
+    init_pair (++max_index, foreground, background);
+    return max_index;
+}
+
+int
+try_alloc_color_pair (char *fg, char *bg)
+{
+    static struct colors_avail {
+	struct colors_avail *next;
+	char *fg, *bg;
+	int index;
+    } *p, c =
+    {
+	0, 0, 0, 0
+    };
+
+    c.index = NORMAL_COLOR;
+    p = &c;
+    for (;;) {
+	if (((fg && p->fg) ? !strcmp (fg, p->fg) : fg == p->fg) != 0
+	    && ((bg && p->bg) ? !strcmp (bg, p->bg) : bg == p->bg) != 0)
+	    return p->index;
+	if (!p->next)
+	    break;
+	p = p->next;
+    }
+    p->next = malloc (sizeof (c));
+    p = p->next;
+    p->next = 0;
+    p->fg = fg ? strdup (fg) : 0;
+    p->bg = bg ? strdup (bg) : 0;
+    if (!fg)
+	fg = "white";
+    if (!bg)
+	bg = "blue";
+    p->index = alloc_color_pair (fg, bg);
+    return p->index;
 }
 
 char *color_terminals [] = {
@@ -348,8 +406,15 @@ char *color_terminals [] = {
     "linux",
     "xterm-color",
     "color-xterm",
+    "dtterm",
     "xtermc",
     "ansi",
+/* 'qnx*' terminals have non-ANSI-compatible color sequences... */
+    "qansi",
+    "qansi-g",
+    "qansi-m",
+    "qansi-t",
+    "qansi-w",
     0
 };
 
