@@ -20,6 +20,10 @@
 #include "global.h"
 #include "main.h"
 
+/* This flag is set by xterm detection routine in function main() */
+/* It is used by function view_other_cmd() */
+int xterm_flag = 0;
+
 /* The following routines only work on xterm terminals */
 
 void do_enter_ca_mode (void)
