@@ -27,9 +27,6 @@
 
    */
 #include <config.h>
-#ifdef __os2__
-#    include <direct.h>
-#endif
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -60,6 +57,9 @@
 #include "tree.h"
 #include "cmd.h"
 #include "../vfs/vfs.h"
+#ifdef OS2_NT
+#   include <io.h>
+#endif
 
 extern int command_prompt;
 
@@ -78,7 +78,7 @@ int xtree_mode = 0;
 static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par);
 #define tcallback (callback_fn) tree_callback
 
-/* "$Id: tree.c,v 1.3 1995/02/21 19:07:24 miguel Exp $" */
+/* "$Id: tree.c,v 1.7 1998/04/17 00:22:17 unammx Exp $" */
 
 /* Returns number of common characters */
 static inline int str_common (char *s1, char *s2)
@@ -623,7 +623,7 @@ void save_tree (WTree *tree)
     file = fopen (filename, "w");
     free (filename);
     if (!file){
-	fprintf (stderr, "Can't open the " MC_TREE " file for writing:\n%s\n",
+	fprintf (stderr, _("Can't open the %s file for writing:\n%s\n"), MC_TREE,
 		 unix_error_string (errno));
 	return;
     }
@@ -636,7 +636,7 @@ void save_tree (WTree *tree)
 	else
 	    i = fprintf (file, "%s\n", current->name);
 	if (i == EOF){
-	    fprintf (stderr, "Can't write to the " MC_TREE " file:\n%s\n",
+	    fprintf (stderr, _("Can't write to the %s file:\n%s\n"), MC_TREE,
 		 unix_error_string (errno));
 	    break;
 	}
@@ -1095,15 +1095,15 @@ void tree_copy (WTree *tree, char *default_dest)
 
     if (!tree->selected_ptr)
 	return;
-    sprintf (cmd_buf, "Copy \"%s\" directory to:",
+    sprintf (cmd_buf, _("Copy \"%s\" directory to:"),
 	     name_trunc (tree->selected_ptr->name, 50));
-    dest = input_expand_dialog (" Copy ", cmd_buf, default_dest);
+    dest = input_expand_dialog (_(" Copy "), cmd_buf, default_dest);
     if (!dest || !*dest){
 	return;
     }
     create_op_win (OP_COPY, 0);
     file_mask_defaults ();
-    copy_dir_dir (tree->selected_ptr->name, dest, 1, 0);
+    copy_dir_dir (tree->selected_ptr->name, dest, 1, 0, 0, 0);
     destroy_op_win ();
     free (dest);
 }
@@ -1128,20 +1128,20 @@ void tree_move (WTree *tree, char *default_dest)
 
     if (!tree->selected_ptr)
 	return;
-    sprintf (cmd_buf, "Move \"%s\" directory to:",
+    sprintf (cmd_buf, _("Move \"%s\" directory to:"),
 	     name_trunc (tree->selected_ptr->name, 50));
-    dest = input_expand_dialog (" Move ", cmd_buf, default_dest);
+    dest = input_expand_dialog (_(" Move "), cmd_buf, default_dest);
     if (!dest || !*dest){
 	return;
     }
     if (stat (dest, &buf)){
-	message (1, " Error ", " Can't stat the destination \n %s ",
+	message (1, _(" Error "), _(" Can't stat the destination \n %s "),
 		 unix_error_string (errno));
 	free (dest);
 	return;
     }
     if (!S_ISDIR (buf.st_mode)){
-	message (1, " Error ", " The destination isn't a directory ");
+	message (1, _(" Error "), _(" The destination isn't a directory "));
 	free (dest);
 	return;
     }
@@ -1176,26 +1176,26 @@ static int tree_mkdir_cmd (WTree *tree)
     return 1;
 }
 
-static int tree_rmdir_cmd (WTree *tree)
+static void tree_rmdir_cmd (WTree *tree)
 {
     char old_dir [MC_MAXPATHLEN];
 
     if (tree->selected_ptr){
 	if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
-	    return 0;
+	    return;
 	if (mc_chdir (PATH_SEP_STR))
-	    return 0;
+	    return;
 	if (confirm_delete){
 	    char *cmd_buf;
 	    int result;
 
 	    cmd_buf = xmalloc (strlen (tree->selected_ptr->name) + 20,
 			       "tree, rmdir_cmd");
-	    sprintf (cmd_buf, "  Delete %s?  ", tree->selected_ptr->name);
-	    result = query_dialog (" Delete ", cmd_buf, 3, 2, "&Yes", "&No");
+	    sprintf (cmd_buf, _("  Delete %s?  "), tree->selected_ptr->name);
+	    result = query_dialog (_(" Delete "), cmd_buf, 3, 2, _("&Yes"), _("&No"));
 	    free (cmd_buf);
 	    if (result != 0){
-		return 0;
+		return;
 	    }
 	}
 	create_op_win (OP_DELETE, 0);
@@ -1203,9 +1203,9 @@ static int tree_rmdir_cmd (WTree *tree)
 	    tree_forget_cmd (tree);
 	destroy_op_win ();
 	mc_chdir (old_dir);
-	return 1;
+	return;
     } else
-	return 0;
+	return;
 }
 
 #if 0
@@ -1229,8 +1229,8 @@ static void tree_toggle_navig (Dlg_head *h)
 void set_navig_label (Dlg_head *h)
 {
     define_label_data (h, (Widget *)tree,
-        4, tree_navigation_flag ? "Static" : "Dynamc",
-	(void (*)(void *))tree_toggle_navig, h);
+		       4, tree_navigation_flag ? _("Static") : _("Dynamc"),
+		       (void (*)(void *))tree_toggle_navig, h);
 }
 
 static void move_down (WTree *tree)
@@ -1310,7 +1310,7 @@ static void chdir_sel (WTree *tree)
 	paint_panel (cpanel);
 	select_item (cpanel);
     } else {
-	message (1, " Error ", " Can't chdir to \"%s\" \n %s ",
+	message (1, MSG_ERROR, _(" Can't chdir to \"%s\" \n %s "),
 		 tree->selected_ptr->name, unix_error_string (errno));
     }
     change_panel ();
@@ -1365,6 +1365,7 @@ static key_map tree_keymap [] = {
     { XCTRL('s'), start_search },
     { ALT('s'),   start_search },
     { XCTRL('r'), tree_rescan_cmd },
+    { KEY_DC,     tree_rmdir_cmd },
     { 0, 0 }
     };
 
@@ -1378,37 +1379,45 @@ static inline int tree_key (WTree *tree, int key)
 	        tree->searching = 0;
 	    (*tree_keymap [i].fn)(tree);
 	    show_tree (tree);
-	    break;
+	    return 1;
 	}
     }
+
     /* We do not want to use them if we do not need to */
     /* Input line may want to take the motion key event */
     if (key == KEY_LEFT)
 	return move_left (tree);
 
     if (key == KEY_RIGHT)
-	move_right (tree);
+	return move_right (tree);
 
     if (is_abort_char (key)) {
-	tree->searching = 0;
+	if (tree->is_panel) {
+	    tree->searching = 0;
+	    show_tree (tree);
+	    return 1;  /* eat abort char */
+	}
+	return 0;  /* modal tree dialog: let upper layer see the
+		      abort character and close the dialog */
     }
 
-    if (tree->searching){
-	tree_do_search (tree, key);
-	return 1;
-    }
-    if (!command_prompt)
-	start_search (tree);
+    /* Do not eat characters not meant for the tree below ' ' (e.g. C-l). */
+    if ((key >= ' '&& key <= 255) || key == 8 || key == KEY_BACKSPACE) {
+	if (tree->searching){
+	    tree_do_search (tree, key);
+	    show_tree (tree);
+	    return 1;
+	}
 
-    if (key == -1)
-	return 0;
-    
-    if (tree_keymap [i].key_code == 0){
-	if (tree->search_buffer [0])
-	    tree->search_buffer [0] = 0;
-	show_tree (tree);
+	if (!command_prompt) {
+	    start_search (tree);
+	    tree_do_search (tree, key);
+	    return 1;
+	}
+	return tree->is_panel;
     }
-    return  (key > ' ') ? tree->is_panel : 0;
+
+    return 0;
 }
 
 static void tree_frame (Dlg_head *h, WTree *tree)
@@ -1439,24 +1448,24 @@ static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par)
 
     case WIDGET_FOCUS:
 	tree->active = 1;
-	define_label (h, (Widget *)tree, 1, "Help", (voidfn) tree_help_cmd);
+	define_label (h, (Widget *)tree, 1, _("Help"), (voidfn) tree_help_cmd);
 	define_label_data (h, (Widget *)tree, 
-	    2, "Rescan", (buttonbarfn)tree_rescan_cmd, tree);
+	    2, _("Rescan"), (buttonbarfn)tree_rescan_cmd, tree);
 	define_label_data (h, (Widget *)tree, 
-	    3, "Forget", (buttonbarfn)tree_forget_cmd, tree);
+	    3, _("Forget"), (buttonbarfn)tree_forget_cmd, tree);
 	define_label_data (h, (Widget *)tree, 
-	    5, "Copy",   (buttonbarfn) tree_copy_cmd, tree);
+	    5, _("Copy"),   (buttonbarfn) tree_copy_cmd, tree);
 	define_label_data (h, (Widget *)tree, 
-	    6, "RenMov", (buttonbarfn) tree_move_cmd, tree);
+	    6, _("RenMov"), (buttonbarfn) tree_move_cmd, tree);
 #if 0
 	/* FIXME: mkdir is currently defunct */
 	define_label_data (h, (Widget *)tree, 
-	    7, "Mkdir",  (buttonbarfn) tree_mkdir_cmd, tree);
+	    7, _("Mkdir"),  (buttonbarfn) tree_mkdir_cmd, tree);
 #else
         define_label (h, (Widget *)tree, 7, "", 0);
 #endif
 	define_label_data (h, (Widget *)tree, 
-	    8, "Rmdir",  (buttonbarfn) tree_rmdir_cmd, tree);
+	    8, _("Rmdir"),  (buttonbarfn) tree_rmdir_cmd, tree);
 	set_navig_label (h);
 	redraw_labels (h, (Widget *)tree);
 
