@@ -17,9 +17,8 @@
 
 #include <config.h>
 #include <stdio.h>
-#ifdef __os2__
+#ifdef OS2_NT
 #    include <io.h>
-#    include <sys/stat.h>
 #endif
 #include "tty.h"
 #include <stdlib.h>	/* For free() */
@@ -27,7 +26,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-
+#include <fcntl.h>
 #include "mad.h"
 #include "util.h"
 #include "global.h"
@@ -47,7 +46,7 @@
 
 #include "view.h" /* for default_* externs */
 
-/* "$Id: user.c,v 1.16 1995/02/21 19:07:28 miguel Exp $" */
+/* "$Id: user.c,v 1.6 1998/05/08 02:25:27 unammx Exp $" */
 
 #define MAX_ENTRIES 40
 #define MAX_ENTRY_LEN 60
@@ -261,7 +260,7 @@ char *check_patterns (char *p)
 	if (value == 0 || value == 1)
 	    easy_patterns = value;
 	else
-	    message (1, " Error ", " Invalid shell pattern defination \"%c\". ", value + '0');
+	    message (1, MSG_ERROR, _(" Invalid shell pattern defination \"%c\". "), value + '0');
     }
     while (*p == '\n' || *p == '\t' || *p == ' ') p++;
     return p;
@@ -393,7 +392,7 @@ void debug_out (char *start, char *end, int cond)
 	    len = strlen (msg);
 	    if (len)
 		msg [len - 1] = 0;
-	    message (0, " Debug ", msg);
+	    message (0, _(" Debug "), msg);
 	    debug_flag = 0;
 	}
     } else {
@@ -402,13 +401,13 @@ void debug_out (char *start, char *end, int cond)
 	    return;
 	/* Save the result of the condition */
 	if (debug_error){
-	    strcat (msg, " ERROR: ");
+	    strcat (msg, _(" ERROR: "));
 	    debug_error = 0;
 	}
 	else if (cond)
-	    strcat (msg, " True:  ");
+	    strcat (msg, _(" True:  "));
 	else
-	    strcat (msg, " False: ");
+	    strcat (msg, _(" False: "));
 	/* Copy condition statement */
 	len = strlen (msg);
 	if (end == NULL){
@@ -492,6 +491,7 @@ void execute_menu_command (char *s)
 {
     char *commands;
     FILE *cmd_file;
+    int  cmd_file_fd;
     int  expand_prefix_found = 0;
     int parameter_found = 0;
     int do_quote;
@@ -503,11 +503,12 @@ void execute_menu_command (char *s)
     /* OS/2 and NT requires the command to end in .cmd */
     file_name = copy_strings (file_name, ".cmd", NULL);
 #endif
-    if ((cmd_file = fopen (file_name, "w+")) == NULL){
-	message (1, " Error ", " Can't create temporary command file \n %s ",
+    if ((cmd_file_fd = open (file_name, O_RDWR | O_CREAT | O_TRUNC | O_EXCL, 0600)) == -1){
+	message (1, MSG_ERROR, _(" Can't create temporary command file \n %s "),
 		 unix_error_string (errno));
 	return;
     }
+    cmd_file = fdopen (cmd_file_fd, "w");
     commands = strchr (s, '\n');
     if (!commands){
 	fclose (cmd_file);
@@ -530,7 +531,7 @@ void execute_menu_command (char *s)
 		char *parameter;
 		char *tmp;
 		parameter_found = 0;
-		parameter = input_dialog (" Parameter ", prompt, "");
+		parameter = input_dialog (_(" Parameter "), prompt, "");
 		if (!parameter || !*parameter){
 		    /* User canceled */
 		    fclose (cmd_file);
@@ -550,7 +551,7 @@ void execute_menu_command (char *s)
 		    prompt [len] = *commands;
 		    prompt [len+1] = 0;
 		} else
-		    prompt [sizeof (prompt)] = 0;
+		    prompt [sizeof (prompt)-1] = 0;
 	    }
 	} else if (expand_prefix_found){
 	    expand_prefix_found = 0;
@@ -603,9 +604,9 @@ menu_file_own(char* path)
 		extern int verbose;
 		if (verbose)
 		{
-			message (0, " Warning -- ignoring file ",
-				"File %s is not owned by root or you or is world writable.\n"
-				"Using it may compromise your security",
+			message (0, _(" Warning -- ignoring file "),
+				 _("File %s is not owned by root or you or is world writable.\n"
+				   "Using it may compromise your security"),
 				path
 			);
 		}
@@ -621,8 +622,8 @@ void user_menu_cmd (void)
     Listbox *listbox;
     
     if (!vfs_current_is_local ()){
-	message (1, " Oops... ",
-	    " I can't run programs while logged on a non local directory ");
+	message (1, _(" Oops... "),
+		 _(" I can't run programs while logged on a non local directory "));
 	return;
     }
     
@@ -637,7 +638,7 @@ void user_menu_cmd (void)
     }
 
     if ((data = load_file (menu)) == NULL){
-	message (1, " Error ", " Can't open file %s \n %s ",
+	message (1, MSG_ERROR, _(" Can't open file %s \n %s "),
 		 menu, unix_error_string (errno));
 	free (menu);
 	return;
@@ -660,7 +661,9 @@ void user_menu_cmd (void)
 	    } else if (*p == '+'){
 		if (*(p+1) == '='){
 		    /* Combined adding and default */
-		    p = test_line (++p, &accept_entry);
+		    char *q = p++;
+		    
+		    p = test_line (q, &accept_entry);
 		    if (selected == 0 && accept_entry)
 			selected = menu_lines;
 		} else {
@@ -669,8 +672,9 @@ void user_menu_cmd (void)
 		}
 	    } else if (*p == '='){
 		if (*(p+1) == '+'){
+		    char *q = p++;
 		    /* Combined adding and default */
-		    p = test_line (++p, &accept_entry);
+		    p = test_line (q, &accept_entry);
 		    if (selected == 0 && accept_entry)
 			selected = menu_lines;
 		} else {
@@ -706,7 +710,7 @@ void user_menu_cmd (void)
     max_cols = min (max (max_cols, col), MAX_ENTRY_LEN);
 
     /* Create listbox */
-    listbox = create_listbox_window (max_cols+2, menu_lines, " User menu ",
+    listbox = create_listbox_window (max_cols+2, menu_lines, _(" User menu "),
 				     "[Menu File Edit]");
     
     /* insert all the items found */
