@@ -84,7 +84,7 @@ int message (int error, char *header, char *text, ...);
 #include "user.h"		/* expand_format */
 #include "../vfs/vfs.h"
 
-/* "$Id: util.c,v 1.17 1995/02/21 19:07:33 miguel Exp $" */
+/* "$Id: util.c,v 1.6 1998/04/08 19:37:08 pavel Exp $" */
 
 char app_text [] = "Midnight-Commander";
 
@@ -229,7 +229,7 @@ char *name_trunc (char *txt, int trunc_len)
 	txt = PATH_SEP_STR;
     
     if (trunc_len > sizeof (x)-1){
-	fprintf (stderr, "name_trunc: too big");
+	fprintf (stderr, _("name_trunc: too big"));
 	trunc_len = sizeof (x)-1;
     }
     txt_len = strlen (txt);
@@ -321,7 +321,7 @@ char *string_perm (mode_t mode_bits)
     if (ismode (mode_bits, S_IXUSR)) mode [3] = 'x';
     if (ismode (mode_bits, S_IWUSR)) mode [2] = 'w';
     if (ismode (mode_bits, S_IRUSR)) mode [1] = 'r';
-#ifndef __os2__
+#ifndef OS2_NT
     if (ismode (mode_bits, S_ISUID)) mode [3] = (mode [3] == 'x') ? 's' : 'S';
     if (ismode (mode_bits, S_ISGID)) mode [6] = (mode [6] == 'x') ? 's' : 'S';
     if (ismode (mode_bits, S_IFCHR)) mode [0] = 'c';
@@ -436,10 +436,11 @@ static char *maybe_end_group (char *d, int do_group, int *was_wildcard)
 char *convert_pattern (char *pattern, int match_type, int do_group)
 {
     char *s, *d;
-    static char new_pattern [100];
+    char *new_pattern;
     int was_wildcard = 0;
 
     if (easy_patterns){
+	new_pattern = malloc (MC_MAXPATHLEN);
 	d = new_pattern;
 	if (match_type == match_file)
 	    *d++ = '^';
@@ -474,20 +475,9 @@ char *convert_pattern (char *pattern, int match_type, int do_group)
 	*d = 0;
 	return new_pattern;
     } else
-	return pattern;
+	return strdup (pattern);
 }
 
-/* 1 if string matches
-   0 if string doesn't match
-   -1 if error in pattern */
-#ifdef OS2_NT
-#  define STRCOMP stricmp
-#  define MC_ARCH_FLAGS REG_ICASE
-#else
-#  define STRCOMP strcmp
-#  define MC_ARCH_FLAGS 0
-#endif
-
 int regexp_match (char *pattern, char *string, int match_type)
 {
     static regex_t r;
@@ -501,9 +491,11 @@ int regexp_match (char *pattern, char *string, int match_type)
 	    free (old_pattern);
 	}
 	pattern = convert_pattern (pattern, match_type, 0);
-	if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB|MC_ARCH_FLAGS))
+	if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB|MC_ARCH_FLAGS)) {
+	    free (pattern);
 	    return -1;
-	old_pattern = strdup (pattern);
+	}
+	old_pattern = pattern;
 	old_type = match_type;
     }
     rval = !regexec (&r, string, 0, NULL, 0);
