@@ -10,15 +10,15 @@
 extern char* strdup(const char*);
 #endif
 
-int is_printable (int c);
-int msglen (char *text, int *lines);
+int  is_printable (int c);
+int  msglen (char *text, int *lines);
 char *trim (char *s, char *d, int len);
 char *name_quote (const char *c, int quote_percent);
 char *fake_name_quote (const char *c, int quote_percent);
 char *name_trunc (char *txt, int trunc_len);
 char *size_trunc (long int size);
 char *size_trunc_sep (long int size);
-int is_exe (mode_t mode);
+int  is_exe (mode_t mode);
 char *string_perm (mode_t mode_bits);
 char *strip_home_and_password(char *dir);
 char *extension (char *);
@@ -104,7 +104,11 @@ void close_error_pipe (int error, char *text);
 
 /* Process spawning */
 void my_putenv (char*, char*);
-int my_system (int as_shell_command, const char *shell, const char *command);
+#define EXECUTE_INTERNAL   1
+#define EXECUTE_TEMPFILE   2
+#define EXECUTE_AS_SHELL   4
+int my_system (int flags, const char *shell, const char *command);
+int my_system_get_child_pid (int flags, const char *shell, const char *command, pid_t *pid);
 void save_stop_handler (void);
 extern struct sigaction startup_handler;
 
@@ -120,9 +124,6 @@ char *get_current_wd (char *buffer, int size);
 int my_mkdir (char *s, mode_t mode);
 int my_rmdir (char *s);
 
-/* Returns true if the err means the directory is not empty */
-int errno_dir_not_empty (int err);
-
 /* Filesystem status */
 struct my_statfs {
     int type;
@@ -177,12 +178,16 @@ void message_2s (int flags, char *title, char *str1, char *str2);
 void message_3s (int flags, char *title, char *str1, char *str2, const char *str3);
 void message_1s1d (int flags, char *title, char *str, int d);
 void tell_parent (int msg);
+int max_open_files (void);
 
 #ifdef OS2_NT
 #    define PATH_SEP '\\'
 #    define PATH_SEP_STR "\\"
 #    define PATH_ENV_SEP ';'
 #    define OS_SORT_CASE_SENSITIVE_DEFAULT 0
+#    define STRCOMP stricmp
+#    define STRNCOMP strnicmp
+#    define MC_ARCH_FLAGS REG_ICASE
      char *get_default_shell (void);
      char *get_default_editor (void);
      int lstat (const char* pathname, struct stat *buffer);
@@ -192,12 +197,12 @@ void tell_parent (int msg);
 #    define PATH_ENV_SEP ':'
 #    define get_default_editor() "vi"
 #    define OS_SORT_CASE_SENSITIVE_DEFAULT 1
+#    define STRCOMP strcmp
+#    define STRNCOMP strncmp
+#    define MC_ARCH_FLAGS 0
 #endif
 
-/* GNOME provides the internationalization code as well */
-#ifndef HAVE_GNOME
-#   define _(x) x
-#endif
+#include "i18n.h"
 
 /* taken from regex.c: */
 /* Jim Meyering writes:
