@@ -59,6 +59,9 @@
 #    endif
 #  include <sys/ioctl.h>
 #endif
+#ifdef __QNX__
+#   include <unix.h>		/* exec*() from <process.h> */
+#endif
 #include "util.h"
 #include "global.h"
 #include "fsusage.h"
@@ -67,6 +70,7 @@
 #include "mad.h"
 #include "dialog.h"		/* message() */
 #include "../vfs/vfs.h"		/* mc_read() */
+#include "x.h"
 
 struct sigaction startup_handler;
 extern struct mount_entry *mount_list;
@@ -74,6 +78,26 @@ extern struct mount_entry *mount_list;
 uid_t current_user_uid;
 user_in_groups *current_user_gid;
 
+int
+max_open_files (void)
+{
+	static int files;
+
+	if (files)
+		return files;
+
+#ifdef HAVE_SYSCONF
+	files = sysconf (_SC_OPEN_MAX);
+	if (files != -1)
+		return files;
+#endif
+#ifdef OPEN_MAX
+	return files = OPEN_MAX;
+#else
+	return files = 256;
+#endif
+}
+
 void init_groups (void)
 {
     int i;
@@ -90,7 +114,7 @@ void init_groups (void)
        return;
     
     setgrent ();
-    while (grp = getgrent ())
+    while ((grp = getgrent ()))
        for (i = 0; grp->gr_mem[i]; i++)
            if (!strcmp (pwd->pw_name,grp->gr_mem[i]))
                {
@@ -179,7 +203,7 @@ char *get_owner (int uid)
     struct passwd *pwd;
     static char ibuf [8];
     char   *name;
-    static uid_last;
+    static int uid_last;
     
     if ((name = i_cache_match (uid, uid_cache, UID_CACHE_SIZE)) != NULL)
 	return name;
@@ -223,12 +247,18 @@ void save_stop_handler (void)
     sigaction (SIGTSTP, NULL, &startup_handler);
 }
 
-int my_system (int as_shell_command, const char *shell, const char *command)
+#ifdef HAVE_GNOME
+#define PORT_HAS_MY_SYSTEM 1
+#endif
+
+#ifndef PORT_HAS_MY_SYSTEM
+int my_system (int flags, const char *shell, const char *command)
 {
     struct sigaction ignore, save_intr, save_quit, save_stop;
     pid_t pid;
     int status = 0;
-
+    int as_shell_command = flags & EXECUTE_AS_SHELL;
+    
     ignore.sa_handler = SIG_IGN;
     sigemptyset (&ignore.sa_mask);
     ignore.sa_flags = 0;
@@ -275,6 +305,7 @@ int my_system (int as_shell_command, const char *shell, const char *command)
 
     return WEXITSTATUS(status);
 }
+#endif
 
 /* Returns a newly allocated string, if directory does not exist, return 0 */
 char *tilde_expand (char *directory)
@@ -343,11 +374,11 @@ static int old_error;		/* File descriptor of old standard error */
 void open_error_pipe (void)
 {
     if (pipe (error_pipe) < 0){
-	message (0, " Warning ", " Pipe failed ");
+	message (0, _(" Warning "), _(" Pipe failed "));
     }
     old_error = dup (2);
     if(old_error < 0 || close(2) || dup (error_pipe[1]) != 2){
-	message (0, " Warning ", " Dup failed ");
+	message (0, _(" Warning "), _(" Dup failed "));
 	close (error_pipe[0]);
 	close (error_pipe[1]);
     }
@@ -361,7 +392,7 @@ void close_error_pipe (int error, char *text)
     int len = 0;
 
     if (error)
-	title = " Error ";
+	title = MSG_ERROR;
     else
 	title = " Warning ";
     if (old_error >= 0){
@@ -413,7 +444,7 @@ void check_error_pipe (void)
 	close (error_pipe[0]);
     }
     if (len > 0)
-        message (0, " Warning ", error);
+        message (0, _(" Warning "), error);
 }
 
 static struct sigaction ignore, save_intr, save_quit, save_stop;
@@ -430,7 +461,8 @@ int mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
     pid_t pid;
 
 #define closepipes() close(pipe0[0]);close(pipe0[1]);close(pipe1[0]);close(pipe1[1])
-
+#define is_a_pipe_fd(f) ((pipe0[0] == f) || (pipe0[1] == f) || (pipe1[0] == f) || (pipe1[1] == f))
+    
     pipe (pipe0); pipe (pipe1);
     ignore.sa_handler = SIG_IGN;
     sigemptyset (&ignore.sa_mask);
@@ -441,10 +473,10 @@ int mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
     sigaction (SIGTSTP, &startup_handler, &save_stop);
 
     switch (pid = fork ()) {
-    	case -1:
+    case -1:
             closepipes ();
 	    return -1;
-	case 0:
+    case 0: {
 	    sigaction (SIGINT, &save_intr, NULL);
 	    sigaction (SIGQUIT, &save_quit, NULL);
 
@@ -452,34 +484,35 @@ int mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
 	    	case -1:
 	    	    closepipes ();
 	    	    exit (1);
-	    	case 0:
-		    {
+	    	case 0: {
 #define MAXARGS 16
-		        int argno;
-			char *args[MAXARGS];
-			va_list ap;
-			int nulldevice;
+		     int argno;
+		     char *args[MAXARGS];
+		     va_list ap;
+		     int nulldevice;
 
-			nulldevice = open ("/dev/null", O_WRONLY);
-		        close (0);
-			dup (pipe0 [0]);
-			close (1);
-			dup (pipe1 [1]);
-			close (2);
-			dup (nulldevice);
-			close (nulldevice);
-			closepipes ();
-			va_start (ap, command);
-			argno = 0;
-			while ((args[argno++] = va_arg(ap, char *)) != NULL)
-			  if (argno == (MAXARGS - 1)) {
-			      args[argno] = NULL;
-			      break;
-			}
-			va_end (ap);
-			execvp (command, args);
-			exit (0);
-		      }
+		     port_shutdown_extra_fds ();
+	    
+		     nulldevice = open ("/dev/null", O_WRONLY);
+		     close (0);
+		     dup (pipe0 [0]);
+		     close (1);
+		     dup (pipe1 [1]);
+		     close (2);
+		     dup (nulldevice);
+		     close (nulldevice);
+		     closepipes ();
+		     va_start (ap, command);
+		     argno = 0;
+		     while ((args[argno++] = va_arg(ap, char *)) != NULL)
+		         if (argno == (MAXARGS - 1)) {
+			     args[argno] = NULL;
+			     break;
+		         }
+		     va_end (ap);
+		     execvp (command, args);
+		     exit (0);
+		}
 	    	default:
 	    	    {
 	    	    	char buffer [8192];
@@ -502,12 +535,15 @@ int mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
 	   		while (waitpid (pid, &i, 0) < 0)
 			    if (errno != EINTR)
 				break;
+
+			port_shutdown_extra_fds ();
 	   		exit (i);
 	    	    }
 	    }
-	default:
-	    *the_pid = pid;
-	    break;
+    }
+    default:
+	*the_pid = pid;
+	break;
     }
     close (pipe0 [0]);
     close (pipe0 [1]);
@@ -796,14 +832,6 @@ int gettimeofday (struct timeval *tp, void *tzp)
 }
 #endif
 
-int
-errno_dir_not_empty (int err)
-{
-    if (err == ENOTEMPTY || err == EEXIST)
-	return 1;
-    return 0;
-}
-
 #ifdef SCO_FLAVOR
 /* Define this only for SCO */
 #ifdef USE_NETCODE
