@@ -23,7 +23,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-/* "$Id: widget.c,v 1.3 1995/02/21 19:07:43 miguel Exp $" */
+/* "$Id: widget.c,v 1.17 1998/05/24 22:22:05 unammx Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -58,10 +58,18 @@
 #   define x_redefine_label(a,b)
 #endif
 
-#ifndef HAVE_TK
+#ifndef PORT_HAS_DESTROY_CMD
 #   define x_destroy_cmd(w)
 #endif
 
+#ifndef PORT_HAS_RADIO_FOCUS_ITEM
+#   define x_radio_focus_item(r)
+#endif
+
+#ifndef PORT_HAS_RADIO_TOGGLE
+#   define x_radio_toggle
+#endif
+
 static int button_event (Gpm_Event *event, WButton *b);
 
 int quote = 0;
@@ -78,8 +86,7 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 	return x_create_button (h, h->wdata, b);
 #ifndef	HAVE_XVIEW
     case WIDGET_HOTKEY:
-        if (b->hotkey == Par ||
-	    (b->hotkey >= 'a' && b->hotkey <= 'z' && b->hotkey-32 == Par)){
+        if (b->hotkey == Par ||  toupper(b->hotkey) == Par){
 	    button_callback (h, b, WIDGET_KEY, ' '); /* to make action */
 	    return 1;
 	} else
@@ -92,8 +99,8 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 	if (b->callback)
 	    stop = (*b->callback)(b->action, b->callback_data);
 	if (!b->callback || stop){
-	    h->running = 0;
 	    h->ret_value = b->action;
+	    dlg_stop (h);
 	}
 	return 1;
 
@@ -110,7 +117,22 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 #else
 	
     case WIDGET_CURSOR:
-	widget_move (&b->widget, 0, b->hotpos);
+	switch (b->flags) {
+	    case DEFPUSH_BUTTON:
+		off = 3;
+		break;
+	    case NORMAL_BUTTON:
+		off = 2;
+		break;
+	    case NARROW_BUTTON:
+		off = 1;
+		break;
+	    case HIDDEN_BUTTON:
+	    default:
+		off = 0;
+		break;
+        }
+	widget_move (&b->widget, 0, b->hotpos + off);
 	return 1;
 
     case WIDGET_UNFOCUS:
@@ -150,9 +172,13 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 	if (b->hotpos >= 0){
 	    attrset ((b->selected) ? HOT_FOCUSC : HOT_NORMALC);
 	    widget_move (&b->widget, 0, b->hotpos+off);
-	    addch (b->text [b->hotpos]);
+	    addch ((unsigned char)b->text [b->hotpos]);
         }
-	return 1;	                                      
+	if (Msg == WIDGET_FOCUS)
+	    break;
+	else
+	    return 1;
+	break;
 #endif	
 #endif /* !HAVE_XVIEW */
     }	
@@ -208,12 +234,29 @@ button_len (const char *text, unsigned int flags)
 #endif
 }
 
+/*
+ * Assuming that button text is malloc'ed, we may safely change it
+ * (as opposed to statically allocated); from other hand, excluding &
+ * and shifting data past it to the left results to one unused byte.
+ * This does not harm though :)
+ */
+void
+button_scan_hotkey(WButton* b)
+{
+    char* cp = strchr (b->text, '&');
+
+    if (cp != NULL && cp[1] != '\0'){
+	strcpy (cp, cp+1);
+	b->hotkey = tolower (*cp);
+	b->hotpos = cp - b->text;
+    }
+}
+
 WButton *
 button_new (int y, int x, int action, int flags, char *text, 
 	    int (*callback)(int, void *), void *callback_data, char *tkname)
 {
     WButton *b = xmalloc (sizeof (WButton), "new_button");
-    char *s, *t;
 
     init_widget (&b->widget, y, x, 1, button_len (text, flags),
 		 (callback_fn) button_callback,
@@ -228,21 +271,8 @@ button_new (int y, int x, int action, int flags, char *text,
     widget_want_hotkey (b->widget, 1);
     b->hotkey = 0;
     b->hotpos = -1;
-    
-    /* Scan for the hotkey */
-    for (s = text, t = b->text; *s; s++, t++){
-	if (*s != '&'){
-	    *t = *s;
-	    continue;
-	}
-	s++;
-	*t = *s;
-	if (*s){
-	    b->hotkey = tolower (*s);
-	    b->hotpos = t - b->text;
-	}
-    }    
-    *t = 0;
+
+    button_scan_hotkey(b);
     return b;
 }
 
@@ -252,8 +282,9 @@ button_set_text (WButton *b, char *text)
     free (b->text);
     b->text = strdup (text);
     b->widget.cols = button_len (text, b->flags);
+    button_scan_hotkey(b);
 #ifdef HAVE_X
-    x_button_set (b, text);
+    x_button_set (b, b->text);
 #else
     dlg_redraw (b->widget.parent);
 #endif
@@ -274,13 +305,16 @@ radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
 	
 #ifndef HAVE_XVIEW
     case WIDGET_HOTKEY:
-	if (r->upper_letter_is_hotkey){
-	    int  i;
-	    char *s;
+	{
+	    int i, lp = tolower(Par);
+	    char *cp;
 	    
 	    for (i = 0; i < r->count; i++){
-		for (s = r->texts [i]; *s; s++){
-		    if (!(*s >= 'A' && *s <= 'Z' && (0x20|*s) == (0x20|Par)))
+		cp = strchr (r->texts [i], '&');
+		if (cp != NULL && cp[1] != '\0'){
+		    int c = tolower (cp [1]);
+		    
+		    if (c != lp)
 			continue;
 		    r->pos = i;
 		    radio_callback (h, r, WIDGET_KEY, ' '); /* Take action */
@@ -296,12 +330,14 @@ radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
 	    r->sel = r->pos;
     	    (*h->callback) (h, h->current->dlg_id, DLG_ACTION);
 	    radio_callback (h, r, WIDGET_FOCUS, ' ');
+	    x_radio_toggle (r);
 	    return 1;
 
 	case KEY_UP:
 	case KEY_LEFT:
 	    if (r->pos > 0){
 		r->pos--;
+		x_radio_focus_item (r);
 		return 1;
 	    }
 	    return 0;
@@ -310,21 +346,17 @@ radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
 	case KEY_RIGHT:
 	    if (r->count - 1 > r->pos) {
 		r->pos++;
+		x_radio_focus_item (r);
 		return 1;
 	    }
 	}
 	return 0;
-	
-#ifdef HAVE_TK
-	/* We manage a group of widgets, while Tk manages independent
-	 * ones, so we have to do the focusing manually
-	 */
+
+#ifdef HAVE_X
     case WIDGET_FOCUS:
     case WIDGET_CURSOR:
-	tk_evalf ("focus %s.%d", (char *)(r->widget.wdata)+1, r->pos);
-	/* Do not call default_proc: we did the tk focus command */
-	return 1;
-
+	    x_radio_focus_item (r);
+	    return 1;
 #endif
 #endif
 	
@@ -339,23 +371,23 @@ radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
     case WIDGET_FOCUS:
     case WIDGET_DRAW:
 	for (i = 0; i < r->count; i++){
+		register unsigned char* cp;
 	    attrset ((i==r->pos && Msg==WIDGET_FOCUS) ? FOCUSC :NORMALC);
 	    widget_move (&r->widget, i, 0);
-	    printw ("(%c) %s", (r->sel == i) ? '*' : ' ', r->texts[i]);
 
-	    /* Draw the hotkey */
-	    if (r->upper_letter_is_hotkey){
-		char *t = r->texts [i];
-
-		while (*t && !(*t >= 'A' && *t <= 'Z'))
-		    t++;
-		if (*t){
-		    widget_move (&r->widget, i, 4+t - r->texts [i]);
-		    attrset ((i==r->pos && Msg==WIDGET_FOCUS)
-			     ? HOT_FOCUSC :HOT_NORMALC);
-		    addch (*t);
+		printw("(%c) ", (r->sel == i) ? '*' : ' ');
+		for (cp =  r->texts[i]; *cp; cp++)
+		{
+			if (*cp == '&')
+			{
+				attrset ((i==r->pos && Msg==WIDGET_FOCUS) 
+					? HOT_FOCUSC : HOT_NORMALC);
+				addch(*++cp);
+				attrset ((i==r->pos && Msg==WIDGET_FOCUS) ? FOCUSC : NORMALC);
+			}
+			else
+				addch(*cp);
 		}
-	    }
 	}
 	return 1;
 	break;
@@ -466,7 +498,7 @@ check_callback (Dlg_head *h, WCheck *c, int Msg, int Par)
 	if (c->hotpos >= 0){
 	    attrset ((Msg == WIDGET_FOCUS) ? HOT_FOCUSC : HOT_NORMALC);
 	    widget_move (&c->widget, 0, + c->hotpos+4);
-	    addch (c->text [c->hotpos]);
+	    addch ((unsigned char)c->text [c->hotpos]);
 	}
 	return 1;
 #endif /* !HAVE_X */
@@ -791,7 +823,7 @@ static void draw_history_button (WInput * in)
 static char *kill_buffer = 0;
 
 void
-update_input (WInput *in)
+update_input (WInput *in, int clear_first)
 {
 #ifndef HAVE_XVIEW
     int has_history = 0;
@@ -816,7 +848,10 @@ update_input (WInput *in)
     /* Adjust the mark */
     if (in->mark > buf_len)
 	in->mark = buf_len;
+    
 #ifdef HAVE_X
+    if (clear_first && in->first)
+	    in->first = -1;
     x_update_input (in);
 #else
 
@@ -838,9 +873,11 @@ update_input (WInput *in)
 	addch (c);
     }
     widget_move (&in->widget, 0, in->point - in->first_shown);
+
+    if (clear_first)
+	    in->first = 0;
 #endif
-    
-    in->first = 0;
+
 #endif
 }
 
@@ -849,7 +886,7 @@ winput_set_origin (WInput *in, int x, int field_len)
 {
     in->widget.x    = x;
     in->field_len = in->widget.cols = field_len;
-    update_input (in);
+    update_input (in, 0);
 }
 
 /* {{{ history saving and loading */
@@ -899,6 +936,7 @@ Hist *history_get (char *input_name)
     return new;			/* return pointer to last entry in list */
 }
 
+#ifdef PORT_WIDGET_WANTS_HISTORY
 void history_put (char *input_name, Hist *h)
 {
     int i;
@@ -943,6 +981,11 @@ void history_put (char *input_name, Hist *h)
     }
     free (profile);
 }
+#else
+void history_put (char *input_name, Hist *h)
+{
+}
+#endif
 
 /* }}} history saving and loading */
 
@@ -970,18 +1013,18 @@ int history_callback (Dlg_head * h, int Par, int Msg)
 
 static inline int listbox_fwd (WListbox *l);
 
-static void show_hist (WInput * in)
+char *show_hist (Hist *history, int widget_x, int widget_y)
 {
     Hist *hi, *z;
     int maxlen = strlen(history_title), i, count = 0;
     int x, y, w, h;
-    char *q;
+    char *q, *r = 0;
     Dlg_head *query_dlg;
     WListbox *query_list;
 
-    z = in->history;
+    z = history;
     if (!z)
-		return;
+		return 0;
 
     while (z->prev)		/* goto first */
 		z = z->prev;
@@ -993,7 +1036,7 @@ static void show_hist (WInput * in)
 		hi = hi->next;
     }
 
-       y = in->widget.y;
+       y = widget_y;
 	h = count + 2;
 	if (h <= y || y > LINES - 6)
 	{
@@ -1006,7 +1049,7 @@ static void show_hist (WInput * in)
 		h = min(h, LINES - y);
 	}
 
-	x = in->widget.x - 2;
+	x = widget_x - 2;
 	if ((w = maxlen + 4) + x > COLS)
 	{
 		w = min(w,COLS);
@@ -1018,7 +1061,7 @@ static void show_hist (WInput * in)
     query_list = listbox_new (1, 1, w - 2, h - 2, listbox_finish, 0, NULL);
     add_widget (query_dlg, query_list);
     hi = z;
-    if (y < in->widget.y) {
+    if (y < widget_y) {
 		while (hi) {		/* traverse */
 		    listbox_add_item (query_list, 0, 0, hi->text, NULL);
 	    	hi = hi->next;
@@ -1037,11 +1080,21 @@ static void show_hist (WInput * in)
     if (query_dlg->ret_value != B_CANCEL) {
 		listbox_get_current (query_list, &q, NULL);
 		if (q)
-	    	assign_text (in, q);
+		    r = strdup (q);
     }
     destroy_dlg (query_dlg);
+    return r;
 }
 
+static void do_show_hist (WInput * in)
+{
+    char *r;
+    r = show_hist (in->history, in->widget.x, in->widget.y);
+    if (r) {
+	assign_text (in, r);
+	free (r);
+    }
+}
 
 /* }}} history display */
 
@@ -1057,7 +1110,7 @@ input_destroy (WInput *in)
     if (in->history){
 	Hist *current, *old;
 
-	if (!in->is_password)	/* don't save passwords ;-) */
+	if (!in->is_password && PORT_WIDGET_WANTS_HISTORY)	/* don't save passwords ;-) */
 	    history_put (in->history_name, in->history);
 
 	current = in->history;
@@ -1089,7 +1142,7 @@ void
 input_enable_update (WInput *in)
 {
     in->disable_update--;
-    update_input (in);
+    update_input (in, 0);
 }
 
 int
@@ -1129,7 +1182,7 @@ new_input (WInput *in)
     in->point = 0;
     in->mark = 0;
     free_completions (in);
-    update_input (in);
+    update_input (in, 0);
 }
 
 static int
@@ -1420,41 +1473,41 @@ static struct {
     void (*fn)(WInput *in);
 } input_map [] = {
     /* Motion */
-    { XCTRL('a'),   beginning_of_line },
-    { KEY_HOME,	  beginning_of_line },
-    { KEY_A1,	  beginning_of_line },
-    { XCTRL('e'),   end_of_line },
-    { KEY_END,   	  end_of_line },
-    { KEY_C1,   	  end_of_line },
-    { KEY_LEFT,     key_left },
-    { XCTRL('b'),   backward_char },
-    { ALT('b'),     backward_word },
-    { KEY_RIGHT,    key_right },
-    { XCTRL('f'),   forward_char },
-    { ALT('f'),     forward_word },
-
-    /* Editing */
-    { 0177,         backward_delete },
-    { KEY_BACKSPACE,backward_delete },
-    { XCTRL('h'),   backward_delete },
-    { KEY_DC,       delete_char },
-    { XCTRL('d'),   delete_char },
-    { ALT('d'),     kill_word },
+    { XCTRL('a'),         beginning_of_line },
+    { KEY_HOME,	          beginning_of_line },
+    { KEY_A1,	          beginning_of_line },
+    { XCTRL('e'),         end_of_line },
+    { KEY_END,            end_of_line },
+    { KEY_C1,             end_of_line },
+    { KEY_LEFT,           key_left },
+    { XCTRL('b'),         backward_char },
+    { ALT('b'),           backward_word },
+    { KEY_RIGHT,          key_right },
+    { XCTRL('f'),         forward_char },
+    { ALT('f'),           forward_word },
+		          
+    /* Editing */         
+    { 0177,               backward_delete },
+    { KEY_BACKSPACE,      backward_delete },
+    { XCTRL('h'),         backward_delete },
+    { KEY_DC,             delete_char },
+    { XCTRL('d'),         delete_char },
+    { ALT('d'),           kill_word },
     { ALT(KEY_BACKSPACE), back_kill_word },
-    { ALT(XCTRL('h')), back_kill_word },
-    { ALT(127),     back_kill_word },
+    { ALT(XCTRL('h')),    back_kill_word },
+    { ALT(127),           back_kill_word },
     
     /* Region manipulation */
-    { 0,            set_mark },
-    { XCTRL('w'),   kill_region },
-    { ALT('w'),     kill_save },
-    { XCTRL('y'),   yank },
-    { XCTRL('k'),   kill_line },
-    
-    /* History */
-    { ALT('p'),     hist_prev },
-    { ALT('n'),     hist_next },
-    { ALT('h'),     show_hist },
+    { 0,              	  set_mark },
+    { XCTRL('w'),     	  kill_region },
+    { ALT('w'),       	  kill_save },
+    { XCTRL('y'),     	  yank },
+    { XCTRL('k'),     	  kill_line },
+    		      	  
+    /* History */     	  
+    { ALT('p'),       	  hist_prev },
+    { ALT('n'),       	  hist_next },
+    { ALT('h'),       	  do_show_hist },
     
     /* Completion */
     { ALT('\t'),	  complete },
@@ -1479,6 +1532,22 @@ is_in_input_map (WInput *in, int c_code)
     return 0;
 }
 
+#ifdef PORT_WINPUT_DELETES_MARKED
+static void
+port_region_marked_for_delete (WInput *in)
+{
+	kill_region (in);
+}
+#else
+static void
+port_region_marked_for_delete (WInput *in)
+{
+    *in->buffer = 0;
+    in->point = 0;
+    in->first = 0;
+}
+#endif
+
 int
 handle_char (WInput *in, int c_code)
 {
@@ -1493,7 +1562,7 @@ handle_char (WInput *in, int c_code)
     if (quote){
     	free_completions (in);
 	v = insert_char (in, c_code);
-	update_input (in);
+	update_input (in, 1);
 	quote = 0;
 	return v;
     }
@@ -1511,15 +1580,14 @@ handle_char (WInput *in, int c_code)
 	if (c_code > 255 || !is_printable (c_code))
 	    return 0;
 	if (in->first){
-	    *in->buffer = 0;
-	    in->point = 0;
+	    port_region_marked_for_delete (in);
 	}
     	free_completions (in);
 	v = insert_char (in, c_code);
 	in->inserted_one = c_code;
     }
     if (!disable_update)
-	update_input (in);
+	update_input (in, 1);
     return v;
 }
 
@@ -1533,7 +1601,7 @@ stuff (WInput *in, char *text, int insert_extra_space)
     if (insert_extra_space)
 	handle_char (in, ' ');
     input_enable_update (in);
-    update_input (in);
+    update_input (in, 1);
 }
 
 void
@@ -1544,7 +1612,7 @@ input_set_point (WInput *in, int pos)
     if (pos != in->point)
     	free_completions (in);
     in->point = pos;
-    update_input (in);
+    update_input (in, 1);
 }
 
 int input_event (Gpm_Event *event, WInput *b);
@@ -1552,8 +1620,6 @@ int input_event (Gpm_Event *event, WInput *b);
 static int
 input_callback (Dlg_head *h, WInput *in, int Msg, int Par)
 {
-    int    t;
-
     switch (Msg){
     case WIDGET_INIT:
 	return x_create_input (h, h->wdata, in);
@@ -1582,10 +1648,7 @@ input_callback (Dlg_head *h, WInput *in, int Msg, int Par)
     case WIDGET_FOCUS:
     case WIDGET_UNFOCUS:	
     case WIDGET_DRAW:
-	/* Very ugly hack */
-	t = in->first;
-	update_input (in);
-	in->first = t;
+	update_input (in, 0);
 	break;
 #endif /* !HAVE_XVIEW */
 #ifndef HAVE_X
@@ -1608,8 +1671,8 @@ input_event (Gpm_Event *event, WInput *in)
 	dlg_select_widget (in->widget.parent, in); 
 
 	if (event->x >= in->field_len - HISTORY_BUTTON_WIDTH + 1 && should_show_history_button (in)) {
-	    show_hist (in);
-	    update_input (in);
+	    do_show_hist (in);
+	    update_input (in, 1);
 	} else {
 	    in->point = strlen (in->buffer);
 	    if (event->x - in->first_shown - 1 < in->point)
@@ -1617,13 +1680,13 @@ input_event (Gpm_Event *event, WInput *in)
 	    if (in->point < 0)
 		in->point = 0;
 
-	    update_input (in);
+	    update_input (in, 1);
 	} 
     } 
 #endif
     return MOU_NORMAL; 
 } 
- 
+
 WInput *
 input_new (int y, int x, int color, int len, char *def_text, char *tkname)
 {
@@ -1637,11 +1700,12 @@ input_new (int y, int x, int color, int len, char *def_text, char *tkname)
     /* history setup */
     in->history = NULL;
     in->history_name = 0;
-    if (tkname)
+    if (tkname && PORT_WIDGET_WANTS_HISTORY){
 	if (*tkname) {
 	    in->history_name = strdup (tkname);
 	    in->history = history_get (tkname);
 	}
+    }
     if (def_text == INPUT_LAST_TEXT) {
 	def_text = "";
 	if (in->history)
@@ -2109,8 +2173,8 @@ listbox_event (Gpm_Event *event, WListbox *l)
 	    break;
 
 	case listbox_finish:
-	    h->running   = 0;
 	    h->ret_value = B_ENTER;
+	    dlg_stop (h);
 	    return MOU_ENDLOOP;
 
 	case listbox_cback:
