@@ -32,7 +32,8 @@
 #include "panel.h"
 
 /* Create the Tk widget */
-void x_create_info (Dlg_head *h, widget_data parent, WInfo *info)
+void
+x_create_info (Dlg_head *h, widget_data parent, WInfo *info)
 {
     char *cmd;
     widget_data container = info->widget.wcontainer;
@@ -43,7 +44,8 @@ void x_create_info (Dlg_head *h, widget_data parent, WInfo *info)
 }
 
 /* Updates the information on the Tk widgets */
-void x_show_info (WInfo *info, struct my_statfs *s, struct stat *b)
+void
+x_show_info (WInfo *info, struct my_statfs *s, struct stat *b)
 {
     char bsize [17];
     char avail_buf [17], total_buf [17];
