@@ -80,3 +80,42 @@ int lookup_keysym (char *s)
 }
 
 
+#if TCL_MAJOR_VERSION > 7
+/* Tcl 8.xx support */
+
+void add_select_channel (int fd, select_fn callback, void *info)
+{
+    Tcl_CreateFileHandler (fd, TCL_READABLE, (Tcl_FileProc *) callback, 0);
+}
+
+void delete_select_channel (int fd)
+{
+    Tcl_DeleteFileHandler (fd);
+}
+
+#else
+/* Tcl 7.xx support */
+   
+void add_select_channel (int fd, select_fn callback, void *info)
+{
+    Tcl_File handle;
+
+    handle = Tcl_GetFile ((ClientData) fd, TCL_UNIX_FD);
+    Tcl_CreateFileHandler (handle, TCL_READABLE, (Tcl_FileProc *) callback, 0);
+}
+
+void delete_select_channel (int fd)
+{
+    Tcl_File handle;
+
+    handle = Tcl_GetFile ((ClientData) fd, TCL_UNIX_FD);
+    Tcl_DeleteFileHandler (handle);
+    Tcl_FreeFile (handle);
+}
+#endif
+
+int
+mi_getch ()
+{
+	return tk_getch();
+}
