@@ -329,6 +329,11 @@ x_focus_widget (Widget_Item *p)
     tk_evalf ("focus %s", wname+1);
 }
 
+void
+x_unfocus_widget (Widget_Item *p)
+{
+}
+
 /* Setup done before using tkrundlg_event */
 void
 x_init_dlg (Dlg_head *h)
@@ -448,6 +453,12 @@ tk_dispatch_all (void)
 	;
 }
 
+void
+x_destroy_dlg_start (Dlg_head *h)
+{
+	/* nothing */
+}
+
 void
 x_destroy_dlg (Dlg_head *h)
 {
@@ -470,3 +481,8 @@ edition_pre_exec (void)
     if (iconify_on_exec)
 	tk_evalf ("wm iconify .");
 }
+
+int
+try_alloc_color_pair (char *fg, char *bg)
+{
+}
