@@ -32,8 +32,6 @@
 #include "cmd.h"		/* copy_cmd, ren_cmd, delete_cmd, ... */
 #include "tkscreen.h"
 
-void do_enter (WPanel *panel);
-
 /* These two variables keep track of the dimensions of a character in the
  * panel listing.
  */
@@ -554,3 +552,14 @@ panel_update_cols (Widget *widget, int frame_size)
 {
 	/* nothing */
 }
+
+void
+display_mini_info (WPanel *panel)
+{
+	/* FIXME: implement */
+}
+
+void
+x_panel_destroy (WPanel *panel)
+{
+}
