@@ -44,12 +44,14 @@
 #include "vfs.h"
 #include "extfs.h"
 
-static struct extfs_archive *first_archive = NULL;
+static struct extfs_entry *
+extfs_find_entry (struct extfs_entry *dir, char *name, int make_dirs, int make_file);
+
+static struct extfs_archive *first_archive = NULL;;
 static int extfserrno = 0;
 static struct stat hstat;		/* Stat struct corresponding */
 static char *current_file_name, *current_link_name;
 static char *extfs_current_dir;
-static struct extfs_entry *extfs_find_entry (struct extfs_entry *dir, char *name, int make_dirs, int make_file);
 
 enum {
     EFS_ARG_EMPTY,
@@ -758,7 +760,7 @@ static void *extfs_open (char *file, int flags, int mode)
                             " ", q, " ", entry->inode->local_filename, 0);
 	free (q);
 	free (archive_name);
-        if (my_system (1, shell, cmd) && !do_create){
+        if (my_system (EXECUTE_AS_SHELL, shell, cmd) && !do_create){
             free (entry->inode->local_filename);
             entry->inode->local_filename = NULL;
             free (cmd);
@@ -821,7 +823,7 @@ static int extfs_close (void *data)
 			    file->entry->inode->local_filename, 0);
 	free (archive_name);
 	free (file_name);
-	if (my_system (1, shell, cmd))
+	if (my_system (EXECUTE_AS_SHELL, shell, cmd))
 	    errno_code = EIO;
 	free (cmd);
         {
