@@ -386,10 +386,19 @@ ftpfs_connection_destructor(void *data)
     ftpfs_free_bucket (data);
 }
 
+/* some defines only used by changetype */
+/* These two are valid values for the second parameter */
+#define TYPE_ASCII    0
+#define TYPE_BINARY   1
+
+/* This one is only used to initialize bucket->isbinary, don't use it as
+   second parameter to changetype. */
+#define TYPE_UNKNOWN -1 
+
 static int
 changetype (struct ftpfs_connection *bucket, int binary)
 {
-    if (binary || binary != bucket->isbinary) {
+    if (binary != bucket->isbinary) {
         if (command (bucket, WAIT_REPLY, "TYPE %c", binary ? 'I' : 'A') != COMPLETE) {
 	    ftpfserrno = EIO;
             return -1;
@@ -417,7 +426,8 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
     char *op;
     char *name;			/* login user name */
     int  anon = 0;
-    
+
+    bucket->isbinary = TYPE_UNKNOWN;    
     if (netrcpass)
         op = strdup (netrcpass);
     else {
@@ -431,7 +441,7 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 	    if (!bucket->password){
 		p = copy_strings (" FTP: Password required for ", quser(bucket), 
 				  " ", NULL);
-		op = input_dialog (p, "Password:", "");
+		op = input_dialog (p, _("Password:"), "");
 		free (p);
 		if (op == NULL) {
 		    ftpfserrno = EPERM;
@@ -465,7 +475,7 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 	    wipe_password (proxypass);
 	p = copy_strings(" Proxy: Password required for ", proxyname, " ",
 			 NULL);
-	proxypass = input_dialog (p, "Password:", "");
+	proxypass = input_dialog (p, _("Password:"), "");
 	free(p);
 	if (proxypass == NULL) {
 	    ftpfserrno = EPERM;
@@ -824,7 +834,7 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
     bucket->password = 0;
     bucket->use_passive_connection = ftpfs_use_passive_connections | source_route;
     bucket->use_source_route = source_route;
-    bucket->isbinary = -1;
+    bucket->isbinary = TYPE_UNKNOWN;
 
     /* We do not want to use the passive if we are using proxies */
     if (bucket->use_proxy)
@@ -1243,11 +1253,11 @@ resolve_symlink(struct ftpfs_connection *bucket, struct ftpfs_dir *dir)
             print_vfs_message("ftpfs: CWD failed.");
 	    return;
         }
-        sock = open_data_connection (bucket, "LIST -lLa", ".", 0);
+        sock = open_data_connection (bucket, "LIST -lLa", ".", TYPE_ASCII);
     }
     else
         sock = open_data_connection (bucket, "LIST -lLa", 
-                                     dir->remote_path, 0);
+                                     dir->remote_path, TYPE_ASCII);
 
     if (sock == -1) {
 	print_vfs_message("ftpfs: couldn't resolve symlink");
@@ -1272,6 +1282,10 @@ resolve_symlink(struct ftpfs_connection *bucket, struct ftpfs_dir *dir)
 	while (1) {
 	    if (fgets (buffer, sizeof (buffer), fp) == NULL)
 		goto done;
+	    if (ftpfs_debug_server_dialog){
+		fputs (buffer, ftpfs_logfile);
+	        fflush (ftpfs_logfile);
+	    }
 	    if (parse_ls_lga (buffer, &s, &filename, NULL)) {
 		int r = strcmp(fe->name, filename);
 		free(filename);
@@ -1396,11 +1410,11 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
     dcache->count = 1;
 
     if (has_spaces)
-        sock = open_data_connection (bucket, "LIST -la", ".", 0);
+        sock = open_data_connection (bucket, "LIST -la", ".", TYPE_ASCII);
     else {
 	char *path = copy_strings (remote_path, PATH_SEP_STR, ".", (char *) 0);
 	
-	sock = open_data_connection (bucket, "LIST -la", path, 0);
+	sock = open_data_connection (bucket, "LIST -la", path, TYPE_ASCII);
 	free (path);
     }
 
@@ -1434,6 +1448,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 
 	if (ftpfs_debug_server_dialog){
 	    fputs (buffer, ftpfs_logfile);
+            fputs ("\n", ftpfs_logfile);
 	    fflush (ftpfs_logfile);
 	}
 	if (buffer [0] == 0 && eof)
@@ -1529,7 +1544,7 @@ store_file(struct ftpentry *fe)
 	return 0;
     }
     fstat(local_handle, &s);
-    sock = open_data_connection(fe->bucket, "STOR", fe->remote_filename, 1);
+    sock = open_data_connection(fe->bucket, "STOR", fe->remote_filename, TYPE_BINARY);
     if (sock < 0) {
 	close(local_handle);
 	return 0;
@@ -1606,7 +1621,7 @@ int retrieve_file_start(struct ftpentry *fe)
 	ftpfserrno = ENOMEM;
 	return 0;
     }
-    remotesock = open_data_connection(fe->bucket, "RETR", fe->remote_filename, 1);
+    remotesock = open_data_connection(fe->bucket, "RETR", fe->remote_filename, TYPE_BINARY);
     if (remotesock == -1) {
 	ftpfserrno = EACCES;
 	free (fe->local_filename);
@@ -1739,14 +1754,14 @@ int retrieve_file(struct ftpentry *fe)
 	ftpfserrno = ENOMEM;
 	return 0;
     }
-    local_handle = open(fe->local_filename, O_RDWR | O_CREAT | O_TRUNC, 0600);
+    local_handle = open(fe->local_filename, O_RDWR | O_CREAT | O_TRUNC | O_EXCL, 0600);
     if (local_handle == -1) {
 	ftpfserrno = EIO;
 	free(fe->local_filename);
 	fe->local_filename = NULL;
 	return 0;
     }
-    sock = open_data_connection(fe->bucket, "RETR", fe->remote_filename, 1);
+    sock = open_data_connection(fe->bucket, "RETR", fe->remote_filename, TYPE_BINARY);
     if (sock == -1) {
 	ftpfserrno = EACCES;
 	goto error_3;
@@ -1901,8 +1916,7 @@ _get_file_entry(struct ftpfs_connection *bucket, char *file_name,
 			    ftpfserrno = ENOMEM;
 			    return NULL;
 			}
-			handle = open(ent->local_filename, O_CREAT | O_TRUNC | 
-				 O_RDWR, 0600);
+			handle = open(ent->local_filename, O_CREAT | O_TRUNC | O_RDWR | O_EXCL, 0600);
 			if (handle < 0) {
 			    ftpfserrno = EIO;
 			    return NULL;
@@ -2368,14 +2382,11 @@ static int ftpfs_rename (char *path1, char *path2)
 static int
 __ftpfs_chdir (struct ftpfs_connection *bucket ,char *remote_path)
 {
-#ifdef ARE_WE_SURE_WE_DONT_NEED_THIS_P
     int r;
-#endif
     
-    if (ftpfs_same_dir (remote_path, bucket))
+    if (!bucket->cwd_defered && ftpfs_same_dir (remote_path, bucket))
 	return COMPLETE;
 
-#ifdef ARE_WE_SURE_WE_DONT_NEED_THIS_P
     r = command (bucket, WAIT_REPLY, "CWD %s", remote_path);
     if (r != COMPLETE) {
 	ftpfserrno = EIO;
@@ -2383,35 +2394,25 @@ __ftpfs_chdir (struct ftpfs_connection *bucket ,char *remote_path)
 	if (qcdir(bucket))
 	    free(qcdir(bucket));
 	qcdir(bucket) = strdup (remote_path);
+	bucket->cwd_defered = 0;
     }
     return r;
-#else
-    
-    if (qcdir(bucket))
-	free(qcdir(bucket));
-    qcdir(bucket) = strdup (remote_path);
-    
-    return COMPLETE;
-#endif
 }
 
 static int ftpfs_chdir (char *path)
 {
     char *remote_path;
     struct ftpfs_connection *bucket;
-    int r;
 
- retry:
     if (!(remote_path = ftpfs_get_path(&bucket, path)))
 	return -1;
-    got_sigpipe = 0;
-    r = __ftpfs_chdir (bucket, remote_path);
-    free (remote_path);
-    if (got_sigpipe)
-	goto retry;
+    if (qcdir(bucket))
+        free(qcdir(bucket));
+    qcdir(bucket) = remote_path;
+    bucket->cwd_defered = 1;
     
     vfs_add_noncurrent_stamps (&ftpfs_vfs_ops, (vfsid) bucket, NULL);
-    return r == COMPLETE ? 0 : -1;
+    return 0;
 }
 
 static int ftpfs_lseek (void *data, off_t offset, int whence)
