@@ -45,6 +45,8 @@ struct ftpfs_connection {
     int use_source_route;
     int use_passive_connection;
     int isbinary;
+    int cwd_defered;  /* current_directory was changed but CWD command hasn't
+                         been sent yet */
 };
 
 #define qhost(b) (b)->host
@@ -67,8 +69,8 @@ struct ftpfs_connection {
 
 extern char *ftpfs_anonymous_passwd;
 extern char *ftpfs_proxy_host;
-extern ftpfs_directory_timeout;
-extern ftpfs_always_use_proxy;
+extern int ftpfs_directory_timeout;
+extern int ftpfs_always_use_proxy;
 
 void ftpfs_init_passwd ();
 #endif
