@@ -45,6 +45,8 @@
 #include <malloc.h>
 #include <fcntl.h>
 #include <stdlib.h>
+/* asm/types.h defines its own umode_t :-( */
+#undef umode_t
 #include <linux/ext2_fs.h>
 #include <ext2fs/ext2fs.h>
 #include <ctype.h>
