@@ -1156,9 +1156,9 @@ char *vfs_get_current_dir (void)
 static void vfs_setup_wd (void)
 {
     current_dir = xmalloc (MC_MAXPATHLEN, "Current directory");
-    get_current_wd (current_dir, MC_MAXPATHLEN);
-    if (errno != 0)
-	current_dir [0] = 0;
+    if (!get_current_wd (current_dir, MC_MAXPATHLEN))
+	if (errno != 0)
+	     current_dir [0] = 0;
     current_dir [MC_MAXPATHLEN - 1] = 0;
     last_current_dir = current_dir;
     return;
@@ -1465,11 +1465,12 @@ void vfs_fill_names (void (*func)(char *))
 
 static char *columns [MAXCOLS];	/* Points to the string in column n */
 static int   column_ptr [MAXCOLS]; /* Index from 0 to the starting positions of the columns */
-static int  numcols;
 
 static int split_text (char *p)
 {
     char *original = p;
+    int  numcols;
+
 
     for (numcols = 0; *p && numcols < MAXCOLS; numcols++){
 	while (*p == ' ' || *p == '\r' || *p == '\n'){
@@ -1495,6 +1496,7 @@ static int is_num (int idx)
 int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 {
     int idx, idx2, num_cols, isconc = 0;
+    int i;
     long l;
     struct tm tim;
     int extfs_format_date = 0;
@@ -1753,15 +1755,36 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 #ifdef HAVE_ST_BLOCKS
     s->st_blocks = (s->st_size + 511) / 512;
 #endif
+
+    for (i = idx + 1, idx2 = 0; i < num_cols; i++ ) 
+	if (strcmp (columns [i], "->") == 0) {
+	    idx2 = i;
+	    break;
+	}
     
     if (((S_ISLNK (s->st_mode) || 
-        (numcols == idx + 3 && s->st_nlink > 1))) /* Maybe a hardlink? (in extfs) */
-        && (!strcmp (columns [idx+1], "->"))){
-	if (filename)
-	    *filename = strdup (columns [idx]);
-	if (linkname)
-	    *linkname = strdup (columns [idx+2]);
-	idx += 2;
+        (num_cols == idx + 3 && s->st_nlink > 1))) /* Maybe a hardlink? (in extfs) */
+        && idx2){
+	int p;
+	char *s;
+	    
+	if (filename) {
+	    p = column_ptr [idx2] - column_ptr [idx];
+	    s = xmalloc (p, "filename");
+	    strncpy (s, p_copy + column_ptr [idx], p - 1);
+	    s[p - 1] = '\0';
+	    *filename = s;
+	}
+	if (linkname) {
+	    s = strdup (p_copy + column_ptr [idx2+1]);
+	    p = strlen (s);
+	    if (s [p-1] == '\r' || s [p-1] == '\n')
+		s [p-1] = 0;
+	    if (s [p-2] == '\r' || s [p-2] == '\n')
+		s [p-2] = 0;
+		
+	    *linkname = s;
+	}
     } else {
 	/* Extract the filename from the string copy, not from the columns
 	 * this way we have a chance of entering hidden directories like ". ."
